@@ -12,7 +12,7 @@ SPDX-License-Identifier: MIT
 #include <stdint.h>
 #undef HAS_INCLUDE_TYPE_TRAITS
 #ifdef _MSC_VER
-#if !defined(__clang__) || (__clang_major__ > 16)
+#if !defined(__clang__) || (__clang_major__ > 17)
 #define HAS_INCLUDE_TYPE_TRAITS
 #endif
 #endif // _MSC_VER
@@ -25,7 +25,7 @@ SPDX-License-Identifier: MIT
 // We need to disable the warnings in such way because this C++ file may be compiled with Clang, GCC, and Microsoft
 // Visual Studio Compiler This works around (at least) two known problems:
 //  - QuickBuild treats compiler warnings as errors
-//  - RTStackReflection.exe generates warnings when compiling this file
+//  - IRBuilderGenerator.exe generates warnings when compiling this file
 //
 // The only macros that are used by the main code are:
 //     DISABLE_WARNING_PUSH
@@ -115,6 +115,12 @@ public:
   uint64_t GetStatelessScratchPtr() const { return 0; };
   uint32_t GetBaseSSHOffset() const { return 0; };
   uint32_t GetUberTilesMap() const { return 0; };
+  uint64_t GetSamplerDescriptorHeap() const { return 0; };
+  uint64_t GetScratchSpaceBufferPointer() const { return 0; };
+  uint64_t GetResourceDescriptorHeap() const { return 0; };
+  uint64_t GetBaseSurfaceStatePointer() const { return 0; };
+  uint32_t GetResourceDescriptorHeapOffset() const { return 0; };
+  uint32_t GetSamplerDescriptorHeapOffset() const { return 0; };
 };
 
 // Layout used to pass global data to the shaders
@@ -133,15 +139,18 @@ struct RayDispatchGlobalData {
     uint64_t ProfilingBufferGpuVa; // GTPin buffer to collect profiling data
     union {
       uint64_t padding;
+      uint64_t SamplerDescriptorHeap; // base pointer for sampler heap
     };
     union {
-      uint64_t statelessScratchPtr; // Stateless scratch buffer pointer
+      uint64_t statelessScratchPtr;       // Stateless scratch buffer pointer
+      uint64_t ScratchSpaceBufferPointer; // surface state pointer for scratch space
     };
     uint64_t pCallableShaderBasePtr; // base pointer of callable shader record array (8-bytes alignment)
     uint32_t pCallableShaderStride;  // stride of callable shader records (8-bytes alignment)
     uint32_t pNumDSSRTStacks;        // number of stacks per DSS
     union {
-      uint64_t bindlessHeapBasePtr; // base pointer of bindless heap
+      uint64_t bindlessHeapBasePtr;    // base pointer of bindless heap
+      uint64_t ResourceDescriptorHeap; // base pointer for resource heap
     };
     uint64_t pHitGroupBasePtr;   // base pointer of hit group shader record array (16-bytes alignment)
     uint64_t pMissShaderBasePtr; // base pointer of miss shader record array (8-bytes alignment)
@@ -149,16 +158,27 @@ struct RayDispatchGlobalData {
     uint32_t pMissShaderStride;  // stride of miss shader records (8-bytes alignment)
     uint64_t pRtMemBasePtr;      // base address of the allocated stack memory
 
+    uint32_t SamplerDescriptorHeapOffset; // Offset in surface states to the start of the sampler descriptor heap
+
     union {
       struct {
         uint32_t baseSSHOffset; // (index/bindless offset) of first memory region for stateful indirect accesses
         uint32_t paddingBits3;  // 32-bits of padding
       };
 
+      // For Efficient64 there is no baseSSH, the full address of the
+      // resource must calculated in the shader. Instead of delivering
+      // separately the offset to the first Stack SurfaceState, and
+      // the pointer to the baseSurfaceState (in Constants), UMD
+      // will deliver baseSurfaceStatePointer + OffsetToTheFirstStackSS
+      // in baseSurfaceStatePointer.
+      struct {
+        uint64_t baseSurfaceStatePointer;
+      };
     };
 
-    uint64_t uberTilesMap; // base address of the uber tiles map used for AtomicPull model
-    uint32_t paddingBits4; // 1 dword of padding to make sure the struct size is multiple of 8 on all platforms
+    uint64_t uberTilesMap;                 // base address of the uber tiles map used for AtomicPull model
+    uint32_t ResourceDescriptorHeapOffset; // Offset in surface states to the start of the resource descriptor heap
 
     uint32_t pStackSizePerRay;          // maximal stack size of a ray
     uint32_t swStackSizePerRay;         // size in bytes per ray of the SWStack
@@ -168,15 +188,25 @@ struct RayDispatchGlobalData {
       printfBufferBasePtr = umd.GetPrintfBufferBasePtr();
       ProfilingBufferGpuVa = umd.GetProfilingBufferGpuVa();
       statelessScratchPtr = umd.GetStatelessScratchPtr();
+      SamplerDescriptorHeap = umd.GetSamplerDescriptorHeap();
+      if (uint64_t Tmp = umd.GetScratchSpaceBufferPointer())
+        ScratchSpaceBufferPointer = Tmp;
       pCallableShaderBasePtr = umd.GetCallableShaderTable();
       pCallableShaderStride = umd.GetCallableStride();
       bindlessHeapBasePtr = umd.GetBindlessHeapBasePtr();
+      if (uint64_t Tmp = umd.GetResourceDescriptorHeap())
+        ResourceDescriptorHeap = Tmp;
       pHitGroupBasePtr = umd.GetHitGroupTable();
       pMissShaderBasePtr = umd.GetMissShaderTable();
       pHitGroupStride = umd.GetHitGroupStride();
       pMissShaderStride = umd.GetMissStride();
       pRtMemBasePtr = umd.GetRayStackBufferAddress();
       baseSSHOffset = umd.GetBaseSSHOffset();
+      // Check if baseSurfaceStatePointer is delivered from the UMD, as
+      // it shares the same memory with baseSSHOffset in the RayDispatchGlobalDataCommon.
+      // baseSurfaceStatePointer will be written only if it is not zero.
+      if (uint64_t Tmp = umd.GetBaseSurfaceStatePointer())
+        baseSurfaceStatePointer = Tmp;
       pStackSizePerRay = umd.GetStackSizePerRay();
       swStackSizePerRay = umd.GetSWStackSizePerRay();
       pNumDSSRTStacks = umd.GetNumDSSRTStacks();
@@ -184,6 +214,8 @@ struct RayDispatchGlobalData {
       dispatchRaysDimensions[1] = umd.GetDispatchHeight();
       dispatchRaysDimensions[2] = umd.GetDispatchDepth();
       uberTilesMap = umd.GetUberTilesMap();
+      ResourceDescriptorHeapOffset = umd.GetResourceDescriptorHeapOffset();
+      SamplerDescriptorHeapOffset = umd.GetSamplerDescriptorHeapOffset();
 
     }
   };
@@ -283,12 +315,15 @@ struct RayDispatchGlobalData {
       uint64_t hitGroupBasePtr;   // base pointer of hit group shader record array (16-bytes alignment)
       uint64_t missShaderBasePtr; // base pointer of miss shader record array (8-bytes alignment)
 
-      uint32_t _align_mbz[4];     // pad hardware section to 64 bytes
+      uint32_t _align_mbz[4]; // pad hardware section to 64 bytes
 
       // HW doesn't read anything below this point.
       RayDispatchGlobalDataCommon common;
     } xe3;
   } rt;
+  static constexpr uint32_t ScratchSpaceBufferPointerOffset =
+      offsetof(RayDispatchGlobalData::RT::Xe3, common) +
+      offsetof(RayDispatchGlobalData::RayDispatchGlobalDataCommon, ScratchSpaceBufferPointer);
 };
 
 // The actual pointer will probably be aligned to a greater value than this,
@@ -299,6 +334,11 @@ constexpr uint32_t RTGlobalsAlign = 256;
 // chunks of stack so this must be, at minimum, 64-byte aligned.
 constexpr uint32_t RTStackAlign = 128;
 static_assert(RTStackAlign % RayDispatchGlobalData::StackChunkSize == 0, "no?");
+// This is to ensure the alignments for the async and sync stack
+// are 512 on XE3P with Eff64b, i.e., when new stack layout is used.
+// See RTStack doc.
+constexpr uint32_t RTStackXe3PEff64Align = 512;
+static_assert(RTStackXe3PEff64Align % RTStackAlign == 0, "no?");
 
 static_assert((sizeof(RayDispatchGlobalData::RT::Xe) - sizeof(RayDispatchGlobalData::RayDispatchGlobalDataCommon)) %
                       64 ==
@@ -309,7 +349,7 @@ static_assert((sizeof(RayDispatchGlobalData::RT::Xe3) - sizeof(RayDispatchGlobal
                   0,
               "Unexpected GlobalData alignment");
 
-static_assert(sizeof(RayDispatchGlobalData) == 192, "unexpected size?");
+static_assert(sizeof(RayDispatchGlobalData) == 200, "unexpected size?");
 
 static_assert(sizeof(RayDispatchGlobalData::RT::Xe) == sizeof(RayDispatchGlobalData), "unexpected size?");
 static_assert(sizeof(RayDispatchGlobalData::RT::Xe3) == sizeof(RayDispatchGlobalData), "unexpected size?");
@@ -328,7 +368,14 @@ struct RayDispatchInlinedData {
 
   static constexpr unsigned NumElts = 3;
 };
+struct RayDispatchInlinedData_Eff64 {
+  uint64_t IndirectDataPointer;          // Indirect Data Pointer*, r2.0:uq
+  uint64_t RayDispatchDescriptorAddress; // ShaderRecord*, r2.1:uq
+  uint64_t RayDispatchGlobalDataPtr;     //                r2.2:uq
+  uint64_t AtomicPullGlobalQueuePtr;     //                r2.3:uq
 
+  static constexpr unsigned NumElts = 4;
+};
 
 template <typename Type> constexpr Type Align(const Type value, const size_t alignment) {
   // alignment must be power of 2.
