@@ -23,6 +23,9 @@ SPDX-License-Identifier: MIT
 #include <llvm/Analysis/TargetLibraryInfo.h>
 
 #include <llvmWrapper/Transforms/InstCombine/InstCombineWorklist.h>
+#include <llvmWrapper/Transforms/Scalar/SCCP.h>
+#include "llvmWrapper/Transforms/IPO/GlobalDCE.h"
+#include "llvmWrapper/Transforms/IPO/InlineSimple.h"
 #include <llvm/Transforms/Utils.h>
 
 #include "common/LLVMWarningsPop.hpp"
@@ -98,6 +101,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/SPIRMetaDataTranslation.h"
 #include "Compiler/Optimizer/OpenCLPasses/ErrorCheckPass/ErrorCheckPass.h"
 #include "Compiler/Optimizer/OpenCLPasses/PoisonFP64KernelsPass/PoisonFP64KernelsPass.h"
+#include "Compiler/Optimizer/OpenCLPasses/BfloatBuiltins/BfloatBuiltinsResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/BfloatFuncs/BfloatFuncsResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/DpasFuncs/DpasFuncsResolution.hpp"
 #include "Compiler/Optimizer/OpenCLPasses/LSCFuncs/LSCFuncsResolution.hpp"
@@ -115,7 +119,7 @@ SPDX-License-Identifier: MIT
 #include "MoveStaticAllocas.h"
 #include "preprocess_spvir/PreprocessSPVIR.h"
 #include "preprocess_spvir/ConvertUserSemanticDecoratorOnFunctions.h"
-#include "preprocess_spvir/PromoteBools.h"
+#include "preprocess_spvir/PromoteSubByte.h"
 #include "preprocess_spvir/HandleSPIRVDecorations/HandleSpirvDecorationMetadata.h"
 #include "LowerInvokeSIMD.hpp"
 #include "ResolveConstExprCalls.h"
@@ -136,6 +140,7 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/TimeStatsCounter.h"
 #include "Compiler/DebugInfo/ScalarVISAModule.h"
 #include "Compiler/Builtins/BIFFlagCtrl/BIFFlagCtrlResolution.hpp"
+#include "Compiler/Optimizer/OpenCLPasses/SpvPredicatedIOResolution/SpvPredicatedIOResolution.hpp"
 
 #include <string>
 
@@ -276,7 +281,7 @@ static void CommonOCLBasedPasses(OpenCLProgramContext *pContext) {
 
   IGCPassManager mpmSPIR(pContext, "Unify");
   mpmSPIR.add(new PreprocessSPVIR());
-  mpmSPIR.add(new PromoteBools());
+  mpmSPIR.add(new PromoteSubByte());
   mpmSPIR.add(new TypesLegalizationPass());
   mpmSPIR.add(new TargetLibraryInfoWrapperPass());
   mpmSPIR.add(new MetaDataUtilsWrapper(pMdUtils, pContext->getModuleMetaData()));
@@ -332,6 +337,7 @@ static void CommonOCLBasedPasses(OpenCLProgramContext *pContext) {
   }
 
   mpm.add(new JointMatrixFuncsResolutionPass());
+  mpm.add(new BfloatBuiltinsResolution());
 
   mpm.add(new ReduceOptPass());
 
@@ -361,6 +367,7 @@ static void CommonOCLBasedPasses(OpenCLProgramContext *pContext) {
   mpm.add(new Spv2dBlockIOResolution());
 
   mpm.add(new SpvSubgroupMMAResolution());
+  mpm.add(new SpvPredicatedIOResolution());
 
   mpm.add(createProcessBICodeAssumptionPass());
   mpm.add(new PreBIImportAnalysis());
@@ -376,7 +383,7 @@ static void CommonOCLBasedPasses(OpenCLProgramContext *pContext) {
   // OCL has built-ins so it always need to run inlining
   {
     // We need to propagate constexpr casts to resolve pseudo indirect calls
-    mpm.add(createSCCPPass());
+    mpm.add(IGCLLVM::createLegacyWrappedSCCPPass());
     mpm.add(new ResolveConstExprCalls());
 
     // Estimate maximal function size in the module and disable subroutine
@@ -400,12 +407,12 @@ static void CommonOCLBasedPasses(OpenCLProgramContext *pContext) {
 
     if (!IGC::ForceAlwaysInline(pContext)) {
       int Threshold = IGC_GET_FLAG_VALUE(OCLInlineThreshold);
-      mpm.add(createFunctionInliningPass(Threshold));
+      mpm.add(IGCLLVM::createLegacyWrappedSimpleInlinerPass(Threshold));
     } else {
       mpm.add(createAlwaysInlinerLegacyPass());
     }
     // The inliner sometimes fails to delete unused functions, this cleans up the remaining mess.
-    mpm.add(createGlobalDCEPass());
+    mpm.add(IGCLLVM::createLegacyWrappedGlobalDCEPass());
 
     // Check after GlobalDCE in case of doubles in dead functions
     mpm.add(new ErrorCheck());
