@@ -138,6 +138,7 @@ static BasicBlock *findLowestSinkTarget(Instruction *inst, SmallPtrSetImpl<Instr
         break;
     }
   }
+
   BasicBlock *curBlk = inst->getParent();
   Loop *curLoop = LI->getLoopFor(inst->getParent());
   while (tgtBlk && tgtBlk != curBlk) {
@@ -151,7 +152,7 @@ static BasicBlock *findLowestSinkTarget(Instruction *inst, SmallPtrSetImpl<Instr
         // Determine the block of the use.
         Instruction *useInst = cast<Instruction>(*I);
         BasicBlock *useBlock = useInst->getParent();
-        if (useBlock == tgtBlk) {
+        if (useBlock == tgtBlk && !isa<PHINode>(useInst)) {
           usesInBlk.insert(useInst);
         }
       }
@@ -206,6 +207,7 @@ static bool isDPAS(Value *V) {
   switch (Intr->getIntrinsicID()) {
   case GenISAIntrinsic::GenISA_dpas:
   case GenISAIntrinsic::GenISA_sub_group_dpas:
+  case GenISAIntrinsic::GenISA_sub_group_bdpas:
     return true;
   default:
     break;
@@ -760,7 +762,7 @@ bool CodeLoopSinking::runOnFunction(Function &F) {
   // and we could not apply sinking in function B, but we don't recompute FPRE
   // to save compile time, so in this case LoopSinking might apply for loops in function B
 
-  RPE = &getAnalysis<IGCLivenessAnalysis>();
+  RPE = &getAnalysis<IGCLivenessAnalysis>().getLivenessRunner();
   FRPE = &getAnalysis<IGCFunctionExternalRegPressureAnalysis>();
   WI = &FRPE->getWIAnalysis(&F);
 
@@ -784,7 +786,7 @@ void CodeLoopSinking::dumpToFile(const std::string &Log) {
   auto Name = Debug::DumpName(IGC::Debug::GetShaderOutputName())
                   .Hash(CTX->hash)
                   .Type(CTX->type)
-                  .Retry(CTX->m_retryManager.GetRetryId())
+                  .Retry(CTX->m_retryManager->GetRetryId())
                   .Pass("loopsink")
                   .Extension("txt");
   IGC::Debug::DumpLock();
@@ -1134,7 +1136,8 @@ bool CodeLoopSinking::loopSink(Loop *L, LoopSinkMode Mode) {
     return Changed;
   };
 
-  bool ReschedulingIteration = !CTX->platform.isCoreChildOf(IGFX_XE3_CORE) && IGC_IS_FLAG_ENABLED(LoopSinkEnableLoadsRescheduling);
+  bool ReschedulingIteration =
+      !CTX->platform.isCoreChildOf(IGFX_XE3_CORE) && IGC_IS_FLAG_ENABLED(LoopSinkEnableLoadsRescheduling);
   bool LateReschedulingIteration = false;
 
   auto createSimpleCandidates = [&](InstSet &SkipInstructions, CandidateVec &SinkCandidates) {
@@ -1219,7 +1222,6 @@ bool CodeLoopSinking::loopSink(Loop *L, LoopSinkMode Mode) {
   // It's assumed that using std::shared_ptr we will successfully ensure only needed Candidates
   // will remain.
 
-
   InstSet SkipInstructions;
 
   int SinkIterations = 0;
@@ -2193,6 +2195,7 @@ bool CodeLoopSinking::isSafeToLoopSinkLoad(Instruction *InstToSink, Loop *L) {
         case GenISAIntrinsic::GenISA_LSC2DBlockCreateAddrPayload:
         case GenISAIntrinsic::GenISA_dpas:
         case GenISAIntrinsic::GenISA_sub_group_dpas:
+        case GenISAIntrinsic::GenISA_sub_group_bdpas:
           PrintDump(VerbosityLevel::High, "Not a real store instruction, may not alias\n");
           continue;
 
