@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2017-2023 Intel Corporation
+Copyright (C) 2017-2026 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -71,6 +71,7 @@ void ConstantCoalescing::ProcessFunction(Function *function) {
   irBuilder = new IRBuilderWrapper(function->getContext(), m_TT);
   dataLayout = &function->getParent()->getDataLayout();
   wiAns = &getAnalysis<WIAnalysis>();
+  m_BaseOffsets.clear();
 
   // clean up unnecessary lcssa-phi
   for (Function::iterator I = function->begin(), E = function->end(); I != E; ++I) {
@@ -692,6 +693,7 @@ void ConstantCoalescing::CombineTwoLoads(BufChunk *cov_chunk, Instruction *load,
     IGC_ASSERT(nullptr != addr_ptr);
     IGC_ASSERT(nullptr != (cast<PointerType>(addr_ptr->getType())));
     unsigned addrSpace = (cast<PointerType>(addr_ptr->getType()))->getAddressSpace();
+
     if (addrSpace == ADDRESS_SPACE_CONSTANT) {
       // OCL path
       IGC_ASSERT(isa<IntToPtrInst>(addr_ptr) || isa<BitCastInst>(addr_ptr) ||
@@ -708,13 +710,11 @@ void ConstantCoalescing::CombineTwoLoads(BufChunk *cov_chunk, Instruction *load,
       m_TT->RegisterNewValueAndAssignID(ptrcast);
       wiAns->incUpdateDepend(ptrcast, WIAnalysis::RANDOM);
       cov_chunk->chunkIO = irBuilder->CreateLoad(vty, ptrcast, false);
-      cast<LoadInst>(cov_chunk->chunkIO)->setAlignment(getAlign(4)); // \todo, more precise
       wiAns->incUpdateDepend(cov_chunk->chunkIO, WIAnalysis::RANDOM);
     } else {
       IGC_ASSERT(isa<IntToPtrInst>(addr_ptr));
       addr_ptr->mutateType(PointerType::get(vty, addrSpace));
       cov_chunk->chunkIO = irBuilder->CreateLoad(cov_chunk->chunkIO->getType(), addr_ptr, false);
-      cast<LoadInst>(cov_chunk->chunkIO)->setAlignment(getAlign(4)); // \todo, more precise
       wiAns->incUpdateDepend(cov_chunk->chunkIO, WIAnalysis::RANDOM);
       // modify the address calculation if the chunk-start is changed
       if (eltid0 != cov_chunk->chunkStart) {
@@ -761,13 +761,17 @@ void ConstantCoalescing::CombineTwoLoads(BufChunk *cov_chunk, Instruction *load,
       }
     }
     IGC_ASSERT(!ldRaw0->isVolatile() && !ldRaw1->isVolatile());
-    Value *args[] = {ldRaw0->getResourceValue(), offsetInBuffer, ldRaw0->getAlignmentValue(), irBuilder->getFalse()};
+    Value *args[] = {ldRaw0->getResourceValue(), offsetInBuffer, irBuilder->getInt32(m_ChunkMinAlignment),
+                     irBuilder->getFalse()};
     cov_chunk->chunkIO = irBuilder->CreateCall(ldRawFn, args, ldRaw0->getName());
     IGC_ASSERT(CompareMetadata(ldRaw0, ldRaw1));
     CopyMetadata(cov_chunk->chunkIO, ldRaw0);
     wiAns->incUpdateDepend(cov_chunk->chunkIO, wiAns->whichDepend(ldRaw0));
   }
 
+  // Update the alignment of the new load instruction created above
+  SetAlignmentFromOffset(cov_chunk->chunkIO);
+
   // add two splitters
   Instruction *splitter;
   splitter = AddChunkExtract(cov_chunk->chunkIO, eltid0 - cov_chunk->chunkStart);
@@ -807,6 +811,28 @@ void ConstantCoalescing::SetAlignment(Instruction *load, uint alignment) {
   }
 }
 
+void ConstantCoalescing::SetAlignmentFromOffset(Instruction *load) {
+  IGC_ASSERT(isa<LdRawIntrinsic>(load) || isa<LoadInst>(load));
+  Value *offset = nullptr;
+  if (auto loadInst = dyn_cast<LoadInst>(load)) {
+    Value *ptr = loadInst->getOperand(0);
+    if (!isa<IntToPtrInst>(ptr)) {
+      IGC_ASSERT_MESSAGE(false, "Unexpected pointer operand for LoadInst");
+      return;
+    }
+    offset = cast<IntToPtrInst>(ptr)->getOperand(0);
+  } else {
+    offset = cast<LdRawIntrinsic>(load)->getOffsetValue();
+  }
+  IGC_ASSERT(offset != nullptr);
+  const DominatorTree &DT = getAnalysis<DominatorTreeWrapperPass>().getDomTree();
+  KnownBits kb = computeKnownBits(offset, *dataLayout, 0 /*current depth*/, nullptr /*AssumptionCache*/, load, &DT);
+  uint32_t numTrailZeros = std::min(kb.countMinTrailingZeros(), Value::MaxAlignmentExponent);
+  alignment_t alignment = (1ull << std::min(kb.getBitWidth() - 1, numTrailZeros));
+  alignment = std::max<alignment_t>(alignment, m_ChunkMinAlignment);
+  SetAlignment(load, alignment);
+}
+
 void ConstantCoalescing::MergeUniformLoad(Instruction *load, Value *bufIdxV, uint addrSpace, Value *eltIdxV,
                                           uint offsetInBytes, uint maxEltPlus, const ExtensionKind &Extension,
                                           std::vector<BufChunk *> &chunk_vec) {
@@ -1110,6 +1136,51 @@ Value *ConstantCoalescing::SimpleBaseOffset(Value *elt_idxv, uint &offset, Exten
       offset = offset1 + static_cast<uint>(csrc1->getZExtValue());
       return base;
     }
+
+    // clang-format off
+    // Match complex patterns where the `add` with a constant offset
+    // is not the last operation in the chain, e.g.:
+    //  %a = call i32 foo()
+    //  %b = call i32 foo()
+    //  %ox = add i32 %a, %b
+    //  %x = call i32 @llvm.genx.GenISA.ldraw.indexed.i32.p3276800(ptr addrspace(3276800) %bso, i32 %ox, i32 4, i1 false)
+    //  %c = add i32 %b, 4
+    //  %oy = add i32 %a, %c
+    //  %y = call i32 @llvm.genx.GenISA.ldraw.indexed.i32.p3276800(ptr addrspace(3276800) %bso, i32 %oy, i32 4, i1 false)
+    // clang-format on
+    Value *src1 = expr->getOperand(1);
+    // Helper lambda to create a sorted pair of Values
+    auto MakeSortedPair = [](Value *a, Value *b) { return a < b ? std::make_pair(a, b) : std::make_pair(b, a); };
+    // Check if this is a previously seen pair
+    auto it = m_BaseOffsets.find(MakeSortedPair(src0, src1));
+    if (it != m_BaseOffsets.end()) {
+      return it->second;
+    }
+    // Check both operands for constant offsets
+    uint offset0 = 0;
+    uint offset1 = 0;
+    ExtensionKind extension1 = EK_NotExtended;
+    ExtensionKind extension2 = EK_NotExtended;
+    Value *base0 = SimpleBaseOffset(src0, offset0, extension1);
+    Value *base1 = SimpleBaseOffset(src1, offset1, extension2);
+    // Bail if no constant offset was found or extension kinds don't match.
+    if ((offset0 == 0 && offset1 == 0) || extension1 != extension2) {
+      // Check for previously created add instruction with these operands.
+      auto [it, inserted] = m_BaseOffsets.try_emplace(MakeSortedPair(src0, src1), expr);
+      return it->second;
+    }
+    IGC_ASSERT(base0 != src0 || base1 != src1);
+    Extension = extension1;
+    offset = offset0 + offset1;
+    // Find or create an `add` instruction with both bases.
+    auto it1 = m_BaseOffsets.find(MakeSortedPair(base0, base1));
+    if (it1 != m_BaseOffsets.end()) {
+      return it1->second;
+    }
+    Instruction *newBase = BinaryOperator::Create(Instruction::Add, base0, base1, "", expr);
+    wiAns->incUpdateDepend(newBase, std::max(wiAns->whichDepend(base0), wiAns->whichDepend(base1)));
+    m_BaseOffsets.insert({MakeSortedPair(base0, base1), newBase});
+    return newBase;
   } else if (expr->getOpcode() == Instruction::Or) {
     Value *src0 = expr->getOperand(0);
     Value *src1 = expr->getOperand(1);
@@ -1421,9 +1492,10 @@ void ConstantCoalescing::AdjustChunk(BufChunk *cov_chunk, uint start_adj, uint s
     if (addrSpace == ADDRESS_SPACE_CONSTANT || addrSpace == ADDRESS_SPACE_GLOBAL) {
       // ocl path
       IGC_ASSERT(isa<IntToPtrInst>(addr_ptr));
-      irBuilder->SetInsertPoint(dyn_cast<Instruction>(addr_ptr));
+      Instruction *intToPtr = cast<Instruction>(addr_ptr);
+      irBuilder->SetInsertPoint(intToPtr);
       addr_ptr->mutateType(PointerType::get(vty, addrSpace));
-      Value *eac = cast<Instruction>(addr_ptr)->getOperand(0);
+      Value *eac = intToPtr->getOperand(0);
       Instruction *expr = dyn_cast<Instruction>(eac);
       bool foundOffset = false;
       if (expr && expr->getOpcode() == Instruction::Add && expr->hasOneUse()) {
@@ -1460,7 +1532,7 @@ void ConstantCoalescing::AdjustChunk(BufChunk *cov_chunk, uint start_adj, uint s
       if (!foundOffset) {
         // if we cannot modify the offset, create a new chain of address calculation
         eac = FormChunkAddress(cov_chunk, Extension);
-        cast<Instruction>(addr_ptr)->setOperand(0, eac);
+        intToPtr->setOperand(0, eac);
       }
     } else {
       // gfx path
@@ -1515,6 +1587,11 @@ void ConstantCoalescing::AdjustChunk(BufChunk *cov_chunk, uint start_adj, uint s
     }
   }
 
+  // Update the alignment if the starting offset changed.
+  if (start_adj != 0) {
+    SetAlignmentFromOffset(cov_chunk->chunkIO);
+  }
+
   SmallVector<Instruction *, 4> use_set;
   // adjust all the splitters
   Value::user_iterator use_it = cov_chunk->chunkIO->user_begin();
