@@ -84,7 +84,6 @@ public:
 
   void CreateKernelShaderMap(CodeGenContext *ctx, IGC::IGCMD::MetaDataUtils *pMdUtils, llvm::Function &F);
 
-
   bool isVectorEmissionPossible(const SSource sources[2], CVariable *src[2]);
 
   void Frc(const SSource &source, const DstModifier &modifier);
@@ -94,7 +93,7 @@ public:
   void Cmp(llvm::CmpInst::Predicate pred, const SSource sources[2], const DstModifier &modifier,
            uint8_t clearTagMask = 0);
   void VectorCMP(llvm::CmpInst::Predicate pred, const SSource sources[2], const DstModifier &modifier,
-           uint8_t clearTagMask = 0);
+                 uint8_t clearTagMask = 0);
 
   void Sub(const SSource[2], const DstModifier &mofidier);
   void Xor(const SSource[2], const DstModifier &modifier);
@@ -123,7 +122,15 @@ public:
   void CmpBfn(llvm::CmpInst::Predicate predicate, const SSource cmpSources[2], uint8_t booleanFuncCtrl,
               const SSource bfnSources[3], const DstModifier &modifier);
 
-  void Mul64(CVariable *dst, CVariable *src[2], SIMDMode simdMode, bool noMask = false) const;
+  enum class ResultScope { Low = 0b01, Hi = 0b10, All = 0b11 };
+  bool canUseMul64SOA() const;
+  void emitMul64SOA(CVariable *DstL, CVariable *DstH, CVariable *L0, CVariable *L1, CVariable *H0, CVariable *H1,
+                    std::optional<SIMDMode> Mode = std::nullopt, bool NoMask = false,
+                    const SSource Sources[4] = nullptr) const;
+  void emitShflIdx4(Instruction *inst, bool packed);
+  void emitShflIdx4Lut(Instruction *inst);
+  void Mul64(CVariable *dst, CVariable *src[2], SIMDMode simdMode, bool noMask = false,
+             ResultScope mulResultScope = ResultScope::Low, CVariable *outDstHi = nullptr) const;
 
   template <int N> void Alu(e_opcode opCode, const SSource sources[N], const DstModifier &modifier);
 
@@ -182,6 +189,8 @@ public:
   void emitVectorCopyToOrFromAOS(uint32_t AOSBytes, CVariable *Dst, CVariable *Src, uint32_t nElts,
                                  uint32_t DstSubRegOffset, uint32_t SrcSubRegOffset, bool IsToAOS);
   void emitCopyToOrFromLayoutStruct(llvm::Value *D, llvm::Value *S);
+  void emitLayoutStructCopyAOSToAOS(uint32_t AOSBytes, CVariable *Dst, CVariable *Src, uint32_t nElts,
+                                    uint32_t DstSubRegOffset = 0, uint32_t SrcSubRegOffset = 0);
 
   /// stack-call code-gen functions
   void emitStackCall(llvm::CallInst *inst);
@@ -204,14 +213,14 @@ public:
 
   // TODO: unify the functions below and clean up
   void emitStore(llvm::StoreInst *inst, llvm::Value *varOffset, llvm::ConstantInt *immOffset,
-                 ConstantInt *immScale = nullptr
-  );
+                 ConstantInt *immScale = nullptr, llvm::Value *uniformBase = nullptr, bool signExtendOffset = false,
+                 bool zeroExtendOffset = false);
   void emitPredicatedStore(llvm::Instruction *inst);
   void emitStore3DInner(llvm::Value *pllValToStore, llvm::Value *pllDstPtr, llvm::Value *pllElmIdx);
 
   void emitLoad(llvm::LoadInst *inst, llvm::Value *varOffset, llvm::ConstantInt *immOffset,
-                ConstantInt *immScale = nullptr
-  );   // single load, no pattern
+                ConstantInt *immScale = nullptr, llvm::Value *uniformBase = nullptr, bool signExtendOffset = false,
+                bool zeroExtendOffset = false); // single load, no pattern
   void emitPredicatedLoad(llvm::Instruction *inst);
   void emitLoad3DInner(llvm::LdRawIntrinsic *inst, ResourceDescriptor &resource, llvm::Value *elemIdxV);
 
@@ -291,10 +300,10 @@ public:
   void emitUAVSerialize();
 
   void emitScalarAtomics(llvm::Instruction *pInst, ResourceDescriptor &resource, AtomicOp atomic_op,
-                         CVariable *pDstAddr, CVariable *pU, CVariable *pV, CVariable *pR, CVariable *pSrc, bool isA64,
-                         int bitSize, int immOffset, int immScale, LSC_ADDR_SIZE addrSize);
+                         CVariable *pUniformBaseAddr, CVariable *pDstAddr, CVariable *pU, CVariable *pV, CVariable *pR,
+                         CVariable *pSrc, bool isA64, int bitSize, int immOffset, int immScale, LSC_ADDR_SIZE addrSize);
 
-  void emitScalarAtomicLoad(llvm::Instruction *pInst, ResourceDescriptor &resource,
+  void emitScalarAtomicLoad(llvm::Instruction *pInst, ResourceDescriptor &resource, CVariable *pUniformBase,
                             CVariable *pDstAddr, CVariable *pU, CVariable *pV, CVariable *pR, CVariable *pSrc,
                             bool isA64, int bitWidth, int immOffset, int immScale, LSC_ADDR_SIZE addrSize);
 
@@ -338,8 +347,8 @@ public:
 
   bool IsUniformAtomic(llvm::Instruction *pInst);
   void emitAtomicRaw(llvm::GenIntrinsicInst *pInst, Value *varOffset, ConstantInt *immOffset = nullptr,
-                     ConstantInt *immScale = nullptr
-  );
+                     ConstantInt *immScale = nullptr, Value *uniformBase = nullptr, bool signExtendOffset = false,
+                     bool zeroExtendOffset = false);
   void emitAtomicTyped(llvm::GenIntrinsicInst *pInst);
   void emitAtomicCounter(llvm::GenIntrinsicInst *pInst);
   void emitFastClear(llvm::LoadInst *inst);
@@ -433,15 +442,13 @@ public:
   void emitVectorBitCast(llvm::BitCastInst *BCI);
   void emitVectorLoad(llvm::LoadInst *LI, llvm::Value *offset, llvm::ConstantInt *immOffset);
   void emitVectorStore(llvm::StoreInst *SI, llvm::Value *offset, llvm::ConstantInt *immOffset);
-  void emitLSCVectorLoad(llvm::Instruction *Inst, llvm::Value *Ptr,
-                         llvm::Value *offset, llvm::ConstantInt *immOffset, ConstantInt *immScale,
-                         LSC_CACHE_OPTS cacheOpts, LSC_DOC_ADDR_SPACE addrSpace
-  );
-  void emitLSCVectorStore(llvm::Value *Ptr,
-                          llvm::Value *offset, llvm::ConstantInt *immOffset, llvm::ConstantInt *immScale,
-                          llvm::Value *storedVal, llvm::BasicBlock *BB, LSC_CACHE_OPTS cacheOpts, alignment_t align,
-                          bool dontForceDMask, LSC_DOC_ADDR_SPACE addrSpace
-                          ,
+  void emitLSCVectorLoad(llvm::Instruction *Inst, llvm::Value *Ptr, llvm::Value *uniformBase, llvm::Value *offset,
+                         llvm::ConstantInt *immOffset, ConstantInt *immScale, LSC_CACHE_OPTS cacheOpts,
+                         LSC_DOC_ADDR_SPACE addrSpace, bool signExtendOffset, bool zeroExtendOffset);
+  void emitLSCVectorStore(llvm::Value *Ptr, Value *uniformBase, llvm::Value *offset, llvm::ConstantInt *immOffset,
+                          llvm::ConstantInt *immScale, llvm::Value *storedVal, llvm::BasicBlock *BB,
+                          LSC_CACHE_OPTS cacheOpts, alignment_t align, bool dontForceDMask,
+                          LSC_DOC_ADDR_SPACE addrSpace, bool signExtendOffset, bool zeroExtendOffset,
                           llvm::Value *predicate = nullptr);
   void emitUniformVectorCopy(CVariable *Dst, CVariable *Src, uint32_t nElts, uint32_t DstSubRegOffset = 0,
                              uint32_t SrcSubRegOffset = 0, bool allowLargerSIMDSize = false,
@@ -487,6 +494,9 @@ public:
   void emitAsyncStackID(llvm::GenIntrinsicInst *I);
   void emitTraceRay(llvm::TraceRayIntrinsic *I, bool RayQueryEnable);
 
+  void emitTraceRayEff64(TraceRayIntrinsic *instruction, bool rayQueryEnable);
+  void emitRayQueryCheckReleaseEff64(llvm::GenIntrinsicInst *instruction, bool rayQueryCheckEnable = false,
+                                     bool rayQueryReleaseEnable = false);
 
   void emitReadTraceRaySync(llvm::GenIntrinsicInst *I);
 
@@ -503,15 +513,24 @@ public:
   void emitBTD(CVariable *GlobalBufferPtr, CVariable *StackID, CVariable *ShaderRecord, CVariable *Flag,
                bool releaseStackID);
 
+  void emitBTDEff64(CVariable *globalBufferPtr, CVariable *stackID, CVariable *shaderRecord, CVariable *flag,
+                    bool releaseStackID
+  );
 
   void emitBindlessThreadDispatch(llvm::BTDIntrinsic *I);
+
+
   void emitStackIDRelease(llvm::StackIDReleaseIntrinsic *I);
   void emitGetShaderRecordPtr(llvm::GetShaderRecordPtrIntrinsic *I);
   void emitGlobalBufferPtr(llvm::GenIntrinsicInst *I);
   void emitLocalBufferPtr(llvm::GenIntrinsicInst *I);
+  void emitKSPPointer(llvm::KSPPointerIntrinsic *KPI);
   void emitInlinedDataValue(llvm::GenIntrinsicInst *I);
+  void emitBdpas(llvm::GenIntrinsicInst *GII);
   void emitDpas(llvm::GenIntrinsicInst *GII, const SSource *source, const DstModifier &modifier);
   void emitfcvt(llvm::GenIntrinsicInst *GII);
+  void emitLfsr(llvm::GenIntrinsicInst *GII);
+  void emitMaxReduce(llvm::GenIntrinsicInst *GII);
 
   void emitSystemMemoryFence(llvm::GenIntrinsicInst *I);
   void emitUrbFence();
@@ -541,6 +560,7 @@ public:
 
   void emitLscIntrinsicLoad(llvm::GenIntrinsicInst *GII);
   void emitLscIntrinsicPrefetch(llvm::GenIntrinsicInst *GII);
+  void emitLscIntrinsicTypedLoadStatus(llvm::GenIntrinsicInst *GII);
   void emitLscSimdBlockPrefetch(llvm::GenIntrinsicInst *GII);
   void emitLscIntrinsicStore(llvm::GenIntrinsicInst *GII);
   void emitLscIntrinsicLoadCmask(llvm::GenIntrinsicInst *inst);
@@ -554,12 +574,26 @@ public:
 
   void emitLSCAtomic(llvm::GenIntrinsicInst *inst);
   void emitLSCIntrinsic(llvm::GenIntrinsicInst *GII);
+  void emitLSCLoad(llvm::Instruction *inst, CVariable *dst, CVariable *uniformBase, CVariable *offset,
+                   unsigned elemSize, unsigned numElems, unsigned blockOffset, ResourceDescriptor *resource,
+                   LSC_ADDR_SIZE addr_size, LSC_DATA_ORDER data_order, int immOffset, int immScale);
+  void emitLSCLoad(LSC_CACHE_OPTS cacheOpts, CVariable *dst, CVariable *uniformBase, CVariable *offset,
+                   unsigned elemSize, unsigned numElems, unsigned blockOffset, ResourceDescriptor *resource,
+                   LSC_ADDR_SIZE addr_size, LSC_DATA_ORDER data_order, int immOffset, int immScale,
+                   LSC_DOC_ADDR_SPACE addrSpace);
   void emitLSCLoad(llvm::Instruction *inst, CVariable *dst, CVariable *offset, unsigned elemSize, unsigned numElems,
                    unsigned blockOffset, ResourceDescriptor *resource, LSC_ADDR_SIZE addr_size,
                    LSC_DATA_ORDER data_order, int immOffset, int immScale);
   void emitLSCLoad(LSC_CACHE_OPTS cacheOpts, CVariable *dst, CVariable *offset, unsigned elemSize, unsigned numElems,
                    unsigned blockOffset, ResourceDescriptor *resource, LSC_ADDR_SIZE addr_size,
                    LSC_DATA_ORDER data_order, int immOffset, int immScale, LSC_DOC_ADDR_SPACE addrSpace);
+  void emitLSCStore(llvm::Instruction *inst, CVariable *uniformBase, CVariable *src, CVariable *offset,
+                    unsigned elemSize, unsigned numElems, unsigned blockOffset, ResourceDescriptor *resource,
+                    LSC_ADDR_SIZE addr_size, LSC_DATA_ORDER data_order, int immOffset, int immScale);
+  void emitLSCStore(LSC_CACHE_OPTS cacheOpts, CVariable *uniformBase, CVariable *src, CVariable *offset,
+                    unsigned elemSize, unsigned numElems, unsigned blockOffset, ResourceDescriptor *resource,
+                    LSC_ADDR_SIZE addr_size, LSC_DATA_ORDER data_order, int immOffset, int immScale,
+                    LSC_DOC_ADDR_SPACE addrSpace);
   void emitLSCStore(llvm::Instruction *inst, CVariable *src, CVariable *offset, unsigned elemSize, unsigned numElems,
                     unsigned blockOffset, ResourceDescriptor *resource, LSC_ADDR_SIZE addr_size,
                     LSC_DATA_ORDER data_order, int immOffset, int immScale);
@@ -575,6 +609,9 @@ public:
   bool forceCacheCtrl(llvm::Instruction *vectorLdStInst = nullptr);
   uint32_t totalBytesToStoreOrLoad(llvm::Instruction *vectorLdStInst);
   void emitSrnd(llvm::GenIntrinsicInst *GII);
+  void emitDnscl(llvm::GenIntrinsicInst *GII);
+  void emitInt4VectorUnpack(llvm::GenIntrinsicInst *GII);
+  void emitInt4VectorPack(llvm::GenIntrinsicInst *GII);
   void emitStaticConstantPatchValue(llvm::StaticConstantPatchIntrinsic *staticConstantPatch32);
   // Debug Built-Ins
   void emitStateRegID(uint32_t BitStart, uint32_t BitEnd);
@@ -598,6 +635,9 @@ public:
   void emitLoadImplBufferPtr(llvm::GenIntrinsicInst *I);
   void emitLoadLocalIdBufferPtr(llvm::GenIntrinsicInst *I);
   void emitLoadGlobalBufferArg(llvm::GenIntrinsicInst *I);
+  void emitLoadKABPKernelArg(llvm::GenIntrinsicInst *I);
+  void emitLoadgetTileID(llvm::GenIntrinsicInst *I);
+  void emitLoadgetEngineID(llvm::GenIntrinsicInst *I);
 
   void emitMayUnalignedVectorCopy(CVariable *D, uint32_t D_off, CVariable *S, uint32_t S_off, llvm::Type *Ty);
   Function *findStackOverflowDetectionFunction(Function *ParentFunction, bool FindInitFunction);
@@ -648,9 +688,11 @@ public:
   CVariable *BroadcastAndExtend(CVariable *pVar);
   CVariable *TruncatePointer(CVariable *pVar, bool TruncBothHalves = false);
   CVariable *ReAlignUniformVariable(CVariable *pVar, e_alignment align);
+  CVariable *ReAlignOrBroadcastUniformVarIfNotNull(bool destIsUniform, CVariable *pVar, e_alignment align,
+                                                   bool nomask = false);
   CVariable *BroadcastAndTruncPointer(CVariable *pVar);
   CVariable *IndexableResourceIndex(CVariable *indexVar, uint btiIndex);
-  ResourceDescriptor GetResourceVariable(llvm::Value *resourcePtr, bool Check = false);
+  ResourceDescriptor GetResourceVariable(llvm::Value *resourcePtr);
   SamplerDescriptor GetSamplerVariable(llvm::Value *samplerPtr);
   CVariable *ComputeSampleIntOffset(llvm::Instruction *sample, uint sourceIndex);
   void emitPlnInterpolation(CVariable *bary, CVariable *inputvar);
@@ -678,6 +720,7 @@ public:
   void ResourceLoopBackEdge(bool needLoop, CVariable *flag, uint label, uint ResourceLoopMarker = 0);
   bool ResourceLoopNeedsLoop(ResourceDescriptor &resource, SamplerDescriptor &sampler, CVariable *&flag,
                              uint ResourceLoopMarker);
+  bool ResourceLoopNeedsLoop(ResourceDescriptor &resource, CVariable *&flag, uint ResourceLoopMarker);
   template <typename Func>
   void ResourceLoop(ResourceDescriptor &resource, SamplerDescriptor &sampler, const Func &Fn,
                     uint ResourceLoopMarker = 0) {
@@ -948,7 +991,7 @@ private:
 
   CVariable *tryReusingXYZWPayload(llvm::Value *storedVal, llvm::BasicBlock *BB, unsigned numElems, VISA_Type type,
                                    CVariable *pSrc_X, CVariable *pSrc_Y, CVariable *pSrc_Z, CVariable *pSrc_W,
-                                   const unsigned int numEltGRF);
+                                   const unsigned int numEltGRF, bool transposeSIMD1 = false);
 
   // Emit code in slice starting from (reverse) iterator I. Return the
   // iterator to the next pattern to emit.
@@ -1002,22 +1045,24 @@ private:
   CVariable *prepareDataForUniform(CVariable *DataVar, uint32_t ExecSz, e_alignment Align);
   // sub-function of vector load/store
   void emitLSCVectorLoad_subDW(LSC_CACHE_OPTS CacheOpts, bool UseA32, ResourceDescriptor &Resource, CVariable *Dest,
-                               CVariable *Offset, int ImmOffset, int ImmScale, uint32_t NumElts, uint32_t EltBytes,
-                               LSC_DOC_ADDR_SPACE AddrSpace, LSC_ADDR_SIZE AddrSize,
-                               CVariable *inputPredicate = nullptr, CVariable *mergeVal = nullptr);
+                               CVariable *UniformBaseVar, CVariable *Offset, int ImmOffset, int ImmScale,
+                               uint32_t NumElts, uint32_t EltBytes, LSC_DOC_ADDR_SPACE AddrSpace,
+                               LSC_ADDR_SIZE AddrSize, CVariable *inputPredicate = nullptr,
+                               CVariable *mergeVal = nullptr);
   void emitLSCVectorLoad_uniform(LSC_CACHE_OPTS CacheOpts, bool UseA32, ResourceDescriptor &Resource, CVariable *Dest,
-                                 CVariable *Offset, int ImmOffset, int ImmScale, uint32_t NumElts, uint32_t EltBytes,
-                                 uint64_t Align, uint32_t Addrspace, LSC_DOC_ADDR_SPACE UserAddrSpace,
-                                 LSC_ADDR_SIZE AddrSize, CVariable *inputPredicate = nullptr,
-                                 CVariable *mergeVal = nullptr);
+                                 CVariable *UniformBaseVar, CVariable *Offset, int ImmOffset, int ImmScale,
+                                 uint32_t NumElts, uint32_t EltBytes, uint64_t Align, uint32_t Addrspace,
+                                 LSC_DOC_ADDR_SPACE UserAddrSpace, LSC_ADDR_SIZE AddrSize,
+                                 CVariable *inputPredicate = nullptr, CVariable *mergeVal = nullptr);
   void emitLSCVectorStore_subDW(LSC_CACHE_OPTS CacheOpts, bool UseA32, ResourceDescriptor &Resource,
-                                CVariable *StoreVar, CVariable *Offset, int ImmOffset, int ImmScale, uint32_t NumElts,
-                                uint32_t EltBytes, alignment_t Align, LSC_DOC_ADDR_SPACE AddrSpace,
-                                LSC_ADDR_SIZE AddrSize, llvm::Value *predicate = nullptr);
+                                CVariable *UniformBaseVar, CVariable *StoreVar, CVariable *Offset, int ImmOffset,
+                                int ImmScale, uint32_t NumElts, uint32_t EltBytes, alignment_t Align,
+                                LSC_DOC_ADDR_SPACE AddrSpace, LSC_ADDR_SIZE AddrSize, llvm::Value *predicate = nullptr);
   void emitLSCVectorStore_uniform(LSC_CACHE_OPTS CacheOpts, bool UseA32, ResourceDescriptor &Resource,
-                                  CVariable *StoreVar, CVariable *Offset, int ImmOffset, int ImmScale, uint32_t NumElts,
-                                  uint32_t EltBytes, alignment_t Align, LSC_DOC_ADDR_SPACE AddrSpace,
-                                  LSC_ADDR_SIZE AddrSize, llvm::Value *predicate = nullptr);
+                                  CVariable *UniformBaseVar, CVariable *StoreVar, CVariable *Offset, int ImmOffset,
+                                  int ImmScale, uint32_t NumElts, uint32_t EltBytes, alignment_t Align,
+                                  LSC_DOC_ADDR_SPACE AddrSpace, LSC_ADDR_SIZE AddrSize,
+                                  llvm::Value *predicate = nullptr);
   LSC_FENCE_OP getLSCMemoryFenceOp(bool IsGlobalMemFence, bool InvalidateL1, bool EvictL1) const;
 
   CVariable *getStackSizePerThread(llvm::Function *parentFunc);
