@@ -28,6 +28,7 @@ SPDX-License-Identifier: MIT
 #include <llvm/Transforms/Utils/Local.h>
 #include "llvmWrapper/IR/Intrinsics.h"
 #include "llvmWrapper/IR/DerivedTypes.h"
+#include "llvmWrapper/Support/MathExtras.h"
 #include "common/LLVMWarningsPop.hpp"
 #include "GenISAIntrinsics/GenIntrinsics.h"
 #include "common/IGCIRBuilder.h"
@@ -239,15 +240,22 @@ public:
     AU.addRequired<MetaDataUtilsWrapper>();
     AU.addRequired<LoopInfoWrapperPass>();
     AU.addRequired<ScalarEvolutionWrapperPass>();
+    AU.addRequired<DominatorTreeWrapperPass>();
   }
 
+  void releaseMemory() override { truncatedInsts.clear(); }
+
 protected:
   // Helpers
-  Value *getSExtOrTrunc(Value *, Type *) const;
-  Value *truncExpr(Value *, Type *) const;
+  Value *getSExtOrTrunc(Value *, Type *);
+  Value *truncExpr(Value *, Type *, bool);
 
-  bool simplifyGEP(BasicBlock &BB) const;
-  bool lowerGetElementPtrInst(GetElementPtrInst *GEP) const;
+  bool simplifyGEP(BasicBlock &BB);
+  bool lowerGetElementPtrInst(GetElementPtrInst *GEP);
+
+private:
+  DenseMap<Instruction *, Instruction *> truncatedInsts;
+  DominatorTree *DT = nullptr;
 };
 
 char GEPLowering::ID = 0;
@@ -387,7 +395,7 @@ bool GenIRLowering::runOnFunction(Function &F) {
 }
 
 // For each basic block, simplify GEPs based on the analysis result from SCEV.
-bool GEPLowering::simplifyGEP(BasicBlock &BB) const {
+bool GEPLowering::simplifyGEP(BasicBlock &BB) {
   // Pointers with the form base + zext(idx).
   struct PointerExpr {
     GetElementPtrInst *GEP;
@@ -523,14 +531,16 @@ bool GEPLowering::runOnFunction(Function &F) {
 
   DL = &F.getParent()->getDataLayout();
   SE = &getAnalysis<ScalarEvolutionWrapperPass>().getSE();
+  DT = &getAnalysis<DominatorTreeWrapperPass>().getDomTree();
 
   BuilderTy TheBuilder(F.getContext(), TargetFolder(*DL));
   Builder = &TheBuilder;
 
   bool Changed = false;
 
-  if (IGC_IS_FLAG_ENABLED(EnableGEPSimplification))
-  {
+  // For efficient64b, GEP simplification may be not needed or
+  // need different tuning hueristic
+  if (!m_ctx->platform.hasEfficient64bEnabled() && IGC_IS_FLAG_ENABLED(EnableGEPSimplification)) {
     for (auto &BB : F)
       Changed |= simplifyGEP(BB);
 
@@ -557,7 +567,7 @@ bool GEPLowering::runOnFunction(Function &F) {
   return Changed;
 }
 
-Value *GEPLowering::getSExtOrTrunc(Value *Val, Type *NewTy) const {
+Value *GEPLowering::getSExtOrTrunc(Value *Val, Type *NewTy) {
   Type *OldTy = Val->getType();
   unsigned OldWidth;
   unsigned NewWidth;
@@ -577,15 +587,15 @@ Value *GEPLowering::getSExtOrTrunc(Value *Val, Type *NewTy) const {
   }
 
   if (OldWidth > NewWidth) { // Trunc
-    return truncExpr(Val, NewTy);
+    return truncExpr(Val, NewTy, true);
   }
 
   return Val;
 }
 
-Value *GEPLowering::truncExpr(Value *Val, Type *NewTy) const {
+Value *GEPLowering::truncExpr(Value *Val, Type *NewTy, bool initialVal = false) {
   // Truncation on Gen could be as cheap as NOP by creating the proper region.
-  // Instead of truncating the value itself, try to truncate how it's
+  // Instead of truncating the value itself unless it has multiple users, try to truncate how it's
   // calculated.
   if (Constant *C = dyn_cast<Constant>(Val))
     return Builder->CreateIntCast(C, NewTy, false);
@@ -593,7 +603,36 @@ Value *GEPLowering::truncExpr(Value *Val, Type *NewTy) const {
   if (!isa<Instruction>(Val))
     return Builder->CreateTrunc(Val, NewTy);
 
-  Instruction *I = cast<Instruction>(Val);
+  auto dominatesInst = [&](const llvm::Instruction *A, const llvm::Instruction *B) -> bool {
+    if (!A || !B) {
+      return false;
+    }
+    return DT->dominates(A, B);
+  };
+
+  auto getTruncatedInst = [&](llvm::Instruction *I) -> Instruction * {
+    auto truncatedInstIt = truncatedInsts.find(I);
+    if (truncatedInstIt == truncatedInsts.end()) {
+      return nullptr;
+    }
+    if (!dominatesInst(truncatedInstIt->second, I)) {
+      truncatedInstIt->second->moveAfter(I);
+    }
+
+    return truncatedInstIt->second;
+  };
+
+  auto getOrCreateTruncForOp = [&](Value *Op, Type *NewTy) -> Value * {
+    if (isa<Instruction>(Op)) {
+      auto *truncatedIt = getTruncatedInst(cast<Instruction>(Op));
+      if (truncatedIt) {
+        return truncatedIt;
+      }
+    }
+    return truncExpr(Op, NewTy);
+  };
+
+  auto *I = cast<Instruction>(Val);
   unsigned Opc = I->getOpcode();
   switch (Opc) {
   case Instruction::Add:
@@ -602,9 +641,22 @@ Value *GEPLowering::truncExpr(Value *Val, Type *NewTy) const {
   case Instruction::And:
   case Instruction::Or:
   case Instruction::Xor: {
+    // If the value is used in multiple places, we can just re-use it without duplicating calculation since it can not
+    // be removed.
+    if (initialVal && IGC_IS_FLAG_ENABLED(GEPLoweringTruncOptEnabled)) {
+      auto *truncatedIt = getTruncatedInst(I);
+      if (truncatedIt) {
+        return truncatedIt;
+      }
+      if (llvm::any_of(Val->users(), [](const llvm::User *U) { return !llvm::isa<llvm::GetElementPtrInst>(U); })) {
+        auto *newInst = Builder->CreateTrunc(Val, NewTy);
+        truncatedInsts[I] = cast<Instruction>(newInst);
+        return newInst;
+      }
+    }
     BinaryOperator *BO = cast<BinaryOperator>(I);
-    Value *LHS = truncExpr(BO->getOperand(0), NewTy);
-    Value *RHS = truncExpr(BO->getOperand(1), NewTy);
+    Value *LHS = getOrCreateTruncForOp(BO->getOperand(0), NewTy);
+    Value *RHS = getOrCreateTruncForOp(BO->getOperand(1), NewTy);
     return Builder->CreateBinOp(BO->getOpcode(), LHS, RHS);
   }
   case Instruction::Trunc:
@@ -684,7 +736,7 @@ Value *GenIRLowering::rearrangeAdd(Value *val, Loop *loop) const {
   }
 }
 
-bool GEPLowering::lowerGetElementPtrInst(GetElementPtrInst *GEP) const {
+bool GEPLowering::lowerGetElementPtrInst(GetElementPtrInst *GEP) {
   Value *const PtrOp = GEP->getPointerOperand();
   IGC_ASSERT(nullptr != PtrOp);
   PointerType *const PtrTy = dyn_cast<PointerType>(PtrOp->getType());
@@ -1089,8 +1141,8 @@ bool GenIRLowering::combinePack4i8Or2i16(Instruction *inst, uint64_t numBits) co
     // the packed vector.
     KnownBits kb = computeKnownBits(v, DL);
     uint32_t nonZeroBits = ~(static_cast<uint32_t>(kb.Zero.getZExtValue()));
-    uint32_t lsb = findFirstSet(nonZeroBits);
-    uint32_t msb = findLastSet(nonZeroBits);
+    uint32_t lsb = IGCLLVM::findFirstSet(nonZeroBits);
+    uint32_t msb = IGCLLVM::findLastSet(nonZeroBits);
     if (msb != lsb && (msb / numBits) == (lsb / numBits)) {
       uint32_t idx = (prevShlBits / numBits) + (lsb / numBits);
       if (idx < packedVecSize && toPack[idx].first == nullptr) {
@@ -1130,11 +1182,11 @@ bool GenIRLowering::combinePack4i8Or2i16(Instruction *inst, uint64_t numBits) co
     uint64_t rMask = 0;
     // Match patterns that pack the sign and magnitude parts.
     if (match(v, m_Or(m_And(m_Value(lOp), m_ConstantInt(lMask)), m_And(m_Value(rOp), m_ConstantInt(rMask)))) &&
-        (countPopulation(rMask) == 1 || countPopulation(lMask) == 1)) {
-      Value *signOp = countPopulation(rMask) == 1 ? rOp : lOp;
-      Value *magnOp = countPopulation(rMask) == 1 ? lOp : rOp;
-      uint64_t signMask = countPopulation(rMask) == 1 ? rMask : lMask;
-      uint64_t magnMask = countPopulation(rMask) == 1 ? lMask : rMask;
+        (IGCLLVM::popcount(rMask) == 1 || IGCLLVM::popcount(lMask) == 1)) {
+      Value *signOp = IGCLLVM::popcount(rMask) == 1 ? rOp : lOp;
+      Value *magnOp = IGCLLVM::popcount(rMask) == 1 ? lOp : rOp;
+      uint64_t signMask = IGCLLVM::popcount(rMask) == 1 ? rMask : lMask;
+      uint64_t magnMask = IGCLLVM::popcount(rMask) == 1 ? lMask : rMask;
       uint64_t shlBits = 0;
       uint64_t shrBits = 0;
       // %b = shl nsw i32 %a, 24
@@ -1205,5 +1257,6 @@ IGC_INITIALIZE_PASS_BEGIN(GEPLowering, PASS_FLAG2, PASS_DESCRIPTION2, PASS_CFG_O
 IGC_INITIALIZE_PASS_DEPENDENCY(MetaDataUtilsWrapper)
 IGC_INITIALIZE_PASS_DEPENDENCY(CodeGenContextWrapper)
 IGC_INITIALIZE_PASS_DEPENDENCY(LoopInfoWrapperPass)
+IGC_INITIALIZE_PASS_DEPENDENCY(DominatorTreeWrapperPass)
 IGC_INITIALIZE_PASS_DEPENDENCY(ScalarEvolutionWrapperPass);
 IGC_INITIALIZE_PASS_END(GEPLowering, PASS_FLAG2, PASS_DESCRIPTION2, PASS_CFG_ONLY2, PASS_ANALYSIS2)
