@@ -154,7 +154,7 @@ void IGCVectorizer::initializeLogFile(Function &F) {
   auto Name = Debug::DumpName(IGC::Debug::GetShaderOutputName())
                   .Hash(CGCtx->hash)
                   .Type(CGCtx->type)
-                  .Retry(CGCtx->m_retryManager.GetRetryId())
+                  .Retry(CGCtx->m_retryManager->GetRetryId())
                   .Pass(ss.str().c_str())
                   .Extension("ll");
 
@@ -263,8 +263,7 @@ bool isFloatTyped(Instruction *I) {
 }
 
 bool isAllowedType(Instruction *I) {
-    return isFloatTyped(I) ||
-        (IGC_GET_FLAG_VALUE(VectorizerAllowI32) && I->getType()->isIntegerTy(32));
+  return isFloatTyped(I) || (IGC_GET_FLAG_VALUE(VectorizerAllowI32) && I->getType()->isIntegerTy(32));
 }
 
 bool isIntrinsicSafe(Instruction *I) {
@@ -295,7 +294,14 @@ bool isAllowedStub(Instruction *I) {
   return Result;
 }
 
-bool isSafeToVectorize(Instruction *I) {
+bool IGCVectorizer::isSafeToVectorizeSIMD32(Instruction *I) {
+  bool IsExtract = llvm::isa<ExtractElementInst>(I);
+  bool IsInsert = llvm::isa<InsertElementInst>(I);
+  bool Result = isPHISafe(I) || IsExtract || IsInsert;
+  return Result;
+}
+
+bool IGCVectorizer::isSafeToVectorizeSIMD16(Instruction *I) {
 
   bool IsExtract = llvm::isa<ExtractElementInst>(I);
   bool IsInsert = llvm::isa<InsertElementInst>(I);
@@ -307,9 +313,7 @@ bool isSafeToVectorize(Instruction *I) {
   bool IsVectorTyped = I->getType()->isVectorTy();
   bool IsAllowedType = isAllowedType(I);
 
-  bool Result =
-      isPHISafe(I) || IsExtract || IsSelect ||
-      isBinarySafe(I) || isIntrinsicSafe(I) || isAllowedStub(I);
+  bool Result = isPHISafe(I) || IsExtract || IsSelect || isBinarySafe(I) || isIntrinsicSafe(I) || isAllowedStub(I);
 
   // all allowed instructions that are float typed and not vectors
   Result = (Result && IsAllowedType && !IsVectorTyped);
@@ -323,6 +327,17 @@ bool isSafeToVectorize(Instruction *I) {
   return Result;
 }
 
+bool IGCVectorizer::isSafeToVectorize(Instruction *I) {
+
+  if (SIMDSize == 16)
+    return isSafeToVectorizeSIMD16(I);
+  else if (SIMDSize == 32)
+    return isSafeToVectorizeSIMD32(I);
+  else
+    IGC_ASSERT_EXIT_MESSAGE(0, "not supported simd");
+  return false;
+}
+
 bool IGCVectorizer::handleStub(VecArr &Slice) {
   PRINT_LOG("stub vectorization: ");
   PRINT_INST_NL(Slice.front());
@@ -405,7 +420,8 @@ bool IGCVectorizer::handlePHI(VecArr &Slice) {
       Instruction *InsertPoint = BB->getTerminator();
       if (ScalarPhi->getParent() == BB) {
         InsertPoint = getInsertPointForVector(ForVector)->getNextNonDebugInstruction();
-        if (!InsertPoint) return false;
+        if (!InsertPoint)
+          return false;
       }
       auto CreatedVec = createVector(ForVector, InsertPoint);
       PRINT_INST_NL(CreatedVec);
@@ -462,41 +478,43 @@ bool IGCVectorizer::handleInsertElement(VecArr &Slice, Instruction *Final) {
 
 Instruction *IGCVectorizer::getInsertPointForVector(VecArr &Arr) {
 
-  Instruction* Cmp = Arr.front();
+  Instruction *Cmp = Arr.front();
   for (auto &El : Arr)
     if (El->getParent() != Cmp->getParent())
-        return nullptr;
+      return nullptr;
 
-  Instruction* InsertPoint = getMaxPoint(Arr);
+  Instruction *InsertPoint = getMaxPoint(Arr);
   // if insert point is PHI, shift it to the first nonPHI to be safe
   if (llvm::isa<llvm::PHINode>(InsertPoint))
-      InsertPoint = InsertPoint->getParent()->getFirstNonPHI();
+    InsertPoint = InsertPoint->getParent()->getFirstNonPHI();
   if (InsertPoint->isTerminator())
-      InsertPoint = InsertPoint->getPrevNonDebugInstruction();
+    InsertPoint = InsertPoint->getPrevNonDebugInstruction();
 
   return InsertPoint;
 }
 
-Instruction* IGCVectorizer::getInsertPointForCreatedInstruction(VecVal &Operands, VecArr& Slice) {
+Instruction *IGCVectorizer::getInsertPointForCreatedInstruction(VecVal &Operands, VecArr &Slice) {
 
-    VecArr InstOperands;
-    for (auto &El : Operands) {
-        auto Inst = llvm::dyn_cast<Instruction>(El);
-        if (!Inst) continue;
-        if (Inst->getParent() == Slice.front()->getParent())
-            InstOperands.push_back(Inst);
-        PRINT_LOG("insertPointCandidate: "); PRINT_INST_NL(Inst);
-    }
+  VecArr InstOperands;
+  for (auto &El : Operands) {
+    auto Inst = llvm::dyn_cast<Instruction>(El);
+    if (!Inst)
+      continue;
+    if (Inst->getParent() == Slice.front()->getParent())
+      InstOperands.push_back(Inst);
+    PRINT_LOG("insertPointCandidate: ");
+    PRINT_INST_NL(Inst);
+  }
 
-    Instruction* InsertPoint = Slice.front()->getParent()->getFirstNonPHI();
-    if (InstOperands.size() != 0) {
-        InsertPoint = getMaxPoint(InstOperands)->getNextNonDebugInstruction();
-        // if insert point is PHI, shift it to the first nonPHI to be safe
-        if (llvm::isa<llvm::PHINode>(InsertPoint))
-            InsertPoint = InsertPoint->getParent()->getFirstNonPHI();
-    }
+  Instruction *InsertPoint = Slice.front()->getParent()->getFirstNonPHI();
+  if (InstOperands.size() != 0) {
+    InsertPoint = getMaxPoint(InstOperands)->getNextNonDebugInstruction();
+    // if insert point is PHI, shift it to the first nonPHI to be safe
+    if (llvm::isa<llvm::PHINode>(InsertPoint))
+      InsertPoint = InsertPoint->getParent()->getFirstNonPHI();
+  }
 
-    return InsertPoint;
+  return InsertPoint;
 }
 
 Instruction *IGCVectorizer::getMaxPoint(VecArr &Slice) {
@@ -504,7 +522,8 @@ Instruction *IGCVectorizer::getMaxPoint(VecArr &Slice) {
   Instruction *MaxPoint = Slice.front();
   for (auto &El : Slice) {
     unsigned NewPos = getPositionInsideBB(El);
-    PRINT_LOG("pos: " << NewPos << " "); PRINT_INST_NL(El);
+    PRINT_LOG("pos: " << NewPos << " ");
+    PRINT_INST_NL(El);
     if (NewPos > MaxPos) {
       MaxPos = NewPos;
       MaxPoint = El;
@@ -587,30 +606,12 @@ bool IGCVectorizer::handleBinaryInstruction(VecArr &Slice) {
 
   Value *PrevVectorization = nullptr;
   Instruction *First = Slice.front();
-  if (ScalarToVector.count(First)) {
-    auto Vectorized = ScalarToVector[First];
-    if (llvm::isa<InsertElementInst>(Vectorized)) {
-      PRINT_LOG_NL("Was sourced by other vector instruction, but wasn't vectorized");
-      PrevVectorization = Vectorized;
-    } else {
-      PRINT_LOG_NL("Already was vectorized by other slice");
-      return true;
-    }
-  }
+  if (checkPrevVectorization(Slice, PrevVectorization))
+    return true;
+
   VecVal Operands;
-  for (unsigned int OperNum = 0; OperNum < First->getNumOperands(); ++OperNum) {
-    Value *Vectorized = checkOperandsToBeVectorized(First, OperNum, Slice);
-    if (Vectorized)
-      Operands.push_back(Vectorized);
-    else {
-      Value *VectorizedOperand = vectorizeSlice(Slice, OperNum);
-      if (!VectorizedOperand) {
-        PRINT_LOG_NL("Couldn't vectorize Slice");
-        return false;
-      }
-      Operands.push_back(VectorizedOperand);
-    }
-  }
+  if (!collectOperandsForVectorization(0, First->getNumOperands(), First, Slice, Operands))
+    return false;
 
   PRINT_DS("Operands: ", Operands);
   Instruction *InsertPoint = getInsertPointForCreatedInstruction(Operands, Slice);
@@ -646,90 +647,73 @@ bool IGCVectorizer::handleBinaryInstruction(VecArr &Slice) {
   return true;
 }
 
-bool IGCVectorizer::handleSelectInstruction(VecArr& Slice){
+bool IGCVectorizer::handleSelectInstruction(VecArr &Slice) {
 
   bool IsSliceUniform = true;
   for (auto El : Slice)
-      IsSliceUniform &= WI->isUniform(El);
+    IsSliceUniform &= WI->isUniform(El);
 
   if (!IsSliceUniform) {
-      PRINT_LOG_NL("Select is stub vectorized, not uniform");
-      return true;
+    PRINT_LOG_NL("Select is stub vectorized, not uniform");
+    return true;
   }
 
   if (!IGC_GET_FLAG_VALUE(VectorizerAllowUniformSelect))
-      return true;
+    return true;
 
   Value *PrevVectorization = nullptr;
   Instruction *First = Slice.front();
-  if (ScalarToVector.count(First)) {
-    auto Vectorized = ScalarToVector[First];
-    if (llvm::isa<InsertElementInst>(Vectorized)) {
-      PRINT_LOG_NL("Was sourced by other vector instruction, but wasn't vectorized");
-      PrevVectorization = Vectorized;
-    } else {
-      PRINT_LOG_NL("Already was vectorized by other slice");
-      return true;
-    }
-  }
+  if (checkPrevVectorization(Slice, PrevVectorization))
+    return true;
 
   auto Cmp = Slice.front()->getOperand(0);
   bool SamePredicate = true;
   for (auto &El : Slice) {
     Value *Val = El->getOperand(0);
-    PRINT_INST(El); PRINT_LOG(" --> "); PRINT_INST_NL(Val);
+    PRINT_INST(El);
+    PRINT_LOG(" --> ");
+    PRINT_INST_NL(Val);
     SamePredicate &= Val == Cmp;
   }
 
   VecVal Operands;
-  for (unsigned int OperNum = 0; OperNum < First->getNumOperands(); ++OperNum) {
-    Value *Vectorized = checkOperandsToBeVectorized(First, OperNum, Slice);
-    if (Vectorized)
-      Operands.push_back(Vectorized);
-    else {
-      Value *VectorizedOperand = vectorizeSlice(Slice, OperNum);
-      if (!VectorizedOperand) {
-        PRINT_LOG_NL("Couldn't vectorize Slice");
-        return false;
-      }
-      Operands.push_back(VectorizedOperand);
-    }
-  }
+  if (!collectOperandsForVectorization(0, First->getNumOperands(), First, Slice, Operands))
+    return false;
 
   if (SamePredicate) {
-      PRINT_LOG("Same Predicate! \n");
-      if (IGC_IS_FLAG_ENABLED(VectorizerAllowSamePredSelect)) {
-          // our emitter can't properly process arbitrary masks
-          // represented by vector of booleans
-          // it starts emitting "insert elements into <8 x i1>
-          // and fails, it must either have result from CMP instructions
-          // directly, or has single scalar predicate of the same type
-          // fixed constant masks are not supported right now
-          // so we can't give as a predicate something of the form:
-          //
-          // CAN'T PROCESS THIS:
-          // %pred_vector0 = insertelemtnt %cmp0, 0
-          // %pred_vector1 = insertelemtnt %cmp1, 1
-          // %pred_vector2 = insertelemtnt %cmp2, 2
-          // ...
-          // %pred_vector7 = insertelemtnt %cmp7, 7
-          // select <8 x i1> %pred_vector7, <8 x float> ...
-          //
-          // but if the pattern is as follows
-          // %pred_vector0 = insertelemtnt %cmp0, 0
-          // %pred_vector1 = insertelemtnt %cmp0, 1
-          // ....
-          // %pred_vector7 = insertelemtnt %cmp0, 7
-          //
-          // it's possible to substitute with
-          // select i1 %cmp0, <8 x float> ....
-          //
-          // it will work, still don't want to enable this by default
-          // increases test surface.
-          // for example another example check: vectorizer-select-one-bool.ll
-          Operands[0] = Cmp;
-      }
-      else return false;
+    PRINT_LOG("Same Predicate! \n");
+    if (IGC_IS_FLAG_ENABLED(VectorizerAllowSamePredSelect)) {
+      // our emitter can't properly process arbitrary masks
+      // represented by vector of booleans
+      // it starts emitting "insert elements into <8 x i1>
+      // and fails, it must either have result from CMP instructions
+      // directly, or has single scalar predicate of the same type
+      // fixed constant masks are not supported right now
+      // so we can't give as a predicate something of the form:
+      //
+      // CAN'T PROCESS THIS:
+      // %pred_vector0 = insertelemtnt %cmp0, 0
+      // %pred_vector1 = insertelemtnt %cmp1, 1
+      // %pred_vector2 = insertelemtnt %cmp2, 2
+      // ...
+      // %pred_vector7 = insertelemtnt %cmp7, 7
+      // select <8 x i1> %pred_vector7, <8 x float> ...
+      //
+      // but if the pattern is as follows
+      // %pred_vector0 = insertelemtnt %cmp0, 0
+      // %pred_vector1 = insertelemtnt %cmp0, 1
+      // ....
+      // %pred_vector7 = insertelemtnt %cmp0, 7
+      //
+      // it's possible to substitute with
+      // select i1 %cmp0, <8 x float> ....
+      //
+      // it will work, still don't want to enable this by default
+      // increases test surface.
+      // for example another example check: vectorizer-select-one-bool.ll
+      Operands[0] = Cmp;
+    } else
+      return false;
   }
 
   PRINT_DS("Operands: ", Operands);
@@ -738,6 +722,9 @@ bool IGCVectorizer::handleSelectInstruction(VecArr& Slice){
   auto *CreatedInst = SelectInst::Create(Operands[0], Operands[1], Operands[2]);
 
   CreatedInst->setName("vectorized_select");
+  MDNode *Node = MDNode::get(CreatedInst->getContext(), MDString::get(CreatedInst->getContext(), "vectorized"));
+  CreatedInst->setMetadata("vectorized", Node);
+
   CreatedInst->setDebugLoc(First->getDebugLoc());
   CreatedInst->insertBefore(InsertPoint);
   CreatedVectorInstructions.push_back(CreatedInst);
@@ -765,35 +752,27 @@ bool IGCVectorizer::handleSelectInstruction(VecArr& Slice){
   return true;
 }
 
-bool IGCVectorizer::handleCMPInstruction(VecArr& Slice){
+bool IGCVectorizer::checkPrevVectorization(VecArr &Slice, Value *&OutPrevVectorization) {
 
-    bool IsSliceUniform = true;
-    for (auto El : Slice)
-        IsSliceUniform &= WI->isUniform(El);
-
-    if (!IsSliceUniform) {
-        PRINT_LOG_NL("Select is stub vectorized, not uniform");
-        return true;
-    }
-
-    if (!IGC_GET_FLAG_VALUE(VectorizerAllowUniformCMP))
-        return true;
-
-  Value *PrevVectorization = nullptr;
   Instruction *First = Slice.front();
-  if (ScalarToVector.count(First)) {
-    auto Vectorized = ScalarToVector[First];
-    if (llvm::isa<InsertElementInst>(Vectorized)) {
-      PRINT_LOG_NL("Was sourced by other vector instruction, but wasn't vectorized");
-      PrevVectorization = Vectorized;
-    } else {
-      PRINT_LOG_NL("Already was vectorized by other slice");
-      return true;
-    }
-  }
+  if (!ScalarToVector.count(First))
+    return false;
 
-  VecVal Operands;
-  for (unsigned int OperNum = 0; OperNum < First->getNumOperands(); ++OperNum) {
+  auto Vectorized = ScalarToVector[First];
+  if (llvm::isa<InsertElementInst>(Vectorized)) {
+    PRINT_LOG_NL("Was sourced by other vector instruction, but wasn't vectorized");
+    OutPrevVectorization = Vectorized;
+  } else {
+    PRINT_LOG_NL("Already was vectorized by other slice");
+    return true;
+  }
+  return false;
+}
+
+bool IGCVectorizer::collectOperandsForVectorization(unsigned OperNumToStart, unsigned OperNumToStop, Instruction *First,
+                                                    VecArr &Slice, VecVal &Operands) {
+
+  for (unsigned int OperNum = OperNumToStart; OperNum < OperNumToStop; ++OperNum) {
     Value *Vectorized = checkOperandsToBeVectorized(First, OperNum, Slice);
     if (Vectorized)
       Operands.push_back(Vectorized);
@@ -806,6 +785,43 @@ bool IGCVectorizer::handleCMPInstruction(VecArr& Slice){
       Operands.push_back(VectorizedOperand);
     }
   }
+  return true;
+}
+
+bool IGCVectorizer::handleCMPInstruction(VecArr &Slice) {
+
+  bool IsSliceUniform = true;
+  for (auto El : Slice) {
+    IsSliceUniform &= WI->isUniform(El);
+    // we must process users as well
+    // because we do not support
+    // bit flag extraction from predicates
+    for (auto User : El->users()) {
+      bool UserIsUniform = WI->isUniform(User);
+      if (!UserIsUniform) {
+        PRINT_LOG("User is not uniform: ");
+        PRINT_INST_NL(User);
+      }
+      IsSliceUniform &= UserIsUniform;
+    }
+  }
+
+  if (!IsSliceUniform) {
+    PRINT_LOG_NL("Select is stub vectorized, not uniform");
+    return true;
+  }
+
+  if (!IGC_GET_FLAG_VALUE(VectorizerAllowUniformCMP))
+    return true;
+
+  Instruction *First = Slice.front();
+  Value *PrevVectorization = nullptr;
+  if (checkPrevVectorization(Slice, PrevVectorization))
+    return true;
+
+  VecVal Operands;
+  if (!collectOperandsForVectorization(0, First->getNumOperands(), First, Slice, Operands))
+    return false;
 
   PRINT_DS("Operands: ", Operands);
   Instruction *InsertPoint = getInsertPointForCreatedInstruction(Operands, Slice);
@@ -841,8 +857,6 @@ bool IGCVectorizer::handleCMPInstruction(VecArr& Slice){
   }
 
   return true;
-
-
 }
 
 bool IGCVectorizer::handleCastInstruction(VecArr &Slice) {
@@ -883,36 +897,102 @@ bool IGCVectorizer::handleCastInstruction(VecArr &Slice) {
   return true;
 }
 
+bool IGCVectorizer::handleWaveAll(VecArr &Slice) {
+
+  if (!isGenIntrinsicSafe(Slice.front()))
+    return false;
+
+  // if we do not issue waveall joints, we vectorize them as stubs
+  if (!IGC_GET_FLAG_VALUE(VectorizerAllowWAVEALLJoint))
+    return true;
+
+  // if the platform is unsupported we vectorize as stubs
+  if (!AllowedPlatform) {
+    PRINT_LOG_NL("Unsupported platform");
+    return true;
+  }
+
+  auto *First = llvm::dyn_cast<WaveAllIntrinsic>(Slice.front());
+  if (!First)
+    return false;
+
+  for (auto El : Slice) {
+    auto *ElWave = llvm::dyn_cast<WaveAllIntrinsic>(El);
+    if (!ElWave)
+      return false;
+    bool SameType = First->getOpKind() == ElWave->getOpKind();
+    if (!SameType) {
+      PRINT_LOG_NL("Wavealls are not same type");
+      return false;
+    }
+  }
+
+  PRINT_DS("waveall: ", Slice);
+
+  Value *PrevVectorization = nullptr;
+  if (checkPrevVectorization(Slice, PrevVectorization))
+    return true;
+
+  VecVal Operands;
+  if (!collectOperandsForVectorization(0, 1, First, Slice, Operands))
+    return false;
+
+  // scalar operands
+  Operands.push_back(First->getOperand(1));
+  Operands.push_back(First->getOperand(2));
+
+  PRINT_DS("Operands: ", Operands);
+  Instruction *InsertPoint = getInsertPointForCreatedInstruction(Operands, Slice);
+
+  llvm::VectorType *VectorType = llvm::FixedVectorType::get(First->getType(), Slice.size());
+
+  auto IntrinsicID = llvm::cast<GenIntrinsicInst>(First)->getIntrinsicID();
+  auto *Decl = GenISAIntrinsic::getDeclaration(M, IntrinsicID, {VectorType});
+  PRINT_DECL_NL(Decl);
+  auto *CreatedInst = llvm::CallInst::Create(Decl, Operands);
+
+  CreatedInst->setName("vectorized_joint_waveall");
+  CreatedInst->setDebugLoc(First->getDebugLoc());
+  CreatedInst->insertAfter(InsertPoint);
+  CreatedVectorInstructions.push_back(CreatedInst);
+
+  PRINT_LOG("Intrinsic instruction created: ");
+  PRINT_INST_NL(CreatedInst);
+
+  replaceSliceInstructionsWithExtract(Slice, CreatedInst);
+
+  for (auto &el : Slice) {
+    if (ScalarToVector.count(el)) {
+      PRINT_LOG_NL("Vectorized version already present");
+      PRINT_INST(el);
+      PRINT_LOG(" --> ");
+      PRINT_INST_NL(ScalarToVector[el]);
+    }
+    ScalarToVector[el] = CreatedInst;
+  }
+
+  if (PrevVectorization) {
+    PRINT_LOG_NL("Replaced with proper vector version");
+    PrevVectorization->replaceAllUsesWith(CreatedInst);
+  }
+
+  for (auto El : Slice)
+    El->eraseFromParent();
+
+  return true;
+}
+
 bool IGCVectorizer::handleIntrinsic(VecArr &Slice) {
 
   Value *PrevVectorization = nullptr;
   Instruction *First = Slice.front();
-  if (ScalarToVector.count(First)) {
-    auto Vectorized = ScalarToVector[First];
-    if (llvm::isa<InsertElementInst>(Vectorized)) {
-      PRINT_LOG_NL("Was sourced by other vector instruction, but wasn't vectorized");
-      PrevVectorization = Vectorized;
-    } else {
-      PRINT_LOG_NL("Already was vectorized by other slice");
-      return true;
-    }
-  }
+
+  if (checkPrevVectorization(Slice, PrevVectorization))
+    return true;
 
   VecVal Operands;
-  for (unsigned int OperNum = 0; OperNum < First->getNumOperands() - 1; ++OperNum) {
-
-    Value *Vectorized = checkOperandsToBeVectorized(First, OperNum, Slice);
-    if (Vectorized)
-      Operands.push_back(Vectorized);
-    else {
-      Value *VectorizedOperand = vectorizeSlice(Slice, OperNum);
-      if (!VectorizedOperand) {
-        PRINT_LOG_NL("Couldn't vectorize Slice");
-        return false;
-      }
-      Operands.push_back(VectorizedOperand);
-    }
-  }
+  if (!collectOperandsForVectorization(0, First->getNumOperands() - 1, First, Slice, Operands))
+    return false;
 
   PRINT_DS("Operands: ", Operands);
   Instruction *InsertPoint = getInsertPointForCreatedInstruction(Operands, Slice);
@@ -984,9 +1064,6 @@ bool IGCVectorizer::processChain(InsertStruct &InSt) {
     } else if (llvm::isa<CastInst>(First)) {
       if (!handleCastInstruction(Slice))
         return false;
-    } else if (isAllowedStub(First)) {
-      if (!handleStub(Slice))
-        return false;
     } else if (llvm::isa<BinaryOperator>(First)) {
       if (!handleBinaryInstruction(Slice))
         return false;
@@ -994,8 +1071,11 @@ bool IGCVectorizer::processChain(InsertStruct &InSt) {
       if (!handleCMPInstruction(Slice))
         return false;
     } else if (llvm::isa<SelectInst>(First)) {
-        if (!handleSelectInstruction(Slice))
-            return false;
+      if (!handleSelectInstruction(Slice))
+        return false;
+    } else if (llvm::isa<GenIntrinsicInst>(First)) {
+      if (!handleWaveAll(Slice))
+        return false;
     } else if (llvm::isa<IntrinsicInst>(First)) {
       if (!handleIntrinsic(Slice))
         return false;
@@ -1005,6 +1085,9 @@ bool IGCVectorizer::processChain(InsertStruct &InSt) {
     } else if (llvm::isa<InsertElementInst>(First)) {
       if (!handleInsertElement(Slice, InSt.Final))
         return false;
+    } else if (isAllowedStub(First)) {
+      if (!handleStub(Slice))
+        return false;
     } else {
       IGC_ASSERT("we should not be here");
     }
@@ -1171,8 +1254,8 @@ Value *IGCVectorizer::vectorizeSlice(VecArr &Slice, unsigned int OperNum) {
   if (NotVectorizedInstruction.size() == Slice.size()) {
     Instruction *InsertPoint = getInsertPointForVector(NotVectorizedInstruction);
     if (!InsertPoint) {
-        PRINT_LOG_NL("Couldn't find insert point");
-        return nullptr;
+      PRINT_LOG_NL("Couldn't find insert point");
+      return nullptr;
     }
     NewVector = createVector(NotVectorizedInstruction, InsertPoint->getNextNonDebugInstruction());
     PRINT_LOG("New vector created: ");
@@ -1336,6 +1419,8 @@ bool IGCVectorizer::checkDependencyAndTryToEliminate(VecArr &Slice) {
       if (!Poisoned.count(Operand))
         continue;
       PRINT_INST(Operand);
+      PRINT_LOG(" --- ");
+      PRINT_INST(El);
       PRINT_LOG_NL("  <-- operands inside the slice depend on slice results");
       return true;
     }
@@ -1432,31 +1517,37 @@ void IGCVectorizer::collectInstructionToProcess(VecArr &ToProcess, Function &F)
   }
 }
 
-bool IGCVectorizer::checkIfSIMD16(llvm::Function &F) {
+unsigned IGCVectorizer::checkSIMD(llvm::Function &F) {
 
   MDUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
-  bool Result = false;
+  unsigned SimdSize = 0;
   if (MDUtils->findFunctionsInfoItem(&F) != MDUtils->end_FunctionsInfo()) {
     IGC::IGCMD::FunctionInfoMetaDataHandle funcInfoMD = MDUtils->getFunctionsInfoItem(&F);
-    unsigned SimdSize = funcInfoMD->getSubGroupSize()->getSIMDSize();
-    Result = SimdSize == 16;
+    SimdSize = funcInfoMD->getSubGroupSize()->getSIMDSize();
   }
 
-  return Result;
+  return SimdSize;
 }
 
 bool IGCVectorizer::runOnFunction(llvm::Function &F) {
 
-  // DPAS only allowed in simd16 mode + helps to reduce untested cases
-  if (!checkIfSIMD16(F))
-    return false;
-
-  WI = &getAnalysis<WIAnalysis>();
-
   M = F.getParent();
   CGCtx = getAnalysis<CodeGenContextWrapper>().getCodeGenContext();
   initializeLogFile(F);
-  PRINT_LOG_NL("vectorizer: fadd, fdiv, fptrunc");
+
+  AllowedPlatform = CGCtx->platform.isCoreXE2() || CGCtx->platform.isPVC() || CGCtx->platform.isCoreXE3();
+  SIMDSize = checkSIMD(F);
+  // we have DPAS and simd8 for DG2 platforms
+  bool SupportedSIMD = SIMDSize == 16 || SIMDSize == 32;
+  if (!SupportedSIMD) {
+    PRINT_LOG_NL("Unsupported SIMD: " << SIMDSize << " exit");
+    return false;
+  }
+
+  PRINT_LOG_NL(" SIMD Size: " << SIMDSize);
+  PRINT_LOG_NL("vectorizer: fadd, fdiv, fptrunc, select, cmp, intrinsics, genintrinsics, simd32");
+
+  WI = &getAnalysis<WIAnalysis>();
 
   VecArr ToProcess;
   // we collect operands that seem promising for vectorization
