@@ -71,10 +71,16 @@ class IGCVectorizer : public llvm::FunctionPass {
   std::string LogStr;
   llvm::raw_string_ostream OutputLogStream = raw_string_ostream(LogStr);
   Module *M = nullptr;
-  bool checkIfSIMD16(llvm::Function &F);
+  unsigned SIMDSize = 0;
+  bool AllowedPlatform = true;
+  unsigned checkSIMD(llvm::Function &F);
   void initializeLogFile(Function &F);
   void writeLog();
 
+  bool isSafeToVectorize(llvm::Instruction *I);
+  bool isSafeToVectorizeSIMD16(llvm::Instruction *I);
+  bool isSafeToVectorizeSIMD32(llvm::Instruction *I);
+
   void findInsertElementsInDataFlow(llvm::Instruction *I, VecArr &Chain);
   bool checkSlice(VecArr &Slice, InsertStruct &InSt);
   bool processChain(InsertStruct &InSt);
@@ -104,7 +110,11 @@ class IGCVectorizer : public llvm::FunctionPass {
   bool handleBinaryInstruction(VecArr &Slice);
   bool handleCMPInstruction(VecArr &Slice);
   bool handleIntrinsic(VecArr &Slice);
+  bool handleWaveAll(VecArr &Slice);
   bool checkBinaryOperator(VecArr &Slice);
+  bool checkPrevVectorization(VecArr &Slice, Value *&PrevVectorization);
+  bool collectOperandsForVectorization(unsigned OperNumToStart, unsigned OperNumToStop, Instruction *First,
+                                       VecArr &Slice, VecVal &Operands);
   bool handleIntrinsicInstruction(VecArr &Slice);
 
   Value *checkOperandsToBeVectorized(Instruction *First, unsigned int OperNum, VecArr &Slice);
