@@ -106,7 +106,7 @@ public:
   llvm::raw_string_ostream OutputLogStream = raw_string_ostream(LogStr);
 
   CodeGenContext *CGCtx = nullptr;
-  IGCLivenessAnalysis *RPE = nullptr;
+  IGCLivenessAnalysisRunner *RPE = nullptr;
 
 private:
   IGC::REMAT_OPTIONS m_rematFlags = REMAT_NONE;
@@ -547,7 +547,7 @@ void CloneAddressArithmetic::speculateWholeChain(RematSet &ToProcess, unsigned i
 
 bool CloneAddressArithmetic::isRegPressureLow(Function &F) {
 
-  RPE = &getAnalysis<IGCLivenessAnalysis>();
+  RPE = &getAnalysis<IGCLivenessAnalysis>().getLivenessRunner();
   unsigned int SIMD = numLanes(RPE->bestGuessSIMDSize(&F));
   unsigned int PressureLimit = IGC_GET_FLAG_VALUE(RematRPELimit);
   unsigned int MaxPressure = RPE->getMaxRegCountForFunction(F, SIMD, &WI->Runner);
@@ -612,7 +612,6 @@ void CloneAddressArithmetic::collectInstToProcess(RematSet &ToProcess, Function
 
       if (isRematInstruction(V))
         ToProcess.insert(static_cast<Instruction *>(V));
-
     }
   }
 }
@@ -686,7 +685,7 @@ void CloneAddressArithmetic::initializeLogFile(Function &F) {
   auto Name = Debug::DumpName(IGC::Debug::GetShaderOutputName())
                   .Hash(CGCtx->hash)
                   .Type(CGCtx->type)
-                  .Retry(CGCtx->m_retryManager.GetRetryId())
+                  .Retry(CGCtx->m_retryManager->GetRetryId())
                   .Pass(ss.str().c_str())
                   .Extension("ll");
 
