@@ -103,8 +103,10 @@ SPDX-License-Identifier: MIT
 #include "Compiler/Optimizer/IntDivRemCombine.hpp"
 #include "Compiler/Optimizer/SynchronizationObjectCoalescing.hpp"
 #include "Compiler/Optimizer/BarrierControlFlowOptimization.hpp"
+#include "Compiler/Optimizer/BarrierSkipOptimization.hpp"
 #include "Compiler/Optimizer/RuntimeValueVectorExtractPass.h"
 #include "Compiler/Optimizer/WaveShuffleIndexSinking.hpp"
+#include "Compiler/Optimizer/SinkPointerConstAdd.h"
 #include "Compiler/Optimizer/WaveAllJointReduction.hpp"
 #include "Compiler/Optimizer/InstructionHoistingOptimization.hpp"
 #include "Compiler/Optimizer/WaveBallotCSE.hpp"
@@ -155,6 +157,22 @@ SPDX-License-Identifier: MIT
 #include <llvm/Transforms/Utils.h>
 #include <llvm/Transforms/Scalar.h>
 #include <llvm/Bitcode/BitcodeWriter.h>
+#include "llvmWrapper/Transforms/Scalar/ADCE.h"
+#include "llvmWrapper/Transforms/Scalar/CorrelatedValuePropagation.h"
+#include "llvmWrapper/Transforms/Scalar/DeadStoreElimination.h"
+#include "llvmWrapper/Transforms/Scalar/JumpThreading.h"
+#include "llvmWrapper/Transforms/IPO/ConstantMerge.h"
+#include "llvmWrapper/Transforms/IPO/SCCP.h"
+#include "llvmWrapper/Transforms/IPO/GlobalDCE.h"
+#include "llvmWrapper/Transforms/IPO/PostOrderFunctionAttrs.h"
+#include "llvmWrapper/Transforms/Scalar/MemCpyOptimizer.h"
+#include "llvmWrapper/Transforms/Scalar/LoopDeletion.h"
+#include "llvmWrapper/Transforms/Scalar/LoopLoadElimination.h"
+#include "llvmWrapper/Transforms/Scalar/SCCP.h"
+#include "llvmWrapper/Transforms/Scalar/LoopUnrollPass.h"
+#include "llvmWrapper/Transforms/Scalar/LICM.h"
+#include "llvmWrapper/Transforms/Scalar/IndVarSimplify.h"
+
 #include "common/LLVMWarningsPop.hpp"
 #include "Compiler/CISACodeGen/PatternMatchPass.hpp"
 #include "Compiler/CISACodeGen/EmitVISAPass.hpp"
@@ -250,7 +268,7 @@ void AddAnalysisPasses(CodeGenContext &ctx, IGCPassManager &mpm) {
     }
     if (IGC_IS_FLAG_DISABLED(DisableCodeScheduling) && (ctx.type == ShaderType::OPENCL_SHADER) &&
         (ctx.platform.isCoreChildOf(IGFX_XE_HPC_CORE) || ctx.platform.isCoreChildOf(IGFX_XE2_HPG_CORE))) {
-      if (IGC_IS_FLAG_DISABLED(CodeSchedulingOnlyRecompilation) || ctx.m_retryManager.AllowCodeScheduling()) {
+      if (IGC_IS_FLAG_DISABLED(CodeSchedulingOnlyRecompilation) || ctx.m_retryManager->AllowCodeScheduling()) {
         mpm.add(new CodeScheduling());
       }
     }
@@ -356,6 +374,10 @@ void AddAnalysisPasses(CodeGenContext &ctx, IGCPassManager &mpm) {
     mpm.add(new StackOverflowDetectionPass(StackOverflowDetectionPass::Mode::RemoveDummyCalls));
   }
 
+  if (IGC_IS_FLAG_ENABLED(EnableSinkPointerConstAdd)) {
+    mpm.add(createSinkPointerConstAddPass());
+  }
+
   //
   // Generally, passes that change IR should be prior to this place!
   //
@@ -479,7 +501,7 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
 
   if (ctx.m_threadCombiningOptDone) {
     mpm.add(createLoopCanonicalization());
-    mpm.add(llvm::createLoopDeletionPass());
+    mpm.add(IGCLLVM::createLegacyWrappedLoopDeletionPass());
     mpm.add(llvm::createBreakCriticalEdgesPass());
     mpm.add(llvm::createLoopRotatePass(LOOP_ROTATION_HEADER_INST_THRESHOLD));
     mpm.add(llvm::createLowerSwitchPass());
@@ -487,14 +509,17 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     int LoopUnrollThreshold = ctx.m_DriverInfo.GetLoopUnrollThreshold();
 
     if (LoopUnrollThreshold > 0 && (ctx.m_tempCount < 64)) {
-      mpm.add(llvm::createLoopUnrollPass(2, false, false, LoopUnrollThreshold, -1, 1, -1, -1, -1));
+      mpm.add(IGCLLVM::createLegacyWrappedLoopUnrollPass(2, false, false, LoopUnrollThreshold, -1, 1, -1, -1, -1));
     }
 
     mpm.add(createBarrierNoopPass());
 
-    if (IGC_IS_FLAG_ENABLED(allowLICM) && ctx.m_retryManager.AllowLICM()) {
+    bool AllowLICM =
+        IGC_IS_FLAG_SET(allowLICM) ? IGC_IS_FLAG_ENABLED(allowLICM) : ctx.getModuleMetaData()->compOpt.AllowLICM;
+
+    if (ctx.m_retryManager->AllowLICM() && AllowLICM) {
       mpm.add(createSpecialCasesDisableLICM());
-      mpm.add(llvm::createLICMPass(100, 500, true));
+      mpm.add(IGCLLVM::createLegacyWrappedLICMPass(100, 500, true));
     }
     mpm.add(llvm::createLoopSimplifyPass());
   }
@@ -556,7 +581,7 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     // DCE doesn't remove dead control flow; ADCE does (currently)
     // otherwise you'd have to call createCFGSimplificationPass and DCE
     // iteratively e.g..
-    mpm.add(llvm::createAggressiveDCEPass());
+    mpm.add(IGCLLVM::createLegacyWrappedADCEPass());
     // TODO: we probably should be running other passes on the result
 
     if (!IGC::ForceAlwaysInline(&ctx)) {
@@ -649,7 +674,7 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
 
   if (ctx.enableFunctionCall() || ctx.type == ShaderType::RAYTRACING_SHADER) {
     // Sort functions if subroutine/indirect fcall is enabled.
-    mpm.add(llvm::createGlobalDCEPass());
+    mpm.add(IGCLLVM::createLegacyWrappedGlobalDCEPass());
     mpm.add(new PurgeMetaDataUtils());
     mpm.add(createGenXCodeGenModulePass());
   }
@@ -722,9 +747,8 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
       static_cast<OpenCLProgramContext &>(ctx).m_InternalOptions.PromoteStatelessToBindless) {
     if (static_cast<OpenCLProgramContext &>(ctx).m_InternalOptions.UseBindlessLegacyMode) {
       mpm.add(new PromoteStatelessToBindless());
-    }
-    else if (!ctx.getModuleMetaData()->compOpt.GreaterThan4GBBufferRequired && !isOptDisabled)
-    {
+    } else if (!ctx.platform.hasEfficient64bEnabled() &&
+               !ctx.getModuleMetaData()->compOpt.GreaterThan4GBBufferRequired && !isOptDisabled) {
       // Advanced bindless mode used by the regular OpenCL compilation path
       mpm.add(new StatelessToStateful(TargetAddressing::BINDLESS));
     }
@@ -744,7 +768,7 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     const bool allowIPConstProp = !ctx.m_hasStackCalls && IGC_IS_FLAG_DISABLED(DisableIPConstantPropagation);
 
     if (allowIPConstProp) {
-      mpm.add(createIPSCCPPass());
+      mpm.add(IGCLLVM::createLegacyWrappedIPSCCPPass());
     }
     mpm.add(createDeadCodeEliminationPass());
     mpm.add(createCFGSimplificationPass());
@@ -792,6 +816,9 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     // Last instruction combining pass needs to be before Legalization pass, as it can produce illegal instructions.
     mpm.add(new RemoveCodeAssumptions());
     mpm.add(createIGCInstructionCombiningPass());
+    if (ctx.platform.doIntegerMad() && ctx.m_DriverInfo.EnableIntegerMad()) {
+      mpm.add(createCanonicalizeMulAddPass());
+    }
     mpm.add(new GenSpecificPattern());
     // Cases with DPDivSqrtEmu grow significantly.
     // We can disable EarlyCSE when m_hasDPDivSqrtEmu is true,
@@ -801,13 +828,16 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     } else if (highAllocaPressure || isPotentialHPCKernel) {
       mpm.add(createSinkingPass());
     }
-    if (!fastCompile && !highAllocaPressure && !isPotentialHPCKernel && IGC_IS_FLAG_ENABLED(allowLICM) &&
-        ctx.m_retryManager.AllowLICM()) {
+
+    bool AllowLICM =
+        IGC_IS_FLAG_SET(allowLICM) ? IGC_IS_FLAG_ENABLED(allowLICM) : ctx.getModuleMetaData()->compOpt.AllowLICM;
+
+    if (!fastCompile && !highAllocaPressure && !isPotentialHPCKernel && AllowLICM && ctx.m_retryManager->AllowLICM()) {
       mpm.add(createSpecialCasesDisableLICM());
-      mpm.add(llvm::createLICMPass(100, 500, true));
+      mpm.add(IGCLLVM::createLegacyWrappedLICMPass(100, 500, true));
       mpm.add(llvm::createEarlyCSEPass());
     }
-    mpm.add(createAggressiveDCEPass());
+    mpm.add(IGCLLVM::createLegacyWrappedADCEPass());
     // As DPC++ FE apply LICM we cannot reduce register pressure just
     // by turning off LICM at IGC in some cases so apply sinking address arithmetic
     if ((IGC_IS_FLAG_ENABLED(ForceAddressArithSinking) ||
@@ -839,7 +869,7 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
   // Run address remat after GVN as it may hoist address calculations and
   // create PHI nodes with addresses.
   if (IGC_IS_FLAG_ENABLED(RematEnable) ||
-      (ctx.m_retryManager.AllowCloneAddressArithmetic() && ctx.type == ShaderType::OPENCL_SHADER)) {
+      (ctx.m_retryManager->AllowCloneAddressArithmetic() && ctx.type == ShaderType::OPENCL_SHADER)) {
 
     if (IGC_GET_FLAG_VALUE(RematInstCombineBefore))
       mpm.add(createIGCInstructionCombiningPass());
@@ -876,7 +906,7 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     mpm.add(createDeadCodeEliminationPass());
     if (IGC_IS_FLAG_SET(DumpRegPressureEstimate))
       mpm.add(new IGCRegisterPressurePrinter("after_remat"));
-  } else if (ctx.m_retryManager.AllowCloneAddressArithmetic() && IGC_GET_FLAG_VALUE(RematOptionsForRetry) ||
+  } else if (ctx.m_retryManager->AllowCloneAddressArithmetic() && IGC_GET_FLAG_VALUE(RematOptionsForRetry) ||
              ctx.platform.supportsVRT() && IGC_GET_FLAG_VALUE(RematOptionsForVRT)) {
 
     if (IGC_GET_FLAG_VALUE(RematInstCombineBefore))
@@ -893,7 +923,7 @@ void AddLegalizationPasses(CodeGenContext &ctx, IGCPassManager &mpm, PSSignature
     }
 
     // if both retry and VRT checks go through, retry is more important
-    auto rematOptions = ctx.m_retryManager.AllowCloneAddressArithmetic() && IGC_GET_FLAG_VALUE(RematOptionsForRetry)
+    auto rematOptions = ctx.m_retryManager->AllowCloneAddressArithmetic() && IGC_GET_FLAG_VALUE(RematOptionsForRetry)
                             ? static_cast<IGC::REMAT_OPTIONS>(IGC_GET_FLAG_VALUE(RematOptionsForRetry))
                             : static_cast<IGC::REMAT_OPTIONS>(IGC_GET_FLAG_VALUE(RematOptionsForVRT));
 
@@ -1200,6 +1230,10 @@ void OptimizeIR(CodeGenContext *const pContext) {
     mpm.add(new llvm::TargetLibraryInfoWrapperPass(TLI));
     initializeWIAnalysisPass(*PassRegistry::getPassRegistry());
 
+    if (IGC_IS_FLAG_ENABLED(EnableSinkPointerConstAdd)) {
+      mpm.add(createSinkPointerConstAddPass());
+    }
+
        // Do inter-procedural constant propagation early.
     if (pContext->m_enableSubroutine) {
       // Here, we propagate function attributes across calls.  Remaining
@@ -1207,13 +1241,13 @@ void OptimizeIR(CodeGenContext *const pContext) {
       // in ProcessBuiltinMetaData can have that attribute stripped if
       // possible which potentially allows late stage code sinking of
       // those calls by the instruction combiner.
-      mpm.add(createPostOrderFunctionAttrsLegacyPass());
+      mpm.add(IGCLLVM::createLegacyWrappedPostOrderFunctionAttrsPass());
 
       // Don't run IPConstantProp if there are stackcalls
       const bool allowIPConstProp = !pContext->m_hasStackCalls && IGC_IS_FLAG_DISABLED(DisableIPConstantPropagation);
 
       if (allowIPConstProp) {
-        mpm.add(createIPSCCPPass());
+        mpm.add(IGCLLVM::createLegacyWrappedIPSCCPPass());
       }
       // Note / todo: LLVM < 12 also runs simple constant propagation pass
       // regardless of IPSCCP in this case. This pass is not available on
@@ -1254,7 +1288,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
     mpm.add(createScopedNoAliasAAWrapperPass());
 
     if (pContext->m_instrTypes.hasLoadStore) {
-      mpm.add(llvm::createDeadStoreEliminationPass());
+      mpm.add(IGCLLVM::createLegacyWrappedDeadStoreEliminationPass());
       mpm.add(createMarkReadOnlyLoadPass());
     }
 
@@ -1265,7 +1299,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
 
 
     if (pContext->m_instrTypes.CorrelatedValuePropagationEnable) {
-      mpm.add(llvm::createCorrelatedValuePropagationPass());
+      mpm.add(IGCLLVM::createLegacyWrappedCorrelatedValuePropagationPass());
     }
 
     mpm.add(new BreakConstantExpr());
@@ -1326,7 +1360,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
       if (pContext->m_instrTypes.numOfLoop) {
         mpm.add(createLoopDeadCodeEliminationPass());
         mpm.add(createLoopCanonicalization());
-        mpm.add(llvm::createLoopDeletionPass());
+        mpm.add(IGCLLVM::createLegacyWrappedLoopDeletionPass());
         mpm.add(llvm::createBreakCriticalEdgesPass());
         mpm.add(llvm::createLoopRotatePass(LOOP_ROTATION_HEADER_INST_THRESHOLD));
         mpm.add(llvm::createLCSSAPass());
@@ -1342,30 +1376,33 @@ void OptimizeIR(CodeGenContext *const pContext) {
 
     if (pContext->m_instrTypes.hasMultipleBB && !disableGOPT) {
       if (pContext->m_instrTypes.numOfLoop) {
-        bool allowLICM = IGC_IS_FLAG_ENABLED(allowLICM) && pContext->m_retryManager.AllowLICM();
+        bool AllowLICM = pContext->m_retryManager->AllowLICM() &&
+                         (IGC_IS_FLAG_SET(allowLICM) ? IGC_IS_FLAG_ENABLED(allowLICM)
+                                                     : pContext->getModuleMetaData()->compOpt.AllowLICM);
         bool runGEPLSR = IGC_IS_FLAG_ENABLED(EnableGEPLSR) && pContext->type == ShaderType::OPENCL_SHADER &&
-                         pContext->platform.getPlatformInfo().eProductFamily == IGFX_PVC &&
-                         !pContext->useStatelessToStateful() &&
-                         pContext->m_retryManager.IsFirstTry();
+                         (pContext->platform.getPlatformInfo().eProductFamily == IGFX_PVC ||
+                          pContext->platform.getPlatformInfo().eProductFamily == IGFX_CRI) &&
+                         !pContext->useStatelessToStateful() && !pContext->platform.hasEfficient64bEnabled() &&
+                         pContext->m_retryManager->IsFirstTry();
 
         if (runGEPLSR && IGC_IS_FLAG_DISABLED(RunGEPLSRAfterLICM)) {
-          mpm.add(createGEPLoopStrengthReductionPass(allowLICM));
+          mpm.add(createGEPLoopStrengthReductionPass(AllowLICM));
         }
 
-        if (allowLICM) {
+        if (AllowLICM) {
           mpm.add(createSpecialCasesDisableLICM());
           int licmTh = IGC_GET_FLAG_VALUE(LICMStatThreshold);
           mpm.add(new InstrStatistic(pContext, LICM_STAT, InstrStatStage::BEGIN, licmTh));
-          mpm.add(llvm::createLICMPass(100, 500, true));
+          mpm.add(IGCLLVM::createLegacyWrappedLICMPass(100, 500, true));
           mpm.add(new InstrStatistic(pContext, LICM_STAT, InstrStatStage::END, licmTh));
         }
 
         if (runGEPLSR && IGC_IS_FLAG_ENABLED(RunGEPLSRAfterLICM)) {
-          mpm.add(createGEPLoopStrengthReductionPass(allowLICM));
+          mpm.add(createGEPLoopStrengthReductionPass(AllowLICM));
         }
 
 
-        if (!pContext->m_retryManager.IsFirstTry() && pContext->type == ShaderType::OPENCL_SHADER) {
+        if (!pContext->m_retryManager->IsFirstTry() && pContext->type == ShaderType::OPENCL_SHADER) {
           mpm.add(new DisableLoopUnrollOnRetry());
         }
 
@@ -1373,7 +1410,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
         mpm.add(createIGCInstructionCombiningPass());
 
         if (IGC_IS_FLAG_ENABLED(EnableIndVarSimplification) && pContext->type == ShaderType::OPENCL_SHADER) {
-          mpm.add(llvm::createIndVarSimplifyPass());
+          mpm.add(IGCLLVM::createLegacyWrappedIndVarSimplifyPass());
         }
 
         if (IGC_IS_FLAG_ENABLED(EnableLoopHoistConstant)) {
@@ -1403,22 +1440,22 @@ void OptimizeIR(CodeGenContext *const pContext) {
             IsStage1FastestCompile(pContext->m_CgFlag, pContext->m_StagingCtx) &&
             (FastestS1Options(pContext) == FCEXP_NO_EXPRIMENT || (FastestS1Options(pContext) & FCEXP_DISABLE_UNROLL));
         if ((LoopUnrollThreshold > 0 && unroll && !disableLoopUnrollStage1) || hasIndexTemp) {
-          mpm.add(llvm::createLoopUnrollPass(2, false, false, -1, -1, -1, -1, -1, -1));
+          mpm.add(IGCLLVM::createLegacyWrappedLoopUnrollPass(2, false, false, -1, -1, -1, -1, -1, -1));
         }
 
         // Due to what looks like a bug in LICM, we need to break the LoopPassManager between
         // LoopUnroll and LICM.
         mpm.add(createBarrierNoopPass());
 
-        if (allowLICM) {
+        if (AllowLICM) {
           mpm.add(createSpecialCasesDisableLICM());
-          mpm.add(llvm::createLICMPass(100, 500, true));
+          mpm.add(IGCLLVM::createLegacyWrappedLICMPass(100, 500, true));
         }
 
         // Second unrolling with the same threshold.
         unroll = !pContext->getModuleMetaData()->compOpt.DisableLoopUnroll && IGC_IS_FLAG_DISABLED(DisableLoopUnroll);
         if (LoopUnrollThreshold > 0 && unroll) {
-          mpm.add(llvm::createLoopUnrollPass(2, false, false, -1, -1, -1, -1, -1, -1));
+          mpm.add(IGCLLVM::createLegacyWrappedLoopUnrollPass(2, false, false, -1, -1, -1, -1, -1, -1));
         }
         // Should be after LICM to accurately reason about which
         // instructions are loop-dependent or not. Needs to be before
@@ -1427,7 +1464,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
           mpm.add(createDecompose2DBlockFuncsPass());
         }
 
-        mpm.add(llvm::createLoopLoadEliminationPass());
+        mpm.add(IGCLLVM::createLegacyWrappedLoopLoadEliminationPass());
 
         if (!extensiveShader(pContext) && pContext->m_instrTypes.hasNonPrimitiveAlloca) {
           if (pContext->m_DriverInfo.NeedCountSROA()) {
@@ -1465,11 +1502,11 @@ void OptimizeIR(CodeGenContext *const pContext) {
       }
       mpm.add(createGenOptLegalizer());
 
-      mpm.add(llvm::createSCCPPass());
+      mpm.add(IGCLLVM::createLegacyWrappedSCCPPass());
 
       mpm.add(llvm::createDeadCodeEliminationPass());
       if (!extensiveShader(pContext))
-        mpm.add(llvm::createAggressiveDCEPass());
+        mpm.add(IGCLLVM::createLegacyWrappedADCEPass());
 
       mpm.add(new BreakConstantExpr());
       mpm.add(new IGCConstProp(IGC_IS_FLAG_ENABLED(EnableSimplifyGEP)));
@@ -1519,7 +1556,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
 #if LLVM_VERSION_MAJOR >= 15
         // In LLVM-12.x an extra parameter InsertFreezeWhenUnfoldingSelect = false was added
         // to JumpThreading pass, but since LLVM-15.x it was removed again.
-        mpm.add(llvm::createJumpThreadingPass(BBDuplicateThreshold));
+        mpm.add(IGCLLVM::createLegacyWrappedJumpThreadingPass(BBDuplicateThreshold));
 #else  // LLVM_VERSION_MAJOR
         mpm.add(llvm::createJumpThreadingPass(false, BBDuplicateThreshold));
 #endif // LLVM_VERSION_MAJOR
@@ -1551,7 +1588,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
       if (IGC_IS_FLAG_ENABLED(EnableJumpThreading) && !pContext->m_instrTypes.hasAtomics &&
           !extensiveShader(pContext)) {
         // After lowering 'switch', run jump threading to remove redundant jumps.
-        mpm.add(llvm::createJumpThreadingPass());
+        mpm.add(IGCLLVM::createLegacyWrappedJumpThreadingPass());
       }
 
       // run instruction combining to clean up the code after CFG optimizations
@@ -1592,7 +1629,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
                                          ( // FastestS1Options(pContext) == FCEXP_NO_EXPRIMENT ||
                                              (FastestS1Options(pContext) & FCEXP_DISABLE_UNROLL));
           if ((LoopUnrollThreshold > 0 && unroll && !disableLoopUnrollStage1) || hasIndexTemp) {
-            mpm.add(llvm::createLoopUnrollPass(2, false, false, -1, -1, -1, -1, -1, -1));
+            mpm.add(IGCLLVM::createLegacyWrappedLoopUnrollPass(2, false, false, -1, -1, -1, -1, -1, -1));
           }
         }
 
@@ -1654,8 +1691,8 @@ void OptimizeIR(CodeGenContext *const pContext) {
     mpm.add(createGenSimplificationPass());
 
     if (pContext->m_instrTypes.hasLoadStore) {
-      mpm.add(llvm::createDeadStoreEliminationPass());
-      mpm.add(llvm::createMemCpyOptPass());
+      mpm.add(IGCLLVM::createLegacyWrappedDeadStoreEliminationPass());
+      mpm.add(IGCLLVM::createLegacyWrappedMemCpyOptPass());
       mpm.add(createLdShrinkPass());
     }
 
@@ -1685,7 +1722,8 @@ void OptimizeIR(CodeGenContext *const pContext) {
       mpm.add(createIntDivConstantReductionPass());
     }
 
-    if (IGC_IS_FLAG_ENABLED(EnableIntDivRemIncrementReduction)) {
+    if (IGC_IS_FLAG_ENABLED(EnableIntDivRemIncrementReduction) &&
+        !pContext->getModuleMetaData()->compOpt.DisableIntDivRemIncrementReduction) {
       mpm.add(createIntDivRemIncrementReductionPass());
     }
     GFX_ONLY_PASS { mpm.add(createMergeMemFromBranchOptPass()); }
@@ -1695,7 +1733,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
       mpm.add(createSinkLoadOptPass());
     }
 
-    mpm.add(createConstantMergePass());
+    mpm.add(IGCLLVM::createLegacyWrappedConstantMergePass());
     GFX_ONLY_PASS { mpm.add(CreateMCSOptimization()); }
     GFX_ONLY_PASS { mpm.add(CreateGatingSimilarSamples()); }
 
@@ -1717,7 +1755,7 @@ void OptimizeIR(CodeGenContext *const pContext) {
     }
     if (IGC_IS_FLAG_ENABLED(EnableVectorizer)) {
       mpm.add(new IGCVectorizer());
-      mpm.add(llvm::createAggressiveDCEPass());
+      mpm.add(IGCLLVM::createLegacyWrappedADCEPass());
       if (IGC_IS_FLAG_ENABLED(VectorizerCheckScalarizer))
         mpm.add(createScalarizerPass(SelectiveScalarizer::Auto));
     }
