@@ -111,6 +111,7 @@ public:
   void EOTURBWrite();
   CVariable *URBFence(LSC_SCOPE scope = LSC_SCOPE_LOCAL);
   void EOTGateway(CVariable *payload = nullptr);
+  CVariable *prepareRTSurfaceStatePointer(CVariable *rtvHeapBaseAddress, uint RT_BTIndex, CVariable *rtIndexOpnd);
   void EOTRenderTarget(CVariable *r1, bool isPerCoarse);
   virtual void AddEpilogue(llvm::ReturnInst *ret);
 
@@ -224,6 +225,7 @@ public:
   CVariable *GetCE0();
   CVariable *GetDBG();
   CVariable *GetMSG0();
+  CVariable *GetSCRATCHLOC();
   CVariable *GetHWTID();
   CVariable *GetSP();
   CVariable *GetFP();
@@ -233,9 +235,12 @@ public:
   CVariable *GetARGV();
   CVariable *GetRETV();
   CVariable *GetPrivateBase();
+  CVariable *GetScratchPtr();
   CVariable *GetImplArgBufPtr();
   CVariable *GetLocalIdBufPtr();
   CVariable *GetGlobalBufferArg();
+  CVariable *GetTileID();
+  CVariable *GetEngineID();
   void SaveSRet(CVariable *sretPtr);
   CVariable *GetAndResetSRet();
 
@@ -249,7 +254,8 @@ public:
 
   void RemoveBitRange(CVariable *&src, unsigned removebit, unsigned range);
 
-  void AllocateInput(CVariable *var, uint offset, uint instance = 0, bool forceLiveOut = false);
+  void AllocateInput(CVariable *var, uint offset, uint instance = 0, bool forceLiveOut = false,
+                     bool forceOutput = false);
   void AllocateOutput(CVariable *var, uint offset, uint instance = 0);
   void AllocatePred(CVariable *var, uint offset, bool forceLiveOut = false);
   CVariable *ImmToVariable(uint64_t immediate, VISA_Type type, bool isCodePatchCandidate = false);
@@ -311,6 +317,7 @@ public:
   bool HasIndirectCalls() const { return m_HasIndirectCall; }
   bool IsIntelSymbolTableVoidProgram() const { return m_IsIntelSymbolTableVoidProgram; }
   int PrivateMemoryPerWI() const { return m_PrivateMemoryPerWI; }
+  bool TryNoScratchPointer() const { return m_TryNoScratchPointer; }
 
   IGCMD::MetaDataUtils *GetMetaDataUtils() { return m_pMdUtils; }
 
@@ -392,6 +399,7 @@ public:
   uint m_ScratchSpaceSize = 0;
 
   CVariable *m_ScratchSurfaceAddress = nullptr;
+  CVariable *m_KernelArgBufPtr = nullptr;
 
   ShaderStats *m_shaderStats = nullptr;
   GenericShaderState &m_State;
@@ -639,6 +647,7 @@ protected:
   CVariable *m_CR0 = nullptr;
   CVariable *m_CE0 = nullptr;
   CVariable *m_MSG0 = nullptr;
+  CVariable *m_SCRATCHLOC = nullptr;
   CVariable *m_DBG = nullptr;
   CVariable *m_HW_TID = nullptr;
   CVariable *m_SP = nullptr;
@@ -652,6 +661,8 @@ protected:
   CVariable *m_ImplArgBufPtr = nullptr;
   CVariable *m_LocalIdBufPtr = nullptr;
   CVariable *m_GlobalBufferArg = nullptr;
+  CVariable *m_HW_TileID = nullptr;
+  CVariable *m_HW_EngineID = nullptr;
   SProgramOutput m_simdProgram;
 
   // for each vector BCI whose uses are all extractElt with imm offset,
@@ -674,6 +685,10 @@ protected:
 
   uint32_t m_NumSampleBallotLoops = 0;
 
+  // Indicates if IGC tries to compile OCL kernel without scratch pointer.
+  // See COpenCLKernel::canSkipScratchPointer for details.
+  bool m_TryNoScratchPointer = false;
+
   DebugInfoData diData;
 
   // Program function attributes
