@@ -241,8 +241,12 @@ bool RetryManager::IsBetterThanPrevious(CShaderProgram *pCurrent, float threshol
     IGC_ASSERT(currentShader);
     IGC_ASSERT(previousShader);
 
-       // basically a small work around, if we have high spilling kernel on our hands, we are not afraid to use
-       // pre-retry shader, when we have less spills than retry one has
+    // Make sure retry manager will not select kernel without required scratch pointer.
+    if (previousShader->TryNoScratchPointer() && previousShader->m_spillSize > 0)
+      return true;
+
+    // basically a small work around, if we have high spilling kernel on our hands, we are not afraid to use
+    // pre-retry shader, when we have less spills than retry one has
     unsigned int Threshold = IGC_GET_FLAG_VALUE(RetryRevertExcessiveSpillingKernelThreshold);
     bool IsExcessiveSpillKernel = currentShader->m_spillSize >= Threshold;
     float SpillCoefficient = float(IGC_GET_FLAG_VALUE(RetryRevertExcessiveSpillingKernelCoefficient)) / 100.0f;
@@ -322,14 +326,27 @@ LLVMContextWrapper::LLVMContextWrapper(bool createResourceDimTypes) : m_UserAddr
   // This WA_OpaquePointersCL flag is related to the same flag in LLVM itself
   // We're using it here to have consistent behaviour.
   // https://github.com/llvm/llvm-project/blob/release/16.x/llvm/lib/IR/LLVMContextImpl.cpp#L50
+#if LLVM_VERSION_MAJOR < 17 || defined(IGC_LLVM_TRUNK_REVISION)
   auto WA_OpaquePointersCL = cl::getRegisteredOptions()["opaque-pointers"];
-  if (WA_OpaquePointersCL && WA_OpaquePointersCL->getNumOccurrences() > 0) {
+  auto opaquesEnabledByFlag = WA_OpaquePointersCL && WA_OpaquePointersCL->getNumOccurrences() > 0;
+#else
+  auto opaquesEnabledByFlag = true;
+#endif
+
+  if (opaquesEnabledByFlag) {
     IGC_IsPointerModeAlreadySet = true;
   }
+
   if (IGC::canOverwriteLLVMCtxPtrMode(basePtr, IGC_IsPointerModeAlreadySet)) {
     bool enableOpaquePointers = AreOpaquePointersEnabled();
     IGCLLVM::setOpaquePointers(basePtr, enableOpaquePointers);
     IGC_IsPointerModeAlreadySet = true;
+  } else {
+    // On LLVM 14 where opaques are not enabled by default via **cmakes**,
+    // the AreOpaquePointersEnabled() method wasn't aware that opaques were enabled by flag.
+    // We need to save this information in order to be consistent,
+    // otherwise AreOpaquePointersEnabled will return false even when opaques are enabled.
+    InitializeOpaquePointersSettings(opaquesEnabledByFlag);
   }
   // TODO: end
 }
@@ -480,7 +497,25 @@ unsigned int CodeGenContext::getRegisterPointerSizeInBits(unsigned int AS) const
     }
     break;
   default:
-    {
+    if (platform.hasEfficient64bEnabled()) {
+      switch (GetBufferType(AS)) {
+      case IGC::BINDLESS:
+      case IGC::BINDLESS_READONLY:
+      case IGC::BINDLESS_WRITEONLY:
+      case IGC::SSH_BINDLESS:
+      case IGC::BINDLESS_CONSTANT_BUFFER:
+      case IGC::BINDLESS_TEXTURE:
+      case IGC::BINDLESS_SAMPLER:
+      case IGC::RENDER_TARGET:
+      case IGC::SSH_BINDLESS_CONSTANT_BUFFER:
+      case IGC::SSH_BINDLESS_TEXTURE:
+        pointerSizeInRegister = 64;
+        break;
+      default:
+        pointerSizeInRegister = 32;
+        break;
+      }
+    } else {
       pointerSizeInRegister = 32;
     }
     break;
@@ -576,8 +611,8 @@ static bool isEntryPoint(const CodeGenContext *ctx, const llvm::Function *F) {
     return false;
   }
 
-  auto &FuncMD = ctx->getModuleMetaData()->FuncMD;
-  auto FuncInfo = FuncMD.find(const_cast<llvm::Function *>(F));
+  const auto &FuncMD = ctx->getModuleMetaData()->FuncMD;
+  const auto &FuncInfo = FuncMD.find(const_cast<llvm::Function *>(F));
   if (FuncInfo == FuncMD.end()) {
     return false;
   }
@@ -704,6 +739,13 @@ void CodeGenContext::EmitWarning(const char *warningstr, const llvm::Value *cont
   if (IGC_IS_FLAG_ENABLED(DisableWarnings))
     return;
 
+  if (IGC_IS_FLAG_ENABLED(DisableDuplicateWarnings)) {
+    if (m_emittedWarnings.count(warningstr)) {
+      return;
+    }
+    m_emittedWarnings.insert(warningstr);
+  }
+
   this->oclWarningMessage << "\nwarning: ";
   EmitMessage(this->oclWarningMessage, warningstr, context);
   this->oclWarningMessage << "\n";
@@ -726,15 +768,11 @@ int32_t CodeGenContext::getNumThreadsPerEU() const { return -1; }
 
 uint32_t CodeGenContext::getExpGRFSize() const { return 0; }
 
-bool CodeGenContext::allowATOB()
-{
+bool CodeGenContext::allowATOB() {
   bool allow = type == ShaderType::COMPUTE_SHADER ||
-    (type == ShaderType::OPENCL_SHADER &&
-      !(getModuleMetaData()->NBarrierCnt > 0));
+               (type == ShaderType::OPENCL_SHADER && !(getModuleMetaData()->NBarrierCnt > 0));
 
-  if (m_instrTypes.hasSplitBarrier &&
-    m_instrTypes.hasWorkgroupBarrier)
-  {
+  if (m_instrTypes.hasSplitBarrier && m_instrTypes.hasWorkgroupBarrier) {
     // The regular and split barrier cannnot share the
     // same ID, because we could have such scenario:
     // splitbarrier.signal();
