@@ -149,6 +149,9 @@ bool GenIntrinsicsTTIImpl::isGEPLoopConstDerived(GetElementPtrInst *GEP, const L
       case scSMinExpr:
       case scSequentialUMinExpr:
       case scUDivExpr:
+#if LLVM_VERSION_MAJOR > 16 && !defined(IGC_LLVM_TRUNK_REVISION)
+      case scVScale:
+#endif
         return true;
 
       case scAddRecExpr: {
@@ -194,7 +197,7 @@ void GenIntrinsicsTTIImpl::getUnrollingPreferences(Loop *L, ScalarEvolution &SE,
 
   unsigned LoopUnrollThreshold = ctx->m_DriverInfo.GetLoopUnrollThreshold();
   bool UnrollLoopForCodeSizeOnly =
-      IGC_IS_FLAG_ENABLED(UnrollLoopForCodeSizeOnly) || (!ctx->m_retryManager.IsFirstTry());
+      IGC_IS_FLAG_ENABLED(UnrollLoopForCodeSizeOnly) || (!ctx->m_retryManager->IsFirstTry());
 
   // override the LoopUnrollThreshold if the registry key is set
   if (IGC_GET_FLAG_VALUE(SetLoopUnrollThreshold) != 0) {
@@ -365,7 +368,7 @@ void GenIntrinsicsTTIImpl::getUnrollingPreferences(Loop *L, ScalarEvolution &SE,
         // Using alloca size in bytes as the threshold boost seems a bit tricky.
         unsigned AllocaSize = *(AI->getAllocationSizeInBits(DL)) / 8;
         // Assume every iteration consumes 1 DW (64 bytes).
-        if (AllocaSize/8 > UnrollMaxCountForAlloca)
+        if (AllocaSize / 8 > UnrollMaxCountForAlloca)
           continue;
 
         ThresholdBoost += AllocaSize;
@@ -377,10 +380,10 @@ void GenIntrinsicsTTIImpl::getUnrollingPreferences(Loop *L, ScalarEvolution &SE,
     if (AllocaFound) {
       UP.UpperBound = true;
       UP.Force = UnrollLoopForCodeSizeOnly ? false : true;
+      // LLVM default only to 10, boost to UnrollMaxCountForAlloca
+      UP.MaxIterationsCountToAnalyze = UnrollMaxCountForAlloca;
 
-      if (enablePromoteLoopUnrollwithAlloca()){
-        // LLVM default only to 10, boost to UnrollMaxCountForAlloca
-        UP.MaxIterationsCountToAnalyze = UnrollMaxCountForAlloca;
+      if (enablePromoteLoopUnrollwithAlloca()) {
         UP.Threshold += ThresholdBoost;
         LLVM_DEBUG(dbgs() << "Increasing L:" << L->getName() << " threshold to " << UP.Threshold
                           << " due to Alloca accessed by:");
@@ -539,6 +542,34 @@ void GenIntrinsicsTTIImpl::getUnrollingPreferences(Loop *L, ScalarEvolution &SE,
     return;
   }
 
+  auto hasCall = [](BasicBlock *BB) {
+    for (auto BI = BB->begin(), BE = BB->end(); BI != BE; ++BI)
+      if (isa<CallInst>(&*BI) && !BI->isDebugOrPseudoInst())
+        return true;
+    return false;
+  };
+
+  // 4 x LoopUnrollThreshold for 1-BB outermost loops in small OCL kernels
+  // without Call instructions, since we expect better optimization
+  // when these loops are fully unrolled.
+#if LLVM_VERSION_MAJOR < 16
+  const bool AllowSmallKernelOneBBLoopFullUnroll = !ctx->platform.isCoreXE3();
+#else
+  const bool AllowSmallKernelOneBBLoopFullUnroll = true;
+#endif
+  if (AllowSmallKernelOneBBLoopFullUnroll && ctx->type == ShaderType::OPENCL_SHADER && !hasCall(L->getHeader()) &&
+      L->getHeader()->getParent()->size() < 5) {
+    if (!L->getParentLoop() && TripCount != 0 && TripCount < 256) {
+      UP.Count = TripCount;
+      UP.MaxCount = UP.Count;
+      UP.Threshold = LoopUnrollThreshold * 4;
+      UP.Runtime = false;
+      UP.Partial = false;
+      UP.Force = true;
+      return;
+    }
+  }
+
   for (I = loopBlock->begin(); I != loopBlock->end(); I++) {
     if (const auto pIntrinsic = llvm::dyn_cast<llvm::GenIntrinsicInst>(I)) {
       if (isSendMessage(pIntrinsic)) {
@@ -711,7 +742,7 @@ unsigned getLoopSize(const Loop *L, const TargetTransformInfo &TTI) {
   InstructionCost LoopSize;
   LoopSize = Metrics.NumInsts;
 
-  LoopSize = (LoopSize > 3/*BEInsns + 1*/) ? LoopSize : 3;
+  LoopSize = (LoopSize > 3 /*BEInsns + 1*/) ? LoopSize : 3;
   return *LoopSize.getValue();
 }
 
