@@ -30,6 +30,7 @@ void initializeBlockMemOpAddrScalarizationPassPass(llvm::PassRegistry &);
 void initializeBreakConstantExprPass(llvm::PassRegistry &);
 void initializeBuiltinCallGraphAnalysisPass(llvm::PassRegistry &);
 void initializeBuiltinsConverterPass(llvm::PassRegistry &);
+void initializeCanonicalizeMulAddPass(llvm::PassRegistry &);
 void initializeCastToGASAnalysisPass(llvm::PassRegistry &);
 void initializeCastToGASInfoPass(llvm::PassRegistry &);
 void initializeCastToGASInfoWrapperPass(llvm::PassRegistry &);
@@ -156,10 +157,12 @@ void initializeScalarArgAsPointerAnalysisPass(llvm::PassRegistry &);
 void initializeScalarizeFunctionPass(llvm::PassRegistry &);
 void initializeSimd32ProfitabilityAnalysisPass(llvm::PassRegistry &);
 void initializeSetFastMathFlagsPass(llvm::PassRegistry &);
+void initializeSinkPointerConstAddPassPass(llvm::PassRegistry &);
 void initializeSPIRMetaDataTranslationPass(llvm::PassRegistry &);
 void initializeSplitStructurePhisPassPass(llvm::PassRegistry &);
 void initializeMergeScalarPhisPassPass(llvm::PassRegistry &);
 void initializeSpv2dBlockIOResolutionPass(llvm::PassRegistry &);
+void initializeSpvPredicatedIOResolutionPass(llvm::PassRegistry &);
 void initializeSpvSubgroupMMAResolutionPass(llvm::PassRegistry &);
 void initializeSplitLoadsPass(llvm::PassRegistry &);
 void initializeStatelessToStatefulPass(llvm::PassRegistry &);
@@ -181,6 +184,7 @@ void initializeRayTracingIntrinsicLoweringPassPass(llvm::PassRegistry &);
 void initializePromoteToScratchPassPass(llvm::PassRegistry &);
 void initializePrepareLoadsStoresPassPass(llvm::PassRegistry &);
 void initializeRayTracingPredicatedStackIDReleasePassPass(llvm::PassRegistry &);
+void initializeShaderExitBehaviorPassPass(llvm::PassRegistry &);
 void initializeEarlyRematPassPass(llvm::PassRegistry &);
 void initializeLateRematPassPass(llvm::PassRegistry &);
 void initializeRayInfoCSEPassPass(llvm::PassRegistry &);
@@ -279,7 +283,7 @@ void initializeLowerInvokeSIMDPass(llvm::PassRegistry &);
 void initializeRemoveCodeAssumptionsPass(llvm::PassRegistry &);
 void initializePreprocessSPVIRPass(llvm::PassRegistry &);
 void initializeConvertUserSemanticDecoratorOnFunctionsPass(llvm::PassRegistry &);
-void initializePromoteBoolsPass(llvm::PassRegistry &);
+void initializePromoteSubBytePass(llvm::PassRegistry &);
 void initializeResolveConstExprCallsPass(llvm::PassRegistry &);
 void initializeHandleSpirvDecorationMetadataPass(llvm::PassRegistry &);
 void initializeNontemporalLoadsAndStoresInAssertPass(llvm::PassRegistry &);
@@ -289,6 +293,7 @@ void initializeRayTracingBiFModuleInitializePassPass(llvm::PassRegistry &);
 void initializeRayTracingBiFModuleFinalizePassPass(llvm::PassRegistry &);
 void initializeBufferBoundsCheckingPass(llvm::PassRegistry &);
 void initializeBufferBoundsCheckingPatcherPass(llvm::PassRegistry &);
+void initializeBfloatBuiltinsResolutionPass(llvm::PassRegistry &);
 void initializeBfloatFuncsResolutionPass(llvm::PassRegistry &);
 void initializeBarrierControlFlowOptimizationPass(llvm::PassRegistry &);
 void initializeMinimumValidAddressCheckingPass(llvm::PassRegistry &);
