@@ -198,6 +198,9 @@ void Legalization::visitUnaryInstruction(UnaryInstruction &I) {
     }
     break;
   case Instruction::FNeg:
+    if (m_ctx->platform.supportsPureBF()) {
+      return;
+    }
     convertSourceToFloat = true;
     extendDestToFloat = true;
     needsTrunc = true;
@@ -249,17 +252,21 @@ void Legalization::visitUnaryInstruction(UnaryInstruction &I) {
 }
 
 void Legalization::visitBinaryOperator(llvm::BinaryOperator &I) {
-  if (I.getOpcode() == Instruction::FRem && I.getType()->isFloatTy()) {
-    Function *floorFunc = Intrinsic::getDeclaration(m_ctx->getModule(), Intrinsic::floor, I.getType());
+  if (I.getOpcode() == Instruction::FRem && (I.getType()->isFloatTy() || I.getType()->isHalfTy())) {
+    bool hasFP16Floor = !m_ctx->platform.supportFP16Rounding();
+    Type *floorType = hasFP16Floor ? I.getType() : m_builder->getFloatTy();
+    Function *floorFunc = Intrinsic::getDeclaration(m_ctx->getModule(), Intrinsic::floor, floorType);
     m_builder->SetInsertPoint(&I);
     Value *a = I.getOperand(0);
     Value *b = I.getOperand(1);
     Value *mulab = m_builder->CreateFMul(a, b);
     Value *sign = m_builder->CreateFCmpOGE(mulab, m_builder->CreateFNeg(mulab));
     Value *sel = m_builder->CreateSelect(sign, b, m_builder->CreateFNeg(b));
-    Value *selInv = m_builder->CreateFDiv(ConstantFP::get(m_builder->getFloatTy(), 1.f), sel);
+    Value *selInv = m_builder->CreateFDiv(ConstantFP::get(I.getType(), 1.f), sel);
     Value *div = m_builder->CreateFMul(a, selInv);
-    Value *floordiv = m_builder->CreateCall(floorFunc, div);
+    Value *div1 = m_builder->CreateFPExt(div, floorType);
+    Value *floordiv = m_builder->CreateCall(floorFunc, div1);
+    floordiv = m_builder->CreateFPTrunc(floordiv, div->getType());
     Value *frc = m_builder->CreateFSub(div, floordiv);
     Value *result = m_builder->CreateFMul(frc, sel);
     I.replaceAllUsesWith(result);
@@ -280,10 +287,7 @@ void Legalization::visitBinaryOperator(llvm::BinaryOperator &I) {
           break;
         }
         // check select i1 with I not used as condition
-       if (isa<SelectInst>(*U) &&
-           (U->getOperand(0) != &I ||
-            U->getOperand(1) == &I ||
-            U->getOperand(2) == &I)) {
+        if (isa<SelectInst>(*U) && (U->getOperand(0) != &I || U->getOperand(1) == &I || U->getOperand(2) == &I)) {
           flippable = false;
           break;
         }
@@ -315,10 +319,7 @@ void Legalization::visitBinaryOperator(llvm::BinaryOperator &I) {
             br->setCondition(invert);
           }
         }
-        IGC_ASSERT(
-            I.user_empty() &&
-            "Instruction should have no remaining uses after transformation"
-        );
+        IGC_ASSERT(I.user_empty() && "Instruction should have no remaining uses after transformation");
         I.eraseFromParent();
         cast<llvm::Instruction>(src0)->eraseFromParent();
         cast<llvm::Instruction>(src1)->eraseFromParent();
@@ -790,8 +791,20 @@ void Legalization::visitBitCastInst(llvm::BitCastInst &I) {
   }();
 }
 
+static bool isVectorTypeAllowed(Value *I) {
+  IGCLLVM::FixedVectorType *VecType = llvm::dyn_cast<IGCLLVM::FixedVectorType>(I->getType());
+  if (!VecType)
+    return false;
+  auto ElType = VecType->getElementType();
+  return ElType->isIntegerTy(32) || ElType->isFloatTy();
+}
+
 void Legalization::visitSelectInst(SelectInst &I) {
   m_ctx->m_instrTypes.numInsts++;
+
+  // it was vectorized by IGCVectorizer
+  auto *MD = I.getMetadata("vectorized");
+  bool ScalarizeVectorSelect = !(isVectorTypeAllowed(&I) && MD);
   if (I.getType()->isIntegerTy(1)) {
     llvm::Value *pCond = I.getOperand(0);
     llvm::Value *pSrc0 = I.getOperand(1);
@@ -836,7 +849,8 @@ void Legalization::visitSelectInst(SelectInst &I) {
 
     I.replaceAllUsesWith(newVal);
     I.eraseFromParent();
-  } else if (I.getType()->isVectorTy()) {
+    // do not scalarize vector select instruction of a specific type
+  } else if (I.getType()->isVectorTy() && ScalarizeVectorSelect) {
     unsigned int vecSize = (unsigned)cast<IGCLLVM::FixedVectorType>(I.getType())->getNumElements();
     Value *newVec = UndefValue::get(I.getType());
     m_builder->SetInsertPoint(&I);
@@ -1813,8 +1827,8 @@ void Legalization::visitIntrinsicInst(llvm::IntrinsicInst &I) {
   case Intrinsic::floor:
   case Intrinsic::ceil:
   case Intrinsic::trunc: {
-    if (!m_ctx->platform.supportFP16Rounding() && I.getType()->isHalfTy()) {
-      // On platform lacking of FP16 rounding, promote them to FP32 and
+    if (I.getType()->isBFloatTy() || (!m_ctx->platform.supportFP16Rounding() && I.getType()->isHalfTy())) {
+      // On platform lacking of FP16 rounding or for BFloat, promote them to FP32 and
       // demote back.
       Value *Val = Builder.CreateFPExt(I.getOperand(0), Builder.getFloatTy());
       Value *Callee =
