@@ -195,8 +195,8 @@ bool ModuleAllocaAnalysis::safeToUseScratchSpace() const {
     if (SeparateSpillAndScratch(&Ctx) && !supportsStatelessSpacePrivateMemory)
       simd_size = numLanes(Ctx.platform.getMinDispatchMode());
 
-    unsigned maxScratchSpaceBytes = Ctx.platform.maxPerThreadScratchSpace(
-    );
+    unsigned maxScratchSpaceBytes =
+        Ctx.platform.maxPerThreadScratchSpace(Ctx.m_DriverInfo.supports16MBPerThreadScratchSpace());
     unsigned scratchSpaceLimitPerWI = maxScratchSpaceBytes / simd_size;
     //
     // If spill happens, since the offset of scratch block rw send message
@@ -372,6 +372,19 @@ unsigned ModuleAllocaAnalysis::getTotalPrivateMemPerWI(Function *F) const {
       ret = iSTD::RoundPower2(static_cast<DWORD>(ret));
     }
 
+    if ((!modMD->compOpt.UseScratchSpacePrivateMemory || Ctx.m_DriverInfo.supports16MBPerThreadScratchSpace()) &&
+        Ctx.platform.hasEfficient64bEnabled() && Ctx.m_DriverInfo.usePrivateMemoryLimitWorkaround()) {
+      static constexpr unsigned SCRATCH_SPACE_STATELESS_PRIVATE_MEMORY_LIMIT = 0x80000000;
+      static constexpr unsigned XE3P_MAX_EU_THREAD = 4096;
+
+      // tuned to simd8 for relaxing the size threshold.
+      uint32_t privateMemoryLimitPerWI =
+          static_cast<DWORD>(SCRATCH_SPACE_STATELESS_PRIVATE_MEMORY_LIMIT / (XE3P_MAX_EU_THREAD * 8));
+
+      if (ret > privateMemoryLimitPerWI) {
+        ret = privateMemoryLimitPerWI;
+      }
+    }
     return ret;
   }
   return 0;
@@ -553,9 +566,6 @@ unsigned ModuleAllocaAnalysis::getMinSimdSize(llvm::Function *pFunc) const {
     minSimdSize = modMD.csInfo.forcedSIMDSize > 0 ? modMD.csInfo.forcedSIMDSize : minSimdSize;
     break;
   case ShaderType::PIXEL_SHADER:
-    minSimdSize = modMD.compOpt.forcePixelShaderSIMDMode > 0
-                      ? BIT(iSTD::bsf(modMD.compOpt.forcePixelShaderSIMDMode)) << 3
-                      : minSimdSize;
     break;
   case ShaderType::OPENCL_SHADER: {
     IGCMD::MetaDataUtils *pMdUtils = getAnalysis<MetaDataUtilsWrapper>().getMetaDataUtils();
