@@ -18,7 +18,6 @@ SPDX-License-Identifier: MIT
 #include "Compiler/CISACodeGen/LowerGEPForPrivMem.hpp"
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "common/LLVMWarningsPush.hpp"
-#include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/IRBuilder.h"
 #include "llvm/Transforms/Utils/Local.h"
 #include "llvm/IR/DataLayout.h"
@@ -194,7 +193,8 @@ void PrivateMemoryResolution::expandPrivateMemoryForVla(uint32_t &maxPrivateMem)
       "range [1024:20480]. "
       "Greater values can affect performance, and lower ones may lead to incorrect results of your program.\n"
       "To make sure your program runs correctly you can use IGC_StackOverflowDetection feature. See documentation:\n"
-      "https://github.com/intel/intel-graphics-compiler/tree/master/documentation/igc/StackOverflowDetection/StackOverflowDetection.md";
+      "https://github.com/intel/intel-graphics-compiler/tree/master/documentation/igc/StackOverflowDetection/"
+      "StackOverflowDetection.md";
 
   getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->EmitWarning(fullWarningMessage.c_str());
 }
@@ -370,13 +370,15 @@ bool PrivateMemoryResolution::runOnModule(llvm::Module &M) {
         maxPrivateMem = AnalyzeCGPrivateMemUsage(pKernel);
         std::string maxPrivateMemValue = std::to_string(maxPrivateMem);
         std::string fullWarningMessage =
-          "Stack call has been detected, the private memory size is set to " + maxPrivateMemValue +
-          "B. "
-          "You can change this size by setting environmental variable IGC_ForcePerThreadPrivateMemorySize to a value in "
-          "range [1024:20480]. "
-          "Greater values can affect performance, and lower ones may lead to incorrect results of your program.\n"
-          "To make sure your program runs correctly you can use StackOverflowDetection feature. See documentation:\n"
-          "https://github.com/intel/intel-graphics-compiler/tree/master/documentation/igc/StackOverflowDetection/StackOverflowDetection.md";
+            "Stack call has been detected, the private memory size is set to " + maxPrivateMemValue +
+            "B. "
+            "You can change this size by setting environmental variable IGC_ForcePerThreadPrivateMemorySize to a value "
+            "in "
+            "range [1024:20480]. "
+            "Greater values can affect performance, and lower ones may lead to incorrect results of your program.\n"
+            "To make sure your program runs correctly you can use StackOverflowDetection feature. See documentation:\n"
+            "https://github.com/intel/intel-graphics-compiler/tree/master/documentation/igc/StackOverflowDetection/"
+            "StackOverflowDetection.md";
 
         getAnalysis<CodeGenContextWrapper>().getCodeGenContext()->EmitWarning(fullWarningMessage.c_str());
       }
@@ -442,7 +444,7 @@ bool PrivateMemoryResolution::runOnModule(llvm::Module &M) {
 // [6] load j
 // [7] bar(j)
 //
-static void sinkAllocas(SmallVectorImpl<AllocaInst *> &Allocas) {
+static void sinkAllocas(SmallVectorImpl<AllocaInst *> &Allocas, bool highAllocaRAPressure) {
   IGC_ASSERT(false == Allocas.empty());
   DominatorTree DT;
   llvm::LoopInfoBase<llvm::BasicBlock, llvm::Loop> LI;
@@ -499,9 +501,12 @@ static void sinkAllocas(SmallVectorImpl<AllocaInst *> &Allocas) {
     }
 
     // Find the nearest Denominator outside loops to prevent multiple allocations
-    BasicBlock *CurBB = AI->getParent();
-    while (DomBB && DomBB != CurBB && LI.getLoopFor(DomBB) != nullptr) {
-      DomBB = DT.getNode(DomBB)->getIDom()->getBlock();
+    // In case when we have too many allocas that increase RegPressure, skip this optimization
+    if (!highAllocaRAPressure) {
+      BasicBlock *CurBB = AI->getParent();
+      while (DomBB && DomBB != CurBB && LI.getLoopFor(DomBB) != nullptr) {
+        DomBB = DT.getNode(DomBB)->getIDom()->getBlock();
+      }
     }
 
     if (DomBB) {
@@ -861,7 +866,12 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack) {
   if (Ctx.m_instrTypes.numAllocaInsts > IGC_GET_FLAG_VALUE(AllocaRAPressureThreshold)) {
     sinkAllocaSingleUse(allocaInsts);
   }
-  sinkAllocas(allocaInsts);
+
+  // if we have privateOnStack, then address rematerialization will not work for allocas, in this case
+  // we need to sink aggressively, even into loops
+  // TODO: combine this and the upper heuristic into one independent from number of allocas, but from RegPressure
+  bool highAllocaRAPressure = allocaInsts.size() > IGC_GET_FLAG_VALUE(AllocaRAPressureThreshold) && privateOnStack;
+  sinkAllocas(allocaInsts, highAllocaRAPressure);
 
   // Each AllocaInst creates a buffer and all buffers are put together
   // sequentially like the following:
@@ -990,19 +1000,20 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack) {
         Value *stackAlloca = builder.CreateCall(stackAllocaFunc, totalOffset, VALUE_NAME("stackAlloca"));
         privateBuffer =
             builder.CreatePointerCast(stackAlloca, pAI->getType(), VALUE_NAME(pAI->getName() + ".privateBuffer"));
-        auto DbgUses = llvm::FindDbgAddrUses(pAI);
-        for (auto Use : DbgUses) {
-          if (auto DbgDcl = dyn_cast_or_null<DbgDeclareInst>(Use)) {
-            unsigned scalarBufferOffset = m_ModAllocaInfo->getBufferOffset(pAI);
-            unsigned bufferSize = m_ModAllocaInfo->getBufferStride(pAI);
 
-            // Attach metadata to instruction containing offset of storage
-            auto OffsetMD =
-                MDNode::get(builder.getContext(), ConstantAsMetadata::get(builder.getInt32(scalarBufferOffset)));
-            DbgDcl->setMetadata("StorageOffset", OffsetMD);
-            auto SizeMD = MDNode::get(builder.getContext(), ConstantAsMetadata::get(builder.getInt32(bufferSize)));
-            DbgDcl->setMetadata("StorageSize", SizeMD);
-          }
+        // Attaching this metadata is crucial to both properly interpret this locations as stack based ond to inline it.
+        // Because these are stack locations we can safely inline them even with optimizations disabled (O0).
+        auto DbgDcls = llvm::FindDbgDeclareUses(pAI);
+        for (auto DbgDcl : DbgDcls) {
+          unsigned scalarBufferOffset = m_ModAllocaInfo->getBufferOffset(pAI);
+          unsigned bufferSize = m_ModAllocaInfo->getBufferStride(pAI);
+
+          // Attach metadata to instruction containing offset of storage
+          auto OffsetMD =
+              MDNode::get(builder.getContext(), ConstantAsMetadata::get(builder.getInt32(scalarBufferOffset)));
+          DbgDcl->setMetadata("StorageOffset", OffsetMD);
+          auto SizeMD = MDNode::get(builder.getContext(), ConstantAsMetadata::get(builder.getInt32(bufferSize)));
+          DbgDcl->setMetadata("StorageSize", SizeMD);
         }
       }
       // Replace all uses of original alloca with the bitcast
@@ -1091,7 +1102,8 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack) {
       builder.SetCurrentDebugLocation(pAI->getDebugLoc());
 
       // If we can use SOA layout transpose the memory
-      IGC::SOALayoutChecker SOAChecker(*pAI, Ctx.type == ShaderType::OPENCL_SHADER);
+      IGC::SOALayoutChecker SOAChecker(*pAI, Ctx.type == ShaderType::OPENCL_SHADER,
+                                       UseScratchSpacePrivateMemoryOrUseStatelessStrategy);
       IGC::SOALayoutInfo SOAInfo = SOAChecker.getOrGatherInfo();
       // TransposeMemLayout is not prepared to work on 64-bit pointers (originally, the private address space is
       // expressed by 32-bit pointers). Address space casting
@@ -1257,7 +1269,7 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack) {
     Value *bufferBase;
 
     // If we can use SOA layout transpose the memory
-    IGC::SOALayoutChecker SOAChecker(*pAI, Ctx.type == ShaderType::OPENCL_SHADER);
+    IGC::SOALayoutChecker SOAChecker(*pAI, Ctx.type == ShaderType::OPENCL_SHADER, OpenCLShaderStrategy);
     IGC::SOALayoutInfo SOAInfo = SOAChecker.getOrGatherInfo();
     if (!isUniform && SOAInfo.canUseSOALayout && SOAChecker.useNewAlgo(SOAInfo.baseType)) {
       uint32_t chunksize = SOAInfo.SOAPartitionBytes;
@@ -1289,9 +1301,12 @@ bool PrivateMemoryResolution::resolveAllocaInstructions(bool privateOnStack) {
     Value *privateBuffer =
         builder.CreatePointerCast(bufferBase, pAI->getType(), VALUE_NAME(pAI->getName() + ".privateBuffer"));
 
-    auto DbgUses = llvm::FindDbgAddrUses(pAI);
-    for (auto Use : DbgUses) {
-      if (auto DbgDcl = dyn_cast_or_null<DbgDeclareInst>(Use)) {
+    // Attaching this metadata will make this location be inlined.
+    // We can only safely inline such locations with optimizations disabled.
+    // On O2 we have no guarantee the offsets in registers are gonna be valid throughout the entire variable lifetime.
+    if (modMD->compOpt.OptDisable) {
+      auto DbgDcls = llvm::FindDbgDeclareUses(pAI);
+      for (auto DbgDcl : DbgDcls) {
         // Attach metadata to instruction containing offset of storage
         unsigned int scalarBufferOffset = m_ModAllocaInfo->getBufferOffset(pAI);
         auto OffsetMD =
