@@ -396,8 +396,7 @@ bool PreCompiledFuncImport::preProcessDouble() {
 
         handleInstrTypeChange(IEI, newInsertInst);
         toBeDeleted.push_back(IEI);
-      }
-      else if (Inst->getOpcode() == Instruction::FNeg) {
+      } else if (Inst->getOpcode() == Instruction::FNeg) {
         // check if Inst is double instruction or vector of double instructions
         Type *instType = Inst->getType();
         IGCLLVM::FixedVectorType *instVecType = dyn_cast<IGCLLVM::FixedVectorType>(instType);
@@ -753,7 +752,7 @@ bool PreCompiledFuncImport::runOnModule(Module &M) {
 
         if (isDPCallFunc) {
           // To reduce compiling time, disable retry
-          m_pCtx->m_retryManager.Disable(true);
+          m_pCtx->m_retryManager->Disable(true);
         }
       } else {
         // Add AlwaysInline attribute to force inlining all calls.
