@@ -24,6 +24,7 @@ See LICENSE.TXT for details.
 #include "llvm/ADT/DenseMap.h"
 #include <llvmWrapper/ADT/Optional.h>
 #include "llvm/ADT/SmallVector.h"
+#include <llvm/ADT/ArrayRef.h>
 #include "llvm/ADT/StringMap.h"
 #include "llvm/ADT/StringRef.h"
 #include "llvm/Config/llvm-config.h"
@@ -161,6 +162,12 @@ public:
 
   DIEBlock *getDIEBlock() { return new (DIEValueAllocator) DIEBlock(); }
 
+  /// A pair of GlobalVariable and DIExpression.
+  struct GlobalExpr {
+    const llvm::GlobalVariable *Var;
+    const llvm::DIExpression *Expr;
+  };
+
   /// insertDIE - Insert DIE into the map. We delegate the request to DwarfDebug
   /// when the llvm::MDNode can be part of the type system, since DIEs for
   /// the type system can be shared across CUs and the mappings are
@@ -236,10 +243,12 @@ public:
   /// addSourceLine - Add location information to specified debug information
   /// entry.
   void addSourceLine(DIE *Die, llvm::DIScope *S, unsigned Line);
+  void addSourceLine(DIE *Die, unsigned Line, const llvm::DIFile *File);
   void addSourceLine(DIE *Die, llvm::DIImportedEntity *IE, unsigned Line);
   void addSourceLine(DIE *Die, llvm::DIVariable *V);
   void addSourceLine(DIE *Die, llvm::DISubprogram *SP);
   void addSourceLine(DIE *Die, llvm::DIType *Ty);
+  void addSourceLine(DIE *Die, llvm::DIGlobalVariable *GV);
 
   /// addConstantValue - Add constant value entry in variable DIE.
   void addConstantValue(DIE *Die, const llvm::ConstantInt *CI, bool Unsigned);
@@ -285,22 +294,10 @@ public:
   // sampler
   void addBindlessOrStatelessLocation(DIEBlock *Block, const VISAVariableLocation &Loc, uint32_t baseAddr);
 
-  // addStatelessLocation - add a sequence of attributes to calculate stateless
-  // surface location of variable
-  void addStatelessLocation(DIEBlock *Block, const VISAVariableLocation &Loc);
-
-  // addBindlessSurfaceLocation - add a sequence of attributes to calculate
-  // bindless surface location of variable
-  void addBindlessSurfaceLocation(DIEBlock *Block, const VISAVariableLocation &Loc);
-
   // addBindlessSamplerLocation - add a sequence of attributes to calculate
   // bindless sampler location of variable
   void addBindlessSamplerLocation(DIEBlock *Block, const VISAVariableLocation &Loc);
 
-  // addBindlessScratchSpaceLocation - add a sequence of attributes to calculate
-  // bindless scratch space location of variable
-  void addBindlessScratchSpaceLocation(DIEBlock *Block, const VISAVariableLocation &Loc);
-
   // addBE_FP - emits operations to add contents of BE_FP to current top of
   // dwarf stack
   void addBE_FP(IGC::DIEBlock *Block);
@@ -331,20 +328,15 @@ public:
   // emit expression to extract sub-reg value
   void extractSubRegValue(IGC::DIEBlock *Block, unsigned char Sz);
 
-  // Decode line number, file name and location from a string, where a line no.
-  // and file name (including directory) are separated by '-' character:
-  // lineNumber-fileNameIncludingDirectory There is a workaround for DIModule
-  // creation in earlier LLVM versions, where a line and a file parameters are
-  // not supported in DIBuilder.
-  void decodeLineAndFileForISysRoot(llvm::StringRef &lineAndFile, unsigned int *line, std::string *file,
-                                    std::string *directory);
-
   /// Construct import_module DIE.
   IGC::DIE *constructImportedEntityDIE(llvm::DIImportedEntity *Module);
 
   /// getOrCreateNameSpace - Create a DIE for DINameSpace.
   DIE *getOrCreateNameSpace(llvm::DINamespace *NS);
 
+  /// Get or create global variable DIE.
+  DIE *getOrCreateGlobalVariableDIE(llvm::DIGlobalVariable *GV, llvm::ArrayRef<GlobalExpr> GlobalExprs);
+
   /// getOrCreateSubprogramDIE - Create new DIE using SP.
   DIE *getOrCreateSubprogramDIE(llvm::DISubprogram *SP);
 
@@ -365,6 +357,9 @@ public:
   /// constructVariableDIE - Construct a DIE for the given DbgVariable.
   DIE *constructVariableDIE(DbgVariable &DV, bool isScopeAbstract);
 
+  /// Apply attributes for DIE created in constructVariableDIE
+  void applyVariableAttributes(DbgVariable &DV, DIE *VariableDie, bool isScopeAbstract, DIE *AbsDIE = nullptr);
+
   /// Create a DIE with the given Tag, add the DIE to its parent, and
   /// call insertDIE if MD is not null.
   DIE *createAndAddDIE(unsigned Tag, DIE &Parent, llvm::DINode *N = nullptr);
