@@ -33,6 +33,7 @@ See LICENSE.TXT for details.
 #include "llvm/MC/MCSymbol.h"
 #include "llvm/MC/MCValue.h"
 #include "llvm/Support/SourceMgr.h"
+#include "llvm/Support/MD5.h"
 #include "llvmWrapper/ADT/Optional.h"
 #include "llvmWrapper/Support/TargetRegistry.h"
 #include "common/LLVMWarningsPop.hpp"
@@ -306,30 +307,30 @@ public:
   }
 
   bool writeNopData(raw_ostream &OS, uint64_t Count, const MCSubtargetInfo *STI) const override {
-      const char nop = (char)0x90;
-  for (uint64_t i = 0; i < Count; ++i) {
-    OS.write(&nop, 1);
+    const char nop = (char)0x90;
+    for (uint64_t i = 0; i < Count; ++i) {
+      OS.write(&nop, 1);
+    }
+    return true;
   }
-  return true;
-}
 
-/// createObjectWriter - Create a new MCObjectWriter instance for use by the
-/// assembler backend to emit the final object file.
-std::unique_ptr<MCObjectWriter> createObjectWriter(llvm::raw_pwrite_stream &os) const {
-  Triple triple(m_targetTriple);
-  uint8_t osABI = MCELFObjectTargetWriter::getOSABI(triple.getOS());
-  uint16_t eMachine = m_is64Bit ? ELF::EM_X86_64 : ELF::EM_386;
-  // Only i386 uses Rel instead of RelA.
-  bool hasRelocationAddend = eMachine != ELF::EM_386;
-  std::unique_ptr<MCELFObjectTargetWriter> pMOTW =
-      IGCLLVM::make_unique<VISAELFObjectWriter>(m_is64Bit, osABI, eMachine, hasRelocationAddend);
-  return createELFObjectWriter(std::move(pMOTW), os, /*IsLittleEndian=*/true);
-}
+  /// createObjectWriter - Create a new MCObjectWriter instance for use by the
+  /// assembler backend to emit the final object file.
+  std::unique_ptr<MCObjectWriter> createObjectWriter(llvm::raw_pwrite_stream &os) const {
+    Triple triple(m_targetTriple);
+    uint8_t osABI = MCELFObjectTargetWriter::getOSABI(triple.getOS());
+    uint16_t eMachine = m_is64Bit ? ELF::EM_X86_64 : ELF::EM_386;
+    // Only i386 uses Rel instead of RelA.
+    bool hasRelocationAddend = eMachine != ELF::EM_386;
+    std::unique_ptr<MCELFObjectTargetWriter> pMOTW =
+        IGCLLVM::make_unique<VISAELFObjectWriter>(m_is64Bit, osABI, eMachine, hasRelocationAddend);
+    return createELFObjectWriter(std::move(pMOTW), os, /*IsLittleEndian=*/true);
+  }
 
-std::unique_ptr<MCObjectTargetWriter> createObjectTargetWriter() const override {
-  // TODO: implement this
-  IGC_ASSERT_UNREACHABLE(); // Unimplemented
-}
+  std::unique_ptr<MCObjectTargetWriter> createObjectTargetWriter() const override {
+    // TODO: implement this
+    IGC_ASSERT_UNREACHABLE(); // Unimplemented
+  }
 }; // namespace IGC
 
 class VISAMCCodeEmitter : public MCCodeEmitter {
@@ -458,9 +459,9 @@ unsigned StreamEmitter::GetDwarfCompileUnitID() const { return m_pContext->getDw
 
 void StreamEmitter::SetDwarfCompileUnitID(unsigned cuIndex) const { m_pContext->setDwarfCompileUnitID(cuIndex); }
 
-void StreamEmitter::EmitBytes(StringRef data, unsigned addrSpace) const {
-  m_pMCStreamer->emitBytes(data);
-}
+void StreamEmitter::SetDwarfVersion(unsigned DwarfVersion) const { m_pContext->setDwarfVersion(DwarfVersion); }
+
+void StreamEmitter::EmitBytes(StringRef data, unsigned addrSpace) const { m_pMCStreamer->emitBytes(data); }
 
 void StreamEmitter::EmitValue(const MCExpr *value, unsigned size, unsigned addrSpace) const {
 
@@ -485,9 +486,7 @@ void StreamEmitter::EmitULEB128(uint64_t value, llvm::StringRef /*desc*/, unsign
   m_pMCStreamer->emitULEB128IntValue(value);
 }
 
-void StreamEmitter::EmitLabel(MCSymbol *pLabel) const {
-  m_pMCStreamer->emitLabel(pLabel);
-}
+void StreamEmitter::EmitLabel(MCSymbol *pLabel) const { m_pMCStreamer->emitLabel(pLabel); }
 
 void StreamEmitter::EmitLabelDifference(const MCSymbol *pHi, const MCSymbol *pLo, unsigned size) const {
   const MCExpr *hiExpr = MCSymbolRefExpr::create(pHi, *m_pContext);
@@ -608,12 +607,19 @@ void StreamEmitter::EmitDwarfRegOp(unsigned reg, unsigned offset, bool indirect)
   }
 }
 
-bool StreamEmitter::EmitDwarfFileDirective(unsigned fileNo, StringRef directory, StringRef filename,
-                                           unsigned cuID) const {
-  auto result = m_pMCStreamer->emitDwarfFileDirective(fileNo, directory, filename, {}, {}, cuID) != 0;
+bool StreamEmitter::EmitDwarfFileDirective(unsigned fileNo, llvm::StringRef directory, llvm::StringRef filename,
+                                           IGCLLVM::optional<llvm::MD5::MD5Result> checksum,
+                                           IGCLLVM::optional<llvm::StringRef> source, unsigned cuID) const {
+  auto result = m_pMCStreamer->emitDwarfFileDirective(fileNo, directory, filename, checksum, source, cuID) != 0;
   return result;
 }
 
+void StreamEmitter::EmitDwarfFile0Directive(unsigned fileNo, StringRef directory, StringRef filename,
+                                            IGCLLVM::optional<llvm::MD5::MD5Result> checksum,
+                                            IGCLLVM::optional<llvm::StringRef> source, unsigned cuID) const {
+  m_pMCStreamer->emitDwarfFile0Directive(directory, filename, checksum, source, cuID);
+}
+
 void StreamEmitter::EmitDwarfLocDirective(unsigned fileNo, unsigned line, unsigned column, unsigned flags, unsigned isa,
                                           unsigned discriminator, StringRef fileName) const {
   m_pMCStreamer->emitDwarfLocDirective(fileNo, line, column, flags, isa, discriminator, fileName);
@@ -681,8 +687,7 @@ void StreamEmitter::verifyRegisterLocationExpr(const DbgVariable &DV, const Dwar
   DiagnosticBuff Diag;
   if (!DV.currentLocationIsImplicit() && !DV.currentLocationIsSimpleIndirectValue()) {
     if (DbgInst->getExpression()->isComplex()) {
-      Diag.out() << "ValidationFailure [UnexpectedComlexExpression]"
-                 << " for a simple register location\n";
+      Diag.out() << "ValidationFailure [UnexpectedComlexExpression]" << " for a simple register location\n";
     }
   }
   verificationReport(DV, Diag);
