@@ -50,6 +50,7 @@ See LICENSE.TXT for details.
 #include "llvmWrapper/Support/Alignment.h"
 #include "llvmWrapper/IR/Type.h"
 #include "usc_gen7_types.h"
+#include "visa/include/visa_igc_common_header.h"
 
 namespace IGC {
 enum CallableShaderTypeMD : uint32_t;
@@ -111,6 +112,10 @@ public:
   Value *getAlpha() const { return getOperand(operandOffset(6)); }
   Value *getDepth() const { return getOperand(operandOffset(7)); }
   Value *getStencil() const { return getOperand(operandOffset(8)); }
+  Value *getAddress() const {
+    IGC_ASSERT(!isRegular());
+    return getOperand(9);
+  }
   Value *getRTIndex() const { return getOperand(operandOffset(9)); }
   Value *getBlendStateIndex() const { return getOperand(operandOffset(10)); }
   bool hasMask() const { return valueToImm64(getOperand(operandOffset(11))) != 0; }
@@ -128,6 +133,7 @@ public:
   static bool classof(const GenIntrinsicInst *I) {
     switch (I->getIntrinsicID()) {
     case GenISAIntrinsic::GenISA_RTWrite:
+    case GenISAIntrinsic::GenISA_RTWritePtr:
       return true;
     default:
       return false;
@@ -139,6 +145,7 @@ public:
     switch (getIntrinsicID()) {
     case GenISAIntrinsic::GenISA_RTWrite:
       return true;
+    case GenISAIntrinsic::GenISA_RTWritePtr:
     default:
       break;
     }
@@ -169,6 +176,10 @@ public:
   Value *getAlpha1() const { return getOperand(operandOffset(9)); }
   Value *getDepth() const { return getOperand(operandOffset(10)); }
   Value *getStencil() const { return getOperand(operandOffset(11)); }
+  Value *getAddress() const {
+    IGC_ASSERT(!isRegular());
+    return getOperand(12);
+  }
   Value *getRTIndex() const { return getOperand(operandOffset(12)); }
   bool hasMask() const { return valueToImm64(getOperand(operandOffset(13))) != 0; }
   bool hasDepth() const { return valueToImm64(getOperand(operandOffset(14))) != 0; }
@@ -183,6 +194,7 @@ public:
   static bool classof(const GenIntrinsicInst *I) {
     switch (I->getIntrinsicID()) {
     case GenISAIntrinsic::GenISA_RTDualBlendSource:
+    case GenISAIntrinsic::GenISA_RTDualBlendSourcePtr:
       return true;
     default:
       return false;
@@ -195,6 +207,7 @@ private:
     switch (getIntrinsicID()) {
     case GenISAIntrinsic::GenISA_RTDualBlendSource:
       return true;
+    case GenISAIntrinsic::GenISA_RTDualBlendSourcePtr:
     default:
       break;
     }
@@ -210,10 +223,15 @@ protected:
   void setOperand(unsigned i, Value *v) { return GenIntrinsicInst::setOperand(i, v); }
 
 public:
+  Value *getAddress() const {
+    IGC_ASSERT(!isRegular());
+    return getOperand(0);
+  }
   Value *getRTIndex() const { return getOperand(operandOffset(0)); }
   bool hasSampleIndex() const {
     switch (getIntrinsicID()) {
     case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreq:
+    case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreqPtr:
       return true;
     default:
       break;
@@ -247,6 +265,8 @@ public:
     switch (I->getIntrinsicID()) {
     case GenISAIntrinsic::GenISA_RenderTargetRead:
     case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreq:
+    case GenISAIntrinsic::GenISA_RenderTargetReadPtr:
+    case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreqPtr:
       return true;
     default:
       return false;
@@ -260,6 +280,8 @@ private:
     case GenISAIntrinsic::GenISA_RenderTargetRead:
     case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreq:
       return true;
+    case GenISAIntrinsic::GenISA_RenderTargetReadPtr:
+    case GenISAIntrinsic::GenISA_RenderTargetReadSampleFreqPtr:
     default:
       break;
     }
@@ -437,9 +459,17 @@ public:
       return UINT_MAX;
     }
   }
-  inline unsigned int getTextureIndex() const { return getNumOperands() - 5; }
+  inline unsigned int getNumNormalOperands() const {
+    switch (getIntrinsicID()) {
+    default:
+      return getNumOperands();
+    }
+  }
+  inline unsigned int getTextureIndex() const { return getNumNormalOperands() - 5; }
   inline unsigned int getImmediateOffsetsIndex() const { return 6; }
-  inline unsigned int getPairedTextureIndex() const { return hasPairedTextureArg() ? getNumOperands() - 6 : UINT_MAX; }
+  inline unsigned int getPairedTextureIndex() const {
+    return hasPairedTextureArg() ? getNumNormalOperands() - 6 : UINT_MAX;
+  }
   inline Value *getTextureValue() const { return getOperand(getTextureIndex()); }
   inline Value *getPairedTextureValue() const {
     return hasPairedTextureArg() ? getOperand(getPairedTextureIndex()) : nullptr;
@@ -491,10 +521,16 @@ public:
 
 class LdMSIntrinsic : public SamplerLoadIntrinsic {
 public:
-  inline Value *getImmOffset(unsigned int i) { return getOperand(arg_size() - 3 + i); }
-  inline void setImmOffset(unsigned int i, Value *val) { return setOperand(arg_size() - 3 + i, val); }
-  inline Value *getCoordinate(unsigned int i) { return getOperand(arg_size() - 8 + i); }
-  inline void setCoordinate(unsigned int i, Value *val) { return setOperand(arg_size() - 8 + i, val); }
+  inline unsigned int getRegularArgsSize() const {
+    switch (getIntrinsicID()) {
+    default:
+      return arg_size();
+    }
+  }
+  inline Value *getImmOffset(unsigned int i) { return getOperand(getRegularArgsSize() - 3 + i); }
+  inline void setImmOffset(unsigned int i, Value *val) { return setOperand(getRegularArgsSize() - 3 + i, val); }
+  inline Value *getCoordinate(unsigned int i) { return getOperand(getRegularArgsSize() - 8 + i); }
+  inline void setCoordinate(unsigned int i, Value *val) { return setOperand(getRegularArgsSize() - 8 + i, val); }
   static inline bool classof(const GenIntrinsicInst *I) {
     switch (I->getIntrinsicID()) {
     case GenISAIntrinsic::GenISA_ldmsptr:
@@ -552,6 +588,8 @@ public:
     case GenISAIntrinsic::GenISA_sampleDCMlodptr:
     case GenISAIntrinsic::GenISA_samplePOCptr:
     case GenISAIntrinsic::GenISA_samplePOLCptr:
+    case GenISAIntrinsic::GenISA_samplePOBCptr:
+    case GenISAIntrinsic::GenISA_samplePODCptr:
       return true;
     default:
       break;
@@ -565,6 +603,7 @@ public:
     case GenISAIntrinsic::GenISA_sampleBCptr:
     case GenISAIntrinsic::GenISA_sampleBCMlodptr:
     case GenISAIntrinsic::GenISA_samplePOBptr:
+    case GenISAIntrinsic::GenISA_samplePOBCptr:
       return true;
     default:
       break;
@@ -716,6 +755,7 @@ public:
     case GenISAIntrinsic::GenISA_sampleBCMlodptr:
     case GenISAIntrinsic::GenISA_samplePOptr:
     case GenISAIntrinsic::GenISA_samplePOBptr:
+    case GenISAIntrinsic::GenISA_samplePOBCptr:
     case GenISAIntrinsic::GenISA_samplePOCptr:
     case GenISAIntrinsic::GenISA_lodptr:
       return true;
@@ -748,6 +788,8 @@ public:
     case GenISAIntrinsic::GenISA_samplePOCptr:
     case GenISAIntrinsic::GenISA_samplePODptr:
     case GenISAIntrinsic::GenISA_samplePOLCptr:
+    case GenISAIntrinsic::GenISA_samplePOBCptr:
+    case GenISAIntrinsic::GenISA_samplePODCptr:
       return true;
     default:
       return false;
@@ -776,6 +818,8 @@ public:
     case GenISAIntrinsic::GenISA_samplePOCptr:
     case GenISAIntrinsic::GenISA_samplePODptr:
     case GenISAIntrinsic::GenISA_samplePOLCptr:
+    case GenISAIntrinsic::GenISA_samplePOBCptr:
+    case GenISAIntrinsic::GenISA_samplePODCptr:
       return true;
     default:
       break;
@@ -1205,6 +1249,7 @@ public:
   Value *getPayload() const { return getOperand(16); }
   Value *getComparisonValue() const { return getOperand(17); }
   Value *getInternalRayFlags() const { return getOperand(18); }
+  Value *getExtraPipelineFlags() const { return getOperand(19); }
 };
 
 class CallShaderHLIntrinsic : public ContinuationHLIntrinsic {
@@ -1295,6 +1340,12 @@ public:
 
   static inline bool classof(const Value *V) { return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V)); }
 
+  // Getters used in Efficient64b version
+  STACK_ADDRESS_MODE getStackAddressingMode() const {
+    return static_cast<STACK_ADDRESS_MODE>(valueToImm32(getOperand(2)));
+  }
+
+  void setStackAddressingMode(ConstantInt *v) { setOperand(2, v); }
 };
 
 class BTDIntrinsic : public GenIntrinsicInst {
@@ -1312,6 +1363,7 @@ public:
   Value *getShaderRecordAddress() const { return getOperand(2); }
 };
 
+
 class StackIDReleaseIntrinsic : public GenIntrinsicInst {
 public:
   // Methods for support type inquiry through isa, cast, and dyn_cast:
@@ -1431,6 +1483,17 @@ public:
   static inline bool classof(const Value *V) { return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V)); }
 };
 
+class KSPPointerIntrinsic : public GenIntrinsicInst {
+public:
+  // Methods for support type inquiry through isa, cast, and dyn_cast:
+  static inline bool classof(const GenIntrinsicInst *I) {
+    GenISAIntrinsic::ID ID = I->getIntrinsicID();
+    return ID == GenISAIntrinsic::GenISA_KSPPointer;
+  }
+
+  static inline bool classof(const Value *V) { return isa<GenIntrinsicInst>(V) && classof(cast<GenIntrinsicInst>(V)); }
+};
+
 class GlobalBufferPointerIntrinsic : public GenIntrinsicInst {
 public:
   // Methods for support type inquiry through isa, cast, and dyn_cast:
