@@ -10,6 +10,7 @@ SPDX-License-Identifier: MIT
 #define _BUILTINSFRONTEND_H_
 
 #include "common/LLVMWarningsPush.hpp"
+#include "llvm/ADT/SmallVector.h"
 #include <llvm/IR/Module.h>
 #include <llvm/IR/LLVMContext.h>
 #include <llvm/IR/Constants.h>
@@ -154,8 +155,7 @@ public:
   }
 };
 
-template <typename T = llvm::ConstantFolder,
-          typename InserterTy = llvm::IRBuilderDefaultInserter>
+template <typename T = llvm::ConstantFolder, typename InserterTy = llvm::IRBuilderDefaultInserter>
 class LLVM3DBuilder : public llvm::IGCIRBuilder<T, InserterTy> {
 public:
   LLVM3DBuilder(llvm::LLVMContext &pCtx, const PLATFORM &pPlatform)
@@ -617,7 +617,12 @@ public:
                                       llvm::Value *helperLaneMode = nullptr);
   llvm::Value *create_waveMultiPrefixBitCount(llvm::Instruction *I, llvm::Value *Val, llvm::Value *Mask,
                                               llvm::Value *helperLaneMode = nullptr);
+  llvm::Value *create_waveClusteredAll(llvm::Value *src, llvm::Value *reductionType, llvm::Value *clusterSize,
+                                       llvm::Value *helperLaneMode = nullptr);
+  llvm::Value *create_waveClusteredBroadcast(llvm::Value *src, llvm::Value *clusterLane, llvm::Value *clusterSize,
+                                             llvm::Value *helperLaneMode = nullptr);
   llvm::Value *create_quadPrefix(llvm::Value *src, llvm::Value *type, bool inclusive = false);
+  llvm::Value *get16BitLaneID();
   llvm::Value *get32BitLaneID();
   llvm::Value *getSimdSize();
   llvm::Value *getFirstLaneID(llvm::Value *helperLaneMode = nullptr);
