@@ -27,6 +27,7 @@ SPDX-License-Identifier: MIT
 
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include <llvmWrapper/ADT/Optional.h>
+#include "llvmWrapper/Support/MathExtras.h"
 
 #include "Probe/Assertion.h"
 
@@ -47,7 +48,7 @@ class Bale;
 // Utility function to get the integral log base 2 of an integer, or -1 if
 // the input is not a power of 2.
 inline int exactLog2(unsigned Val) {
-  unsigned CLZ = llvm::countLeadingZeros(Val);
+  unsigned CLZ = IGCLLVM::countl_zero(Val);
   if (CLZ != 32 && 1U << (31 - CLZ) == Val)
     return 31 - CLZ;
   return -1;
@@ -58,7 +59,7 @@ inline int exactLog2(unsigned Val) {
 template <typename T> inline int log2(T Val) {
   if (Val <= 0)
     return -1;
-  unsigned CLZ = llvm::countLeadingZeros<uint32_t>(Val);
+  unsigned CLZ = IGCLLVM::countl_zero<uint32_t>(Val);
   IGC_ASSERT_EXIT(CLZ < 32);
   return 31 - CLZ;
 }
