@@ -36,6 +36,7 @@ SPDX-License-Identifier: MIT
 
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/Support/TypeSize.h"
+#include "llvmWrapper/Support/MathExtras.h"
 
 using namespace llvm;
 using namespace genx;
@@ -345,6 +346,8 @@ unsigned genx::getLegalRegionSizeForTarget(const GenXSubtarget &ST,
   IGC_ASSERT_EXIT(GRFByteSize > 0);
   unsigned LogGRFWidth = genx::log2(GRFByteSize);
   unsigned NumGRF = 2;
+  if (ST.hasEfficientSIMD32() && R.Width == 32u && R.ElementBytes == 8u)
+    NumGRF = 4;
 
   if ((!R.Stride || exactLog2(R.Stride) >= 0) &&
       (Allow2D || R.Stride <= MaxStride)) {
@@ -970,7 +973,7 @@ convertRegionInstType(Instruction *Inst, Type *NewScalarTy,
     return std::nullopt;
   // Transformation is not profitable for 2D regions or if it will require
   // legalization.
-  if (R.is2D() || R.NumElements > llvm::PowerOf2Floor(
+  if (R.is2D() || R.NumElements > IGCLLVM::bit_floor(
                                       genx::getExecSizeAllowedBits(Inst, &ST)))
     return std::nullopt;
   return std::make_pair(NewVecTy, R);
@@ -1136,6 +1139,11 @@ Value *llvm::genx::simplifyRegionInst(Instruction *Inst, const DataLayout *DL,
   if (Inst->use_empty())
     return nullptr;
 
+  bool CanSimplify = llvm::none_of(
+      Inst->users(), vc::InternalIntrinsic::isPacked4BitUpconvertLut);
+  if (!CanSimplify)
+    return nullptr;
+
   Value *NewVal = nullptr;
 
   unsigned ID = GenXIntrinsic::getGenXIntrinsicID(Inst);
