@@ -272,10 +272,8 @@ static Value *propagateShuffleVector(ShuffleVectorInst *Shuffle) {
     }
   }
 
-  LLVM_DEBUG(dbgs() << "Combined mask: "; for (int Val
-                                               : CombinedMask) dbgs()
-                                          << Val << " ";
-             dbgs() << "\n");
+  LLVM_DEBUG(dbgs() << "Combined mask: ";
+             for (int Val : CombinedMask) dbgs() << Val << " "; dbgs() << "\n");
 
   // Create the final shuffle vector with the combined mask.
   IRBuilder<> Builder(Shuffle);
@@ -380,10 +378,8 @@ static Value *simplifyShuffleVectorChain(ShuffleVectorInst *Shuffle) {
     }
   }
 
-  LLVM_DEBUG(dbgs() << "Combined mask: "; for (int Val
-                                               : CombinedMask) dbgs()
-                                          << Val << " ";
-             dbgs() << "\n");
+  LLVM_DEBUG(dbgs() << "Combined mask: ";
+             for (int Val : CombinedMask) dbgs() << Val << " "; dbgs() << "\n");
 
   // Create the final shuffle vector with the combined mask.
   IRBuilder<> Builder(Shuffle);
@@ -569,6 +565,11 @@ Value *llvm::SimplifyGenXIntrinsic(CallInst *CI, const DataLayout &DL) {
   switch (IID) {
   case GenXIntrinsic::genx_rdregioni:
   case GenXIntrinsic::genx_rdregionf: {
+    bool CanSimplify = llvm::none_of(
+        CI->users(), vc::InternalIntrinsic::isPacked4BitUpconvertLut);
+    if (!CanSimplify)
+      break;
+
     // Identity rdregion can be simplified to its "old value" input.
     if (RetTy ==
         Args[GenXIntrinsic::GenXRegion::OldValueOperandNum]->getType()) {
