@@ -23,6 +23,7 @@ SPDX-License-Identifier: MIT
 #include <llvmWrapper/IR/BasicBlock.h>
 #include "llvmWrapper/IR/DerivedTypes.h"
 #include "llvmWrapper/IR/Instructions.h"
+#include "llvmWrapper/IR/Type.h"
 #include "llvmWrapper/Support/Alignment.h"
 #include "llvmWrapper/Transforms/Utils/Cloning.h"
 
@@ -272,7 +273,7 @@ Function *GenXPacketize::vectorizeSIMTFunction(Function *F, unsigned Width) {
   std::vector<Type *> ArgTypes;
   for (const Argument &Arg : F->args()) {
     auto *ArgTy = Arg.getType();
-    if (UniformArgs.count(&Arg) || ArgTy->isOpaquePointerTy())
+    if (UniformArgs.count(&Arg) || IGCLLVM::isPointerTy(ArgTy))
       ArgTypes.push_back(ArgTy);
     else if (ArgTy->isPointerTy()) {
       // FIXME: check the pointer defined by an argument or an alloca
@@ -577,7 +578,7 @@ bool GenXPacketize::isUniformIntrinsic(unsigned ID) {
 
 void GenXPacketize::findUniformInsts(Function &F) {
   // global variable load is uniform
-  for (auto &Global : M->getGlobalList()) {
+  for (auto &Global : M->globals()) {
     for (auto UI = Global.use_begin(), UE = Global.use_end(); UI != UE; ++UI) {
       if (auto *LD = dyn_cast<LoadInst>(UI->getUser())) {
         UniformInsts.insert(LD);
@@ -617,6 +618,7 @@ void GenXPacketize::findUniformInsts(Function &F) {
       const Value *Use = (*UI);
       if (auto *LI = dyn_cast<LoadInst>(Use)) {
         UniformInsts.insert(LI);
+        UVSet.push((Value *)LI);
       } else if (auto *GEP = dyn_cast<GetElementPtrInst>(Use)) {
         if (GEP->getPointerOperand() == Def) {
           UniformInsts.insert(GEP);
@@ -865,7 +867,7 @@ Value *GenXPacketize::packetizeLLVMInstruction(Instruction *Inst) {
         uint32_t numElems =
             cast<IGCLLVM::FixedVectorType>(PacketizedSrcTy)->getNumElements();
         ReturnTy = IGCLLVM::FixedVectorType::get(Inst->getType(), numElems);
-      } else if (!Inst->getType()->isOpaquePointerTy()) {
+      } else if (!IGCLLVM::isPointerTy(Inst->getType())) {
         // <N x Ty>*
         auto *DstScalarTy = IGCLLVM::getNonOpaquePtrEltTy(Inst->getType());
         if (VectorType::isValidElementType(DstScalarTy))
@@ -1088,7 +1090,7 @@ Value *GenXPacketize::packetizeLLVMInstruction(Instruction *Inst) {
     auto *TrueSrc = getPacketizeValue(Inst->getOperand(1));
     auto *FalseSrc = getPacketizeValue(Inst->getOperand(2));
     if (!TrueSrc->getType()->isPointerTy() ||
-        TrueSrc->getType()->isOpaquePointerTy()) {
+        IGCLLVM::isPointerTy(TrueSrc->getType())) {
       // simple select packetization
       ReplacedInst = B->SELECT(VecCond, TrueSrc, FalseSrc);
     } else {
@@ -1167,6 +1169,7 @@ Value *GenXPacketize::packetizeGenXIntrinsic(Instruction *Inst) {
       case GenXIntrinsic::genx_usdp4a_sat:
       case GenXIntrinsic::genx_uudp4a_sat:
       case GenXIntrinsic::genx_dpas:
+      case GenXIntrinsic::genx_bdpas:
       case GenXIntrinsic::genx_dpas2:
       case GenXIntrinsic::genx_dpasw:
       case GenXIntrinsic::genx_dpas_nosrc0:
@@ -1755,7 +1758,7 @@ void GenXPacketize::fixupLLVMIntrinsics(Function &F) {
 /// CM SIMD control-flow representation after packetization
 GlobalVariable *GenXPacketize::findGlobalExecMask() {
   // look for the global EMask variable if exists
-  for (auto &Global : M->getGlobalList()) {
+  for (auto &Global : M->globals()) {
     auto *Ty = Global.getValueType();
     if (Ty->isVectorTy() &&
         cast<IGCLLVM::FixedVectorType>(Ty)->getNumElements() ==
