@@ -28,6 +28,7 @@ SPDX-License-Identifier: MIT
 
 #include "llvmWrapper/IR/DerivedTypes.h"
 
+#include <llvm/ADT/SmallString.h>
 #include <llvm/IR/Function.h>
 #include <llvm/IR/IRBuilder.h>
 #include <llvm/IR/InstVisitor.h>
@@ -208,6 +209,8 @@ Value *SPIRVExpander::visitCallInst(CallInst &CI) {
 
   auto IID = StringSwitch<unsigned>(CalleeName)
                  .StartsWith("BitCount", Intrinsic::ctpop)
+                 // SPV_INTEL_sigmoid extension.
+                 .StartsWith("FSigmoidINTEL", vc::InternalIntrinsic::sigmoid)
                  .Default(Intrinsic::not_intrinsic);
 
   if (IID != Intrinsic::not_intrinsic) {
@@ -308,6 +311,7 @@ Value *SPIRVExpander::visitCallInst(CallInst &CI) {
             .StartsWith("mad", Intrinsic::fmuladd)
             .StartsWith("sqrt", Intrinsic::sqrt)
             .StartsWith("rsqrt", GenXIntrinsic::genx_rsqrt)
+            .StartsWith("tanh", vc::InternalIntrinsic::tanh)
             .Default(Intrinsic::not_intrinsic);
 
   if (IID != Intrinsic::not_intrinsic) {
