@@ -12,6 +12,7 @@ SPDX-License-Identifier: MIT
 #include "llvmWrapper/IR/CallSite.h"
 #include "llvmWrapper/IR/Function.h"
 #include "llvmWrapper/IR/Instructions.h"
+#include "llvmWrapper/IR/DerivedTypes.h"
 
 #include "Probe/Assertion.h"
 
@@ -148,7 +149,7 @@ static bool structSafeToPassByVal(const Argument &Arg, StructType *StrTy) {
 
 static Type *getPtrArgElementType(const Argument &PtrArg) {
   auto *PtrArgTy = cast<PointerType>(PtrArg.getType());
-  if (!PtrArgTy->isOpaque())
+  if (!IGCLLVM::isOpaque(PtrArgTy))
     return IGCLLVM::getNonOpaquePtrEltTy(PtrArgTy);
   if (auto *ByValTy = PtrArg.getParamByValType())
     return ByValTy;
@@ -594,8 +595,7 @@ getTransformedFuncCallArgs(CallInst &OrigCall,
     }
   }
 
-  IGC_ASSERT_MESSAGE(NewCallOps.size() ==
-                         OrigCall.arg_size() - OmittedCount,
+  IGC_ASSERT_MESSAGE(NewCallOps.size() == OrigCall.arg_size() - OmittedCount,
                      "varargs are unexpected");
   return NewCallOps;
 }
@@ -795,7 +795,8 @@ static Value *passGlobalAsCallArg(GlobalArgInfo GAI, CallInst &OrigCall) {
   // We should should load the global first to pass it by value.
   if (GAI.Kind == GlobalArgKind::ByValueIn ||
       GAI.Kind == GlobalArgKind::ByValueInOut)
-    return new LoadInst(GAI.GV->getValueType(), GAI.GV, GAI.GV->getName() + ".val",
+    return new LoadInst(GAI.GV->getValueType(), GAI.GV,
+                        GAI.GV->getName() + ".val",
                         /* isVolatile */ false, &OrigCall);
   IGC_ASSERT_MESSAGE(
       GAI.Kind == GlobalArgKind::ByPointer,
