@@ -10,6 +10,8 @@ SPDX-License-Identifier: MIT
 #define IGCLLVM_IR_TYPE_H
 
 #include "llvm/IR/Type.h"
+#include "llvm/Support/ErrorHandling.h"
+#include <utility>
 
 namespace IGCLLVM {
 /// Only use this method in code that is not reachable with opaque pointers,
@@ -19,16 +21,24 @@ namespace IGCLLVM {
 // latest relevant LLVM revision to simplify migration. In this instance,
 // a shorthand is chosen primarily for formatting reasons.
 inline llvm::Type *getNonOpaquePtrEltTy(const llvm::Type *PtrTy) {
+#if LLVM_VERSION_MAJOR < 17 || defined(IGC_LLVM_TRUNK_REVISION)
   return PtrTy->getNonOpaquePointerElementType();
+#else
+  // not supported above LLVM 16
+  llvm_unreachable("Typed pointers are discontinued above LLVM16.");
+  return nullptr;
+#endif
 }
 
-inline bool isOpaquePointerTy(const llvm::Type *PtrTy) {
+inline bool isPointerTy(const llvm::Type *PtrTy) {
+#if LLVM_VERSION_MAJOR < 17 || defined(IGC_LLVM_TRUNK_REVISION)
   return PtrTy->isOpaquePointerTy();
+#else
+  return PtrTy->isPointerTy();
+#endif
 }
 
-inline bool isBFloatTy(llvm::Type *type) {
-  return type->getTypeID() == llvm::Type::TypeID::BFloatTyID;
-}
+inline bool isBFloatTy(llvm::Type *type) { return type->getTypeID() == llvm::Type::TypeID::BFloatTyID; }
 
 inline bool isTargetExtTy(const llvm::Type *PtrTy) {
 #if LLVM_VERSION_MAJOR < 16
@@ -51,4 +61,4 @@ inline llvm::StringRef getTargetExtName(const llvm::Type *PtrTy) {
 }
 } // namespace IGCLLVM
 
-#endif // IGCLLVM_IR_TYPE_H
\ No newline at end of file
+#endif // IGCLLVM_IR_TYPE_H
