@@ -37,7 +37,7 @@ bool isTargetExtTy(const Type *Ty) {
 }
 
 bool isImageBuiltinType(const Type *BuiltinTy) {
-  if (BuiltinTy->isPointerTy() && !IGCLLVM::isOpaquePointerTy(BuiltinTy))
+  if (BuiltinTy->isPointerTy() && !IGCLLVM::isPointerTy(BuiltinTy))
     BuiltinTy = IGCLLVM::getNonOpaquePtrEltTy(BuiltinTy);
 
   if (const StructType *StructTy = dyn_cast<StructType>(BuiltinTy); StructTy && StructTy->isOpaque()) {
@@ -101,10 +101,14 @@ static bool isAnyArgOpenCLTargetExtTy(const Function &F) {
     if (checkIfNeedsRetyping(ArgTy))
       return true;
 
-    if (A.hasStructRetAttr()) {
-      if (checkIfNeedsRetyping(A.getParamStructRetType()))
-        return true;
-    }
+    if (A.hasStructRetAttr() && checkIfNeedsRetyping(A.getParamStructRetType()))
+      return true;
+
+    if (A.hasByValAttr() && checkIfNeedsRetyping(A.getParamByValType()))
+      return true;
+
+    if (A.hasByRefAttr() && checkIfNeedsRetyping(A.getParamByRefType()))
+      return true;
   }
 
   return false;
@@ -231,7 +235,7 @@ void retypeOpenCLTargetExtTyAsPointers(Module *M) {
 
   // Global mapping between TargetExtTy structs and their retyped variant (they are shared between functions).
   DenseMap<StructType *, StructType *> TETtoRetypedStructs;
-  DenseMap<Function *, FunctionSignatureChange> PendingSigChanges;
+  MapVector<Function *, FunctionSignatureChange> PendingSigChanges;
 
   // Remap bodies and collect function signature changes.
   for (Function &F : *M) {
@@ -265,6 +269,14 @@ void retypeOpenCLTargetExtTyAsPointers(Module *M) {
               break;
             }
           }
+          if (auto *AI = dyn_cast<AllocaInst>(&I)) {
+            Type *AllocatedTy = AI->getAllocatedType();
+            if (checkIfNeedsRetyping(AllocatedTy)) {
+              UsesTargetExt = true;
+              break;
+            }
+          }
+
           if (UsesTargetExt)
             break;
         }
@@ -274,9 +286,8 @@ void retypeOpenCLTargetExtTyAsPointers(Module *M) {
     }
 
     // If neither args/return/instructions use OpenCL TargetExtTy, skip.
-    if (!UsesTargetExt) {
+    if (!UsesTargetExt)
       continue;
-    }
 
     OpenCLTargetExtTypeMapper Mapper(F, TETtoRetypedStructs);
     ValueToValueMapTy VM;
@@ -324,6 +335,28 @@ void retypeOpenCLTargetExtTyAsPointers(Module *M) {
             AttrsChanged = true;
           }
         }
+        if (Attrs.hasAttribute(llvm::Attribute::ByVal)) {
+          Type *OldByValTy = Arg.getParamByValType();
+          Type *NewByValTy = Mapper.remapType(OldByValTy);
+          if (NewByValTy != OldByValTy) {
+            AttrBuilder AB(M->getContext(), Attrs);
+            AB.removeAttribute(llvm::Attribute::ByVal);
+            AB.addByValAttr(NewByValTy);
+            Attrs = AttributeSet::get(M->getContext(), AB);
+            AttrsChanged = true;
+          }
+        }
+        if (Attrs.hasAttribute(llvm::Attribute::ByRef)) {
+          Type *OldByRefTy = Arg.getParamByRefType();
+          Type *NewByRefTy = Mapper.remapType(OldByRefTy);
+          if (NewByRefTy != OldByRefTy) {
+            AttrBuilder AB(M->getContext(), Attrs);
+            AB.removeAttribute(llvm::Attribute::ByRef);
+            AB.addByRefAttr(NewByRefTy);
+            Attrs = AttributeSet::get(M->getContext(), AB);
+            AttrsChanged = true;
+          }
+        }
         NewArgAttrs.push_back(Attrs);
       }
 
@@ -331,7 +364,7 @@ void retypeOpenCLTargetExtTyAsPointers(Module *M) {
           AttrsChanged ? AttributeList::get(M->getContext(), OldAttrs.getFnAttrs(), OldAttrs.getRetAttrs(), NewArgAttrs)
                        : OldAttrs;
 
-      PendingSigChanges.try_emplace(&F, FunctionSignatureChange{NewFTy, NewAttrs});
+      PendingSigChanges.insert(std::make_pair(&F, FunctionSignatureChange{NewFTy, NewAttrs}));
     }
   }
 
