@@ -178,8 +178,7 @@ static bool tryParsePassNumber(PassDisableConfig &pdc, const std::string &Token)
   if (std::all_of(tempTokenWithoutSpaces.cbegin(), tempTokenWithoutSpaces.cend(), ::isdigit)) {
     std::cerr << "You want to skip pass ID " << Token << std::endl;
     unsigned int passNumber = std::stoi(Token);
-    auto localSkipComand = SkipCommand(passNumber);
-    pdc.skipCommands.push_back(localSkipComand);
+    pdc.skipCommands.emplace_back(passNumber);
     // move to next Token
     return true;
   }
@@ -234,8 +233,7 @@ static bool tryParsePassRange(PassDisableConfig &pdc, const std::string &Token)
     unsigned int skippingTo = std::stoi(vectorOfSubTokens[1]);
     if (checkSkipCommandBounds(skippingFrom, skippingTo)) {
       std::cerr << "You want to skip passes from " << skippingFrom << " to " << skippingTo << std::endl;
-      auto localSkipComand = SkipCommand(skippingFrom, skippingTo);
-      pdc.skipCommands.push_back(localSkipComand);
+      pdc.skipCommands.emplace_back(skippingFrom, skippingTo);
     }
     break;
   }
@@ -247,9 +245,8 @@ static bool tryParsePassRange(PassDisableConfig &pdc, const std::string &Token)
     if (SkipCommand *sc = getFirstCommand(pdc, TokenSkipCase::OpenRange)) {
       sc->start = std::min(sc->start, skippingFrom);
     } else {
-      auto localSkipComand =
-          SkipCommand("", skippingFrom, std::numeric_limits<unsigned int>::max(), TokenSkipCase::OpenRange);
-      pdc.skipCommands.push_back(localSkipComand);
+      pdc.skipCommands.emplace_back("", skippingFrom, std::numeric_limits<unsigned int>::max(),
+                                    TokenSkipCase::OpenRange);
     }
     break;
   }
@@ -275,9 +272,8 @@ static bool tryParsePassNameSpecificOccurrence(PassDisableConfig &pdc, const std
   }
   std::cerr << "You want to skip pass " << nameOfPassToSkip << " occurrence " << occurrenceOfPass << std::endl;
   unsigned int passOccurrence = std::stoi(occurrenceOfPass);
-  auto localSkipComand =
-      SkipCommand(nameOfPassToSkip, passOccurrence, passOccurrence, TokenSkipCase::SpecificPassOccurrence);
-  pdc.skipCommands.push_back(localSkipComand);
+  pdc.skipCommands.emplace_back(nameOfPassToSkip, passOccurrence, passOccurrence,
+                                TokenSkipCase::SpecificPassOccurrence);
   // move to next Token
   return true;
 }
@@ -315,9 +311,8 @@ static bool tryParsePassNameOccurrenceRange(PassDisableConfig &pdc, const std::s
     if (checkSkipCommandBounds(skippingFrom, skippingTo)) {
       std::cerr << "You want to skip pass " << nameOfPassToSkip << " from occurrence " << skippingFrom
                 << " to occurrence " << skippingTo << std::endl;
-      auto localSkipComand =
-          SkipCommand(nameOfPassToSkip, skippingFrom, skippingTo, TokenSkipCase::SpecificPassOccurrenceRange);
-      pdc.skipCommands.push_back(localSkipComand);
+      pdc.skipCommands.emplace_back(nameOfPassToSkip, skippingFrom, skippingTo,
+                                    TokenSkipCase::SpecificPassOccurrenceRange);
     }
     break;
   }
@@ -330,9 +325,8 @@ static bool tryParsePassNameOccurrenceRange(PassDisableConfig &pdc, const std::s
     if (SkipCommand *sc = getFirstCommand(pdc, TokenSkipCase::SpecificPassOccurrenceOpenRange, nameOfPassToSkip)) {
       sc->start = std::min(sc->start, skippingFrom);
     } else {
-      auto localSkipComand = SkipCommand(nameOfPassToSkip, skippingFrom, std::numeric_limits<unsigned int>::max(),
-                                         TokenSkipCase::SpecificPassOccurrenceOpenRange);
-      pdc.skipCommands.push_back(localSkipComand);
+      pdc.skipCommands.emplace_back(nameOfPassToSkip, skippingFrom, std::numeric_limits<unsigned int>::max(),
+                                    TokenSkipCase::SpecificPassOccurrenceOpenRange);
     }
     break;
   }
@@ -376,8 +370,7 @@ static bool tryParsePassNames(PassDisableConfig &pdc, const std::string &Token)
   case 1: {
     const std::string &nameOfPassToSkip = vectorOfSubTokens[0];
     std::cerr << "You want to skip all occurrence of " << nameOfPassToSkip << std::endl;
-    auto localSkipComand = SkipCommand(nameOfPassToSkip);
-    pdc.skipCommands.push_back(localSkipComand);
+    pdc.skipCommands.emplace_back(nameOfPassToSkip);
     // move to next Token
     return true;
   }
@@ -405,7 +398,7 @@ static void parseShaderPassDisableFlag(PassDisableConfig &pdc) {
     debugOutputStream << "Your input to ShaderPassDisable: " << passToSkip << '\n';
     debugOutputStream << "You want to skip pass ID " << passToSkip << '\n';
 
-    pdc.skipCommands.push_back(SkipCommand(passToSkip));
+    pdc.skipCommands.emplace_back(passToSkip);
   }
   // Treat it as a string otherwise
   else {
@@ -642,6 +635,15 @@ void IGCPassManager::add(Pass *P) {
   }
 }
 
+// should be an std::string_view, but StringRef doesn't implement implicit conversion unless __cplusplus > 201402L
+template <typename StringT> inline static std::string cleanPassName(const StringT &passName) {
+  // Remove non-alphanumeric characters from pass name
+  std::string newName;
+  std::copy_if(passName.begin(), passName.end(), std::back_inserter(newName),
+               [](unsigned char c) { return std::isalnum(c); });
+  return newName;
+}
+
 // List: a comma/semicolon-separated list of pass names.
 //    N: a pass name
 // return true if N is in List.
@@ -652,7 +654,7 @@ bool IGCPassManager::isInList(const StringRef &N, const StringRef &List) const {
     size_t endPos = List.find_first_of(Separators, startPos);
     size_t len = (endPos != StringRef::npos ? endPos - startPos : endPos);
     StringRef Name = List.substr(startPos, len);
-    if (IGCLLVM::equals_insensitive(Name, N)) {
+    if (IGCLLVM::equals_insensitive(cleanPassName(Name), cleanPassName(N))) {
       return true;
     }
     startPos = (endPos != StringRef::npos ? endPos + 1 : StringRef::npos);
@@ -698,14 +700,15 @@ bool IGCPassManager::isPrintAfter(Pass *P) {
 }
 
 void IGCPassManager::addPrintPass(Pass *P, bool isBefore) {
-  std::string passName = m_name + (isBefore ? "_before_" : "_after_") +
-                         (P->getPassName().startswith("Unnamed pass") ? "UnnamedPass" : std::string{P->getPassName()});
+  auto passName = P->getPassName();
+  std::string fullPassName = m_name + (isBefore ? "_before_" : "_after_") +
+                             (passName.startswith("Unnamed pass") ? "UnnamedPass" : cleanPassName(passName.str()));
 
   auto name = IGC::Debug::DumpName(IGC::Debug::GetShaderOutputName())
                   .ShaderName(m_pContext->shaderName)
                   .Type(m_pContext->type)
                   .Hash(m_pContext->hash)
-                  .Pass(passName, std::optional<uint32_t>(m_pContext->m_numPasses++))
+                  .Pass(fullPassName, std::optional<uint32_t>(m_pContext->m_numPasses++))
                   .StagedInfo(m_pContext)
                   .Extension("ll");
 
@@ -735,10 +738,19 @@ void IGCPassManager::addPrintPass(Pass *P, bool isBefore) {
 void DumpLLVMIR(IGC::CodeGenContext *pContext, const char *dumpName) {
   SetCurrentDebugHash(pContext->hash);
   SetCurrentEntryPoints(pContext->entry_names);
+  auto makeName = [&]() {
+    return DumpName(IGC::Debug::GetShaderOutputName())
+        .ShaderName(pContext->shaderName)
+        .Hash(pContext->hash)
+        .Type(pContext->type)
+        .Pass(dumpName)
+        .Retry(pContext->m_retryManager->GetRetryId())
+        .StagedInfo(pContext)
+        .Extension("ll");
+  };
 
   if (IGC_IS_FLAG_ENABLED(DumpLLVMIR)) {
     auto module = pContext->getModule();
-
     if (IGC_IS_FLAG_ENABLED(ShaderDumpInstNamer)) {
       // We first clear out any old entries of x if exists
       // from the ValueSymTab by overriding the x variable to _x
@@ -769,32 +781,24 @@ void DumpLLVMIR(IGC::CodeGenContext *pContext, const char *dumpName) {
 
     pContext->getMetaDataUtils()->save(*pContext->getLLVMContext());
     serialize(*(pContext->getModuleMetaData()), module);
-    using namespace IGC::Debug;
-    auto name = DumpName(IGC::Debug::GetShaderOutputName())
-                    .ShaderName(pContext->shaderName)
-                    .Hash(pContext->hash)
-                    .Type(pContext->type)
-                    .Pass(dumpName)
-                    .Retry(pContext->m_retryManager.GetRetryId())
-                    .Extension("ll");
 
     auto new_annotator = IntrinsicAnnotator();
     auto annotator = (pContext->annotater != nullptr) ? pContext->annotater : &new_annotator;
-    DumpLLVMIRText(module, name, annotator);
+    DumpLLVMIRText(module, makeName(), annotator);
   }
   if (IGC_IS_FLAG_ENABLED(ShaderOverride)) {
-    auto name = DumpName(IGC::Debug::GetShaderOutputName())
-                    .ShaderName(pContext->shaderName)
-                    .Hash(pContext->hash)
-                    .Type(pContext->type)
-                    .Pass(dumpName)
-                    .Retry(pContext->m_retryManager.GetRetryId())
-                    .Extension("ll");
     SMDiagnostic Err;
-    std::string fileName = name.overridePath();
+    std::string fileName = makeName().overridePath();
     FILE *fp = fopen(fileName.c_str(), "r");
     if (fp != nullptr) {
       fclose(fp);
+      // Check if not trying to override dump that cause a crash
+      if (strcmp(dumpName, "codegen") == 0 || pContext->m_retryManager->GetRetryId() != 0) {
+        pContext->EmitWarning(std::string("Override shader attempt detected for an unsupported dump: " + fileName +
+                                          ". Skipping override.")
+                                  .c_str());
+        return;
+      }
       errs() << "Override shader: " << fileName << "\n";
       Module *mod = parseIRFile(fileName, Err, *pContext->getLLVMContext()).release();
       if (mod) {
