@@ -325,6 +325,9 @@ enum class ShaderTypeMD
         // for continuations used in ReorderThread, this field indicates the maximum value of the coherence hint
         uint32_t NumCoherenceHintBits = 0;
 
+        // Informs if the kernel requires extra allocation of the sync HW stack
+        bool useSyncHWStack = false;
+
         // if the function was created by cloning another function
         // this will contain the name of the original shader
         std::string OriginatingShaderName;
@@ -456,9 +459,11 @@ enum class ShaderTypeMD
         bool disableMathRefactoring                     = false;
         bool atomicBranch                               = false;
         bool spillCompression                           = false;
+        bool AllowLICM                                  = true;
         bool DisableEarlyOut                            = false;
         bool ForceInt32DivRemEmu                        = false;
         bool ForceInt32DivRemEmuSP                      = false;
+        bool DisableIntDivRemIncrementReduction         = false;
         bool DisableFastestSingleCSSIMD                 = false;
         bool DisableFastestLinearScan                   = false;
         //if PTSS is enabled and if PrivateData is too large (>256k in XeHP_SDV+),
@@ -493,7 +498,7 @@ enum class ShaderTypeMD
         bool ForceUniformSurfaceSampler                 = false;
         bool EnableIndependentSharedMemoryFenceFunctionality = false;
         bool NewSpillCostFunction                       = false;
-        bool EnableVRT                                  = false;
+        bool EnableVRT                                  = true;
         bool ForceLargeGRFNum4RQ                        = false;
         bool DisableEUFusion                            = false;
         bool DisableFDivToFMulInvOpt                    = false;
@@ -514,8 +519,11 @@ enum class ShaderTypeMD
         unsigned ForcePushConstantMode                  = 0;
         bool UseInstructionHoistingOptimization         = false;
         bool DisableResourceLoopDestLifeTimeStart       = false;
+        unsigned ForceVRTGRFCeiling                     = 0;
+        unsigned DisableSamplerBackingByLSC             = 0;
         bool UseLinearScanRA                            = false;
         bool DisableConvertingAtomicIAddToIncDec        = false;
+        bool ZeroInitRegistersBeforeExecution = false;
     };
 
     enum class ThreadIDLayout
@@ -872,6 +880,7 @@ enum class ShaderTypeMD
     bool isBindless(const IGC::FunctionMetaData &funcMD);
     bool isContinuation(const IGC::FunctionMetaData& funcMD);
     bool isCallStackHandler(const IGC::FunctionMetaData &funcMD);
+    bool isPrologue(const IGC::FunctionMetaData &funcMD);
 
 
     // User annotations query functions
