@@ -27,6 +27,7 @@ SPDX-License-Identifier: MIT
 #include "common/LLVMWarningsPop.hpp"
 
 #include <stdarg.h>
+#include <filesystem>
 #include <fstream>
 #include <sstream>
 #include <iomanip>
@@ -222,14 +223,14 @@ std::string DumpName::AbsolutePath(OutputFolderName folder) const {
   if (m_hash.has_value()) {
 
     if (m_type.has_value() && IGC_IS_FLAG_ENABLED(EnableShaderNumbering)) {
+      hashMapLock.lock();
       bool increment = shaderHashMap.insert({m_hash->asmHash, shaderNum}).second;
       // Need to serialize access to the shaderNum counter in case different threads need to dump the same shader at
       // once.
-      hashMapLock.lock();
       if (increment)
         shaderNum++;
-      hashMapLock.unlock();
       ss << "_" << shaderHashMap[m_hash->asmHash] << "_";
+      hashMapLock.unlock();
     }
 
     if (m_hash->asmHash != 0) {
@@ -497,9 +498,8 @@ private:
 // (In the case where Dump is on the heap and DumpName is on the stack, the reference would go bad
 // when the calling function returns)
 Dump::Dump(DumpName const &dumpName, DumpType type)
-    : m_name(dumpName), m_pStream(std::make_unique<llvm::raw_string_ostream>(m_string)) // buffered stream!
-      ,
-      m_pStringStream(nullptr), m_type(type), m_ClearFile(true) {
+    : m_name(dumpName), m_pStream(std::make_unique<llvm::raw_string_ostream>(m_string)), // buffered stream!
+      m_pStringStream(nullptr), m_type(type) {
   m_pStringStream = m_pStream.get();
   if (isConsolePrintable(m_type)) {
     if (IGC_IS_FLAG_ENABLED(PrintToConsole)) {
@@ -520,21 +520,22 @@ Dump::Dump(DumpName const &dumpName, DumpType type)
 
 void Dump::flush() {
   m_pStringStream->flush();
-  if (m_string.empty() && !m_ClearFile) {
-    return;
-  }
   std::ios_base::openmode mode = std::ios_base::out;
-  if (m_ClearFile) {
-    m_ClearFile = false;
-  } else {
-    mode |= std::ios_base::app;
-  }
   if (!isText(m_type))
     mode |= std::ios_base::binary;
+  const FILENAME_COLLISION_MODE collisionMode =
+      static_cast<FILENAME_COLLISION_MODE>(IGC_GET_FLAG_VALUE(ShaderDumpCollisionMode));
   if (m_name.allow()) {
-    std::ofstream asmFile(m_name.str(), mode);
-    asmFile << m_string;
-    asmFile.close();
+    if (collisionMode == FILENAME_COLLISION_MODE::OVERRIDE || !std::filesystem::exists(m_name.str())) {
+      std::ofstream asmFile(m_name.str(), mode);
+      asmFile << m_string;
+    } else if (isText(m_type)) { // Do NOT append binary files
+      std::ofstream asmFile(m_name.str(), mode | std::ios_base::app);
+      if (collisionMode == FILENAME_COLLISION_MODE::APPEND)
+        asmFile << commentPrefix(m_type) << "Warning: appending filename collides\n" << m_string;
+      else
+        asmFile << commentPrefix(m_type) << "Warning: skipping filename collides\n";
+    }
   }
   m_string.clear();
 }
@@ -614,7 +615,7 @@ DumpName GetDumpNameObj(const IGC::CShader *pProgram, const char *ext) {
   dumpName = dumpName.PostFix(shaderName);
   dumpName = dumpName.DispatchMode(pProgram->m_ShaderDispatchMode);
   dumpName =
-      dumpName.SIMDSize(pProgram->m_State.m_dispatchSize).Retry(context->m_retryManager.GetRetryId()).Extension(ext);
+      dumpName.SIMDSize(pProgram->m_State.m_dispatchSize).Retry(context->m_retryManager->GetRetryId()).Extension(ext);
 
   return dumpName;
 }
@@ -624,7 +625,7 @@ DumpName GetLLDumpName(IGC::CodeGenContext *pContext, const char *dumpName) {
                   .Hash(pContext->hash)
                   .Type(pContext->type)
                   .Pass(dumpName)
-                  .Retry(pContext->m_retryManager.GetRetryId())
+                  .Retry(pContext->m_retryManager->GetRetryId())
                   .Extension("ll");
   return name;
 }
