@@ -41,6 +41,8 @@ typedef enum {
     IGFX_BMG = 1274,
     IGFX_LUNARLAKE = 1275,
     IGFX_PTL = 1300,
+    IGFX_NVL_XE3G = 1340,
+    IGFX_CRI = 1380,
     IGFX_MAX_PRODUCT,
 
 
@@ -90,10 +92,11 @@ typedef enum {
     IGFX_GEN12_CORE   = 17,     // Gen12 Family
     IGFX_GEN12LP_CORE = 18,     // Gen12 LP Family
     IGFX_XE_HP_CORE   = 0x0c05, // XeHP Family
-    IGFX_XE_HPG_CORE = 0x0c07,  // XE_HPG Family
-    IGFX_XE_HPC_CORE = 0x0c08,  // XE_HPC Family
+    IGFX_XE_HPG_CORE  = 0x0c07, // XE_HPG Family
+    IGFX_XE_HPC_CORE  = 0x0c08, // XE_HPC Family
     IGFX_XE2_HPG_CORE = 0x0c09, // XE2_HPG Family
-    IGFX_XE3_CORE     = 0x1e00,  // XE3 Family
+    IGFX_XE3_CORE     = 0x1e00, // XE3 Family
+    IGFX_XE3P_CORE    = 0x2300, // XE3P Family
     IGFX_MAX_CORE,              // Max Family, for lookup table
 
     IGFX_GENNEXT_CORE          = 0x7ffffffe,  //GenNext
@@ -149,6 +152,10 @@ typedef struct GFX_GMD_ID_DEF
     };
 }GFX_GMD_ID;
 
+#define GFX_GMD_ARCH_12                          (12)
+#define GFX_GMD_ARCH_20                          (20)
+#define GFX_GMD_ARCH_30                          (30)
+
 #define GFX_GMD_ARCH_12_RELEASE_XE_LP_MD                 (70)
 #define GFX_GMD_ARCH_12_RELEASE_XE_LP_LG                 (71)
 #define GFX_GMD_ARCH_12_RELEASE_XE_LPG_PLUS_1274         (74)
@@ -160,7 +167,12 @@ typedef struct GFX_GMD_ID_DEF
 #define GFX_GMD_ARCH_30_RELEASE_XE3_LPG_3000             (0)
 #define GFX_GMD_ARCH_30_RELEASE_XE3_LPG_3001             (1)
 #define GFX_GMD_ARCH_30_RELEASE_XE3_LPG_3003             (3)
+#define GFX_GMD_ARCH_30_RELEASE_XE3_LPG_3004             (4)
+#define GFX_GMD_ARCH_30_RELEASE_XE3_LPG_3005             (5)
 
+#define GFX_GET_GMD_ARCH_VERSION_RENDER(p)                ((p).sRenderBlockID.GmdID.GMDArch)
+#define GFX_GET_GMD_ARCH_VERSION_DISPLAY(p)               ((p).sDisplayBlockID.GmdID.GMDArch)
+#define GFX_GET_GMD_ARCH_VERSION_MEDIA(p)                 ((p).sMediaBlockID.GmdID.GMDArch)
 #define GFX_GET_GMD_RELEASE_VERSION_RENDER(p)             ((p).sRenderBlockID.GmdID.GMDRelease)
 #define GFX_GET_GMD_RELEASE_VERSION_DISPLAY(p)            ((p).sDisplayBlockID.GmdID.GMDRelease)
 #define GFX_GET_GMD_RELEASE_VERSION_MEDIA(p)              ((p).sMediaBlockID.GmdID.GMDRelease)
@@ -779,6 +791,16 @@ typedef enum __NATIVEGTTYPE
 #define DEV_ID_FD80                             0xFD80
 #define DEV_ID_FD81                             0xFD81
 
+//CRI
+#define DEV_ID_674C                             0x674C
+
+// NVL
+#define DEV_ID_D740                             0xD740
+#define DEV_ID_D741                             0xD741
+#define DEV_ID_D742                             0xD742
+#define DEV_ID_D743                             0xD743
+#define DEV_ID_D744                             0xD744
+
 #define GFX_IS_DG2_G11_CONFIG(d) ( ( d == DEV_ID_56A5 )             ||   \
                                  ( d == DEV_ID_56A6 )             ||   \
                                  ( d == DEV_ID_5693 )             ||   \
