@@ -125,7 +125,7 @@ class KernelCost;
 // Handles information for GRF selection
 class GRFMode {
 public:
-  GRFMode(const TARGET_PLATFORM platform, Options *op);
+  GRFMode(const TARGET_PLATFORM platform, unsigned regSize, Options *op);
 
   void setModeByNumGRFs(unsigned grfs) {
     unsigned size = configs.size();
@@ -268,6 +268,7 @@ private:
   unsigned upperBoundGRF;
   unsigned GRFModeUpValue;
   const TARGET_PLATFORM platform;
+  unsigned grfSize;
   Options *options;
 };
 
@@ -691,6 +692,18 @@ public:
   int32_t getInt32KernelAttr(Attributes::ID aID) const {
     return getKernelAttrs()->getInt32KernelAttr(aID);
   }
+  bool isScratchLocationSet() const {
+    // checking if scratch location is set by IGC through attributes
+    return (getInt32KernelAttr(Attributes::ATTR_ScratchInlineOffset) > 0 ||
+            getInt32KernelAttr(Attributes::ATTR_ScratchIndirectRegOffset) > 0 ||
+            getInt32KernelAttr(Attributes::ATTR_ScratchIndirectMemOffset) > 0);
+  }
+
+  bool isPrivateMemUsed() const {
+    // checking if private memory is used resulting in scratch access
+    return (getKernelAttrs()->isKernelAttrSet(Attributes::ATTR_SepSpillPvtSS) ||
+            getKernelAttrs()->isKernelAttrSet(Attributes::ATTR_SpillMemOffset));
+  }
   bool getOption(vISAOptions opt) const { return m_options->getOption(opt); }
   uint32_t getuInt32Option(vISAOptions opt) const {
     return m_options->getuInt32Option(opt);
@@ -724,6 +737,8 @@ public:
   void setName(const char *n) { name = n; }
   const char *getName() const { return name; }
 
+  bool updateKernelToSmallerGRF();
+  bool canUpdateKernelToLargerGRF();
   bool updateKernelToLargerGRF();
   void updateKernelByRegPressure(unsigned regPressure,
                                  bool forceGRFModeUp = false);
@@ -911,6 +926,12 @@ public:
     I->setMaskOption(MO);
   }
   // end of WA related
+  struct SampleWithLSCBacking {
+    G4_INST *inst = nullptr;
+    unsigned int opndNum = 0xffffffff;
+    unsigned int bitPos = 0xffffffff;
+  };
+  std::list<struct SampleWithLSCBacking> samplerWithLSCBacking;
 };     // G4_Kernel
 
 } // namespace vISA
