@@ -1,6 +1,6 @@
 /*========================== begin_copyright_notice ============================
 
-Copyright (C) 2021-2022 Intel Corporation
+Copyright (C) 2021-2025 Intel Corporation
 
 SPDX-License-Identifier: MIT
 
@@ -131,6 +131,7 @@ bool vISA::MsgOpHasChMask(MsgOp op) {
   case MsgOp::STORE_QUAD:
   case MsgOp::LOAD_QUAD_MSRT:
   case MsgOp::STORE_QUAD_MSRT:
+  case MsgOp::LOAD_QUAD_STATUS:
     return true;
   default:
     break;
@@ -466,6 +467,69 @@ int vISA::GetNumVecElems(VecElems ves) {
   }
 }
 
+uint32_t vISA::GetCacheControlOpEncoding(CacheControlOperation cc) {
+  switch (cc) {
+  case CacheControlOperation::DIRTY_SET:
+    return 0;
+  case CacheControlOperation::DIRTY_RESET:
+    return 1;
+  default:
+    vISA_ASSERT_UNREACHABLE("invalid cache control operation");
+  }
+  return 0;
+}
+
+uint32_t vISA::GetCacheControlSizeEncoding(CacheControlSize cs) {
+  switch (cs) {
+  case CacheControlSize::S64B:
+    return 0;
+  case CacheControlSize::S128B:
+    return 1;
+  case CacheControlSize::S192B:
+    return 2;
+  case CacheControlSize::S256B:
+    return 3;
+  default:
+    vISA_ASSERT_UNREACHABLE("invalid cache control size");
+  }
+  return 0;
+}
+std::string vISA::ToSymbol(AddrSizeType a) {
+  switch (a) {
+  case AddrSizeType::GLB_A64_A32S:
+    return "a32s";
+  case AddrSizeType::GLB_A64_A32U:
+    return "a32u";
+  case AddrSizeType::GLB_A64_A64:
+    return "a64";
+  case AddrSizeType::GLB_STATE_A32:
+    return "a32u";
+  case AddrSizeType::SLM_A32_A32:
+    return "a32";
+  case AddrSizeType::URB_A32_A32:
+    return "a32";
+  default:
+    return "?";
+  }
+}
+
+uint32_t vISA::GetAddrSizeTypeEncoding(AddrSizeType a) {
+  switch (a) {
+  case AddrSizeType::SLM_A32_A32:
+  case AddrSizeType::URB_A32_A32:
+  case AddrSizeType::GLB_A64_A32U:
+    return 0;
+  case AddrSizeType::GLB_A64_A32S:
+    return 1;
+  case AddrSizeType::GLB_A64_A64:
+    return 2;
+  case AddrSizeType::GLB_STATE_A32:
+    return 3;
+  default:
+    vISA_ASSERT_UNREACHABLE("invalid address size type");
+  }
+  return 0;
+}
 
 std::string vISA::ToSymbol(Caching c) {
   switch (c) {
@@ -496,6 +560,14 @@ std::string vISA::ToSymbol(Caching l1, Caching l3) {
   else
     return ToSymbol(l1) + ToSymbol(l3);
 }
+
+std::string vISA::ToSymbol(Caching l1, Caching l2, Caching l3) {
+  if (l1 == Caching::DF && l2 == Caching::DF && l3 == Caching::DF)
+    return "";
+  else
+    return ToSymbol(l1) + ToSymbol(l2) + ToSymbol(l3);
+}
+
 int ElemsPerAddr::getCount() const {
   if (!isChannelMask())
     return count;
@@ -556,6 +628,503 @@ bool G4_SendDesc::isLSC() const {
   return false;
 }
 
+///////////////////////////////////////////////////////////////////////////////
+// G4_SendgDesc implementations
+///////////////////////////////////////////////////////////////////////////////
+G4_SendgDesc::G4_SendgDesc(SFID _sfid, uint64_t desc, LdStAttrs _attrs,
+                           const IR_Builder &builder)
+    : G4_SendDesc(G4_SendDesc::Kind::GENERALIZED, _sfid, builder), descValue(desc),
+      attrs(_attrs),
+      dstLen(0), src0Len(0), src1Len(0) { }
+
+MsgOp G4_SendgDesc::getOp() const {
+  return MsgOpDecode(sfid, (uint32_t)(descValue & 0x3F));
+}
+
+static constexpr std::array<std::tuple<Caching,Caching,Caching>,16> LDCCS {
+  std::make_tuple(Caching::DF, Caching::DF, Caching::DF), // 0
+  std::make_tuple(Caching::INVALID, Caching::INVALID, Caching::INVALID), // 1
+  std::make_tuple(Caching::UC, Caching::UC, Caching::UC), // 2
+  std::make_tuple(Caching::UC, Caching::UC, Caching::CA), // 3
+  std::make_tuple(Caching::UC, Caching::CA, Caching::UC), // 4
+  std::make_tuple(Caching::UC, Caching::CA, Caching::CA), // 5
+  std::make_tuple(Caching::CA, Caching::UC, Caching::UC), // 6
+  std::make_tuple(Caching::CA, Caching::UC, Caching::CA), // 7
+  std::make_tuple(Caching::CA, Caching::CA, Caching::UC), // 8
+  std::make_tuple(Caching::CA, Caching::CA, Caching::CA), // 9
+  std::make_tuple(Caching::ST, Caching::UC, Caching::UC), // 10
+  std::make_tuple(Caching::ST, Caching::UC, Caching::CA), // 11
+  std::make_tuple(Caching::ST, Caching::CA, Caching::UC), // 12
+  std::make_tuple(Caching::ST, Caching::CA, Caching::CA), // 13
+  std::make_tuple(Caching::RI, Caching::RI, Caching::RI), // 14
+  std::make_tuple(Caching::INVALID, Caching::INVALID, Caching::INVALID), // 15
+};
+static constexpr std::array<std::tuple<Caching,Caching,Caching>,16> STCCS {
+  std::make_tuple(Caching::DF, Caching::DF, Caching::DF), // 0
+  std::make_tuple(Caching::INVALID, Caching::INVALID, Caching::INVALID), // 1
+  std::make_tuple(Caching::UC, Caching::UC, Caching::UC), // 2
+  std::make_tuple(Caching::UC, Caching::UC, Caching::WB), // 3
+  std::make_tuple(Caching::UC, Caching::WB, Caching::UC), // 4
+  std::make_tuple(Caching::UC, Caching::WB, Caching::WB), // 5
+  std::make_tuple(Caching::WT, Caching::UC, Caching::UC), // 6
+  std::make_tuple(Caching::WT, Caching::UC, Caching::WB), // 7
+  std::make_tuple(Caching::WT, Caching::WB, Caching::UC), // 8
+  std::make_tuple(Caching::WT, Caching::WB, Caching::WB), // 9
+  std::make_tuple(Caching::ST, Caching::UC, Caching::UC), // 10
+  std::make_tuple(Caching::ST, Caching::UC, Caching::WB), // 11
+  std::make_tuple(Caching::ST, Caching::WB, Caching::UC), // 12
+  std::make_tuple(Caching::WB, Caching::UC, Caching::UC), // 13
+  std::make_tuple(Caching::WB, Caching::WB, Caching::UC), // 14
+  std::make_tuple(Caching::WB, Caching::UC, Caching::WB), // 15
+};
+
+
+std::tuple<Caching,Caching,Caching> G4_SendgDesc::getCaching() const {
+  bool isLd = MsgOpIsLoad(getOp());
+  bool isSt = MsgOpIsStore(getOp());
+  bool isAt = MsgOpIsAtomic(getOp());
+  if (!isLd && !isSt && !isAt) {
+    return std::make_tuple(Caching::INVALID, Caching::INVALID, Caching::INVALID);
+  }
+
+  uint32_t cachingEnc = (descValue >> 16) & 0xF;
+  if (isLd)
+    return LDCCS[cachingEnc];
+  else
+    return STCCS[cachingEnc];
+}
+
+std::optional<ImmOff> G4_SendgDesc::getOffset() const {
+  const auto op = getOp();
+  if (!isLSC() || sfid == SFID::TGM || !MsgOpIsLoadStoreAtomic(op)) {
+    return std::nullopt;
+  }
+  auto dataSize = (short)getDataSizeBytesMem();
+  auto is2d = MsgOpIs2D(op);
+  auto ast = getAddrSizeType();
+  if (is2d) {
+    // stateless and stateful both have Desc[43:22] for block2d
+    auto offX = (short)getSignedBitField(descValue, 22 +  0, 12);
+    auto offY = (short)getSignedBitField(descValue, 22 + 12, 12);
+    return ImmOff(dataSize * offX, dataSize * offY);
+  } else if (ast == AddrSizeType::GLB_STATE_A32) {
+    // Desc[43:27] in terms of data elements
+    auto enc = getSignedBitField(descValue, 27, 43 - 27 + 1);
+    return ImmOff((int)enc * dataSize);
+  } else {
+    // Desc[43:22] in terms of data elements
+    auto enc = getSignedBitField(descValue, 22, 43 - 22 + 1);
+    return ImmOff((int)enc * dataSize);
+  }
+}
+
+bool G4_SendgDesc::isBarrier() const {
+  return isOp(MsgOp::BARRIER_SIGNAL,
+              MsgOp::BARRIER_SIGNAL_NAMED,
+              MsgOp::BARRIER_SIGNAL_SYSTEM_ROUTINE);
+}
+
+unsigned G4_SendgDesc::getElemsPerAddr() const {
+  auto op = getOp();
+  bool isLdStAt = MsgOpIsLoad(op) || MsgOpIsStore(op) || MsgOpIsAtomic(op);
+  if (!isLdStAt) {
+    return 0;
+  } else if (isOp(MsgOp::LOAD_BLOCK2D, MsgOp::STORE_BLOCK2D)) {
+    return 0; // this is in the header
+  } else if (MsgOpHasChMask(getOp())) {
+    return getDataChMaskCount();
+  } else {
+    return getVecElemsCount();
+  }
+}
+
+std::string G4_SendgDesc::getDescription() const {
+  std::stringstream ss;
+  MsgOp op = getOp();
+  ss << ToSymbol(op);
+  ss << "." << ToSymbol(sfid);
+  bool isLdStAt = MsgOpIsLoadStoreAtomic(op) && !MsgOpIs2D(op);
+  if (isLdStAt) {
+    auto dsz = getDataSize();
+    ss << ".";
+    if (MsgOpHasChMask(op)) {
+      ss << ToSymbol(dsz, getDataChMask());
+    } else {
+      auto dord = getDataOrder();
+      ss << ToSymbol(dsz, getVecElems(), dord);
+    }
+    auto ast = getAddrSizeType();
+    ss << "." << ToSymbol(ast);
+    auto [l1,l2,l3] = getCaching();
+    ss << ToSymbol(l1, l2, l3);
+
+    ss << " ";
+    if (ast == AddrSizeType::GLB_STATE_A32) {
+      if (isScratch()) {
+        ss << "scratch"; // technically surf but cheat for readability
+      } else {
+        ss << "surf";
+      }
+      ss << "[..";
+      if (auto ssIdx = getBitField(descValue, 22, 5)) {
+        ss << "," << ssIdx;
+      }
+      ss << "]";
+    }
+    if (sfid == SFID::TGM) {
+      ss << "[C";
+      auto uOff = getSignedBitField(descValue, 38, 4);
+      auto vOff = getSignedBitField(descValue, 34, 4);
+      auto rOff = getSignedBitField(descValue, 30, 4);
+      if (uOff || vOff || rOff) {
+        ss << "+(" << uOff << "," << vOff << "," << rOff << ")";
+      }
+      ss << "]";
+    } else {
+      auto off = getOffset();
+      ss << "[";
+      if (!off->is2d && sfid == SFID::UGM) {
+        auto dtVcSzBytes = getDataSizeBytesMem() * getElemsPerAddr();
+        auto scaleEnc = (int)getBitField(descValue, 44, 2);
+        if (scaleEnc) {
+          ss << dtVcSzBytes * (1 << scaleEnc) << "*";
+        }
+      }
+      ss << "A";
+      if (off->is2d) {
+        ss << "+(" <<
+          fmtSignedHex(off->immOffX) << ","  <<
+          fmtSignedHex(off->immOffY) << ")";
+      } else {
+        ss << fmtSignedHexTerm(off->immOff);
+      }
+      ss << "]";
+    }
+  } else {
+    // flush op or 2d op or something else
+    ss << "...";
+  }
+
+  return ss.str();
+}
+
+
+SendAccess G4_SendgDesc::getAccessType() const {
+  auto op = getOp();
+  if (MsgOpIsLoad(op))
+    return SendAccess::READ_ONLY;
+  else if (MsgOpIsStore(op))
+    return SendAccess::WRITE_ONLY;
+  else if (MsgOpIsAtomic(op))
+    return SendAccess::READ_WRITE;
+  return SendAccess::READ_ONLY;
+}
+
+AddrSizeType G4_SendgDesc::getAddrSizeType() const {
+  auto op = getOp();
+  if (MsgOpIsExtendedCacheCtrl(op))
+    return AddrSizeType::GLB_A64_A64;
+  if (MsgOpHasChMask(op) && getSFID() == SFID::TGM) {
+    uint32_t addrSizeEnc = (descValue >> 14) & 0x1;
+    vISA_ASSERT(addrSizeEnc == 0, "Must be 32-bit for TGM messages");
+    return AddrSizeType::GLB_STATE_A32;
+  }
+  if (!MsgOpIsLoadStoreAtomic(op))
+    return AddrSizeType::INVALID;
+  uint32_t addrSizeTypeEnc = (descValue >> 14) & 0x3;
+  switch (addrSizeTypeEnc) {
+  case 0:
+    return AddrSizeType::GLB_A64_A32U;
+  case 1:
+    return AddrSizeType::GLB_A64_A32S;
+  case 2:
+    return AddrSizeType::GLB_A64_A64;
+  case 3:
+    return AddrSizeType::GLB_STATE_A32;
+  default:
+    vISA_ASSERT_UNREACHABLE("invalid global addr size type");
+  }
+  return AddrSizeType::INVALID;
+}
+
+uint32_t G4_SendgDesc::getAddrSizeBytes() const {
+  AddrSizeType as = getAddrSizeType();
+  switch (as) {
+  case AddrSizeType::GLB_A64_A32S:
+  case AddrSizeType::GLB_A64_A32U:
+  case AddrSizeType::GLB_STATE_A32:
+    return 4;
+  case AddrSizeType::GLB_A64_A64:
+    return 8;
+  default:
+    vISA_ASSERT_UNREACHABLE("invalid addr size type");
+  }
+  return 0;
+}
+
+DataSize G4_SendgDesc::getDataSize() const {
+  auto op = getOp();
+  if (!MsgOpIsLoadStoreAtomic(op))
+    return DataSize::INVALID;
+  if (MsgOpHasChMask(op) && getSFID() == SFID::TGM) {
+    uint32_t dataSizeEnc = (descValue >> 15) & 0x1;
+    vISA_ASSERT(dataSizeEnc == 0, "Must be 32-bit for TGM messages");
+    return DataSize::D32;
+  }
+  uint32_t dataSizeEnc = (descValue >> 11) & 0x7;
+  switch (dataSizeEnc) {
+  case 0:
+    return DataSize::D8;
+  case 1:
+    return DataSize::D16;
+  case 2:
+    return DataSize::D32;
+  case 3:
+    return DataSize::D64;
+  case 4:
+    return DataSize::D8U32;
+  case 5:
+    return DataSize::D16U32;
+  default:
+    return DataSize::INVALID;
+  }
+}
+
+uint32_t G4_SendgDesc::getDataSizeBytesReg() const {
+  return GetDataSizeBytesReg(getDataSize());
+}
+uint32_t G4_SendgDesc::getDataSizeBytesMem() const {
+  return GetDataSizeBytesMem(getDataSize());
+}
+
+VecElems G4_SendgDesc::getVecElems() const {
+  auto op = getOp();
+  if (!MsgOpIsLoadStoreAtomic(op))
+    return VecElems::INVALID;
+  uint32_t vecElemsEnc = (descValue >> 7) & 0x7;
+  switch (vecElemsEnc) {
+  case 0:
+    return VecElems::V1;
+  case 1:
+    return VecElems::V2;
+  case 2:
+    return VecElems::V3;
+  case 3:
+    return VecElems::V4;
+  case 4:
+    return VecElems::V8;
+  case 5:
+    return VecElems::V16;
+  case 6:
+    return VecElems::V32;
+  case 7:
+    return VecElems::V64;
+  default:
+    return VecElems::INVALID;
+  }
+}
+
+uint32_t G4_SendgDesc::getVecElemsCount() const {
+  const VecElems ve = getVecElems();
+  int numVes = GetNumVecElems(ve);
+  vISA_ASSERT(numVes != 0, "invalid vec elems encoding");
+  return numVes;
+}
+
+DataChMask G4_SendgDesc::getDataChMask() const {
+  vISA_ASSERT(MsgOpHasChMask(getOp()), "message lacks ChMask");
+  return DataChMask((descValue >> 7) & 0xF);
+}
+
+uint32_t G4_SendgDesc::getDataChMaskCount() const {
+  const int chMask = int(getDataChMask());
+  uint32_t channels = 0;
+  for (int i = 0; i < 4; i++)
+    if (int(chMask) & (1 << i))
+      channels++;
+  return channels;
+}
+
+DataOrder G4_SendgDesc::getDataOrder() const {
+  if (!isLSC() || MsgOpHasChMask(getOp())) {
+    return DataOrder::INVALID;
+  }
+  if (isOp(MsgOp::LOAD_BLOCK2D, MsgOp::STORE_BLOCK2D)) {
+    switch (getBitField(descValue, 9, 2)) {
+    case 0: return DataOrder::NONTRANSPOSE;
+    case 1: return DataOrder::VNNI;
+    case 2: return DataOrder::TRANSPOSE;
+    case 3: return DataOrder::TRANSPOSE_VNNI;
+    }
+  }
+  return (descValue & (1 << 10)) ?
+    DataOrder::TRANSPOSE : DataOrder::NONTRANSPOSE;
+}
+
+bool G4_SendgDesc::isDataOrderNonTranspose() const {
+  // The following operations support transpose
+  // load, store, load block 2d
+  // for these operations, extract the transpose field (bit 10) and check
+  if (isOp(MsgOp::LOAD, MsgOp::STORE, MsgOp::LOAD_BLOCK2D)) {
+    return ((descValue >> 10) & 0x1) == 0;
+  }
+  // for all other operations, data order is non transpose
+  return true;
+}
+
+size_t
+G4_SendgDesc::getDataSizeInBytesLdStInst(Gen4_Operand_Number opnd_num) const {
+  vISA_ASSERT(opnd_num == Opnd_src1 || opnd_num == Opnd_dst,
+              "expect dst or src1");
+  uint32_t dataBytes = opnd_num == Opnd_dst ? (dstLen * irb.getGRFSize())
+                                            : (src1Len * irb.getGRFSize());
+  if (isDataOrderNonTranspose()) {
+    // Non-thranspose:
+    // If vecSize > 1, make data size is GRF-algined for simplicity.
+    // Otherwise, data size is the exact bytes accessed by HW.
+    if (getElemsPerAddr() <= 1)
+      dataBytes = execSize * getDataSizeBytesReg();
+  } else {
+      // Transpose
+      dataBytes = getElemsPerAddr() * getDataSizeBytesReg();
+  }
+  return dataBytes;
+}
+
+size_t G4_SendgDesc::getSrc0LenBytes() const {
+  return src0Len * irb.getGRFSize();
+}
+
+size_t G4_SendgDesc::getSrc1LenBytes() const {
+  // Default value is src1Len * irb.getGRFSize().
+  // We can calculate the exact size in bytes accessed by HW on demand.
+  uint32_t src1LenBytes = src1Len * irb.getGRFSize();
+  auto op = getOp();
+  switch (op) {
+  case MsgOp::STORE:
+    src1LenBytes = getDataSizeInBytesLdStInst(Opnd_src1);
+    break;
+  // TODO: handle other LSC op codes
+  default:
+    // Use the defalt value
+    break;
+  }
+  return src1LenBytes;
+}
+
+size_t G4_SendgDesc::getDstLenBytes() const {
+  // Default value is dstLen * irb.getGRFSize().
+  // We can calculate the exact size in bytes accessed by HW on demand.
+  uint32_t dstLenBytest = dstLen * irb.getGRFSize();
+  auto op = getOp();
+  switch (op) {
+  case MsgOp::LOAD:
+    if (dstLen != 0)
+      dstLenBytest = getDataSizeInBytesLdStInst(Opnd_dst);
+    break;
+  // TODO: handle other LSC op codes
+  default:
+    // Use the defalt value
+    break;
+  }
+  return dstLenBytest;
+}
+
+bool G4_SendgDesc::encodeAddrScale(int scale) {
+  // compute address scaling encoding
+  auto op = getOp();
+  bool isLdStAt = MsgOpIsLoadStoreAtomic(op) && !MsgOpIs2D(op);
+  vISA_ASSERT(isLdStAt, "unsupported descriptor");
+  if (!isLdStAt)
+    return false;
+
+  // note that the address scale provided as input (immScale in LSC_ADDR_INFO
+  // struct) is in bytes: the product of data size and vec elems per address.
+  // This is different from the scale factor encoded in the descriptor;
+  // the scale factor encoded in the descriptor is an encoding that conveys how
+  // this address scale should be computed in the first place and is in terms
+  // of data type size
+  // Example 1: if we have a load operation of the form
+  // load.ugm ...d32 flat[0x10*V20]:a64. The data size is 4 bytes (d32),
+  // # vector elements is 1 and the addrScale is 16 (0x10). This load is
+  // accessing a 4B element every 16th byte (stride of 4B). The corresponding
+  // scale encoded in the descriptor is 4, which will give us back the address
+  // scale of encoded scale * data size * # of vector elements
+  // = (4) * 4 * 1 = 16.
+  // Example 2: load.ugm ...d32x4 flat[0x10*V20]:a64. The data size is
+  // 4 bytes, # of vector elements is 4, and the addrScale is 16 (0x10). This
+  // load is accessing 4 4B elements. Here stride is 1B as 16B are loaded
+  // and the next 16B are loaded and so on. The corresponding scale encoded
+  // in the descriptor is 1, which will give us back the address scale of
+  // encoded scale * data size * # of vector elements = (1) * 4 * 4 = 16.
+
+  uint32_t numVecElems = (MsgOpHasChMask(op)) ? 4 : getElemsPerAddr();
+  vISA_ASSERT(numVecElems != 0, "invalid vector/chmask size");
+
+  // cannot encode scale if vec elements is 3
+  if (numVecElems == 3)
+    return false;
+  auto dataSizeBytes = getDataSizeBytesMem();
+  if (dataSizeBytes == 1) { // don't try and scale byte elements
+    return false;
+  }
+  // if numVecElems * dataSizeBytes * N > 32, then HW behavior is undefined
+  // don't encode the scale; vISA will generate emulation sequence
+  if (numVecElems * dataSizeBytes > 32) {
+    return false;
+  }
+  if (numVecElems * dataSizeBytes * 1 == scale) {
+    descValue |= ((uint64_t)1 << 44);
+    return true;
+  }
+  return false;
+}
+
+bool G4_SendgDesc::encodeAddrGlobalOffset(int immOffset) {
+  auto op = getOp();
+  bool isLdStAt = MsgOpIsLoadStoreAtomic(op) && !MsgOpIs2D(op);
+  vISA_ASSERT(isLdStAt, "unsupported descriptor");
+  if (!isLdStAt)
+    return false;
+  bool isStateful = getAddrSizeType() == AddrSizeType::GLB_STATE_A32;
+
+  // note: this immOffset provided from vISA is in terms of bytes
+  // on the other hand, the descriptor encodes the offset in terms of number
+  // of elements
+
+  const int dsize = getDataSizeBytesMem();
+  if (dsize == 0)  {
+    return false;
+  }
+
+  // checks if input fits in allocated bits
+  // assumes signed
+  auto fitsIn = [&](int what, int bits) {
+    return what >= -(1 << (bits - 1)) && what <= (1 << (bits - 1)) - 1;
+  };
+
+  // check if immOffset is a multiple of data size
+  if (immOffset % dsize != 0)
+    return false;
+
+  const int immOffElems = immOffset / dsize;
+
+  // check if immOffset in # of elements can be encoded in provided bit range
+  // this depends on address mode type
+  if (isStateful) {
+    // global offset is encoded in [43:27] for stateful
+    if (!fitsIn(immOffElems, 17))
+      return false;
+    descValue = putBitField(descValue, 27, 43 - 27 + 1, (uint64_t)immOffElems);
+  } else {
+    // global offset is encoded in [43:22] for stateless
+    if (!fitsIn(immOffElems, 22))
+      return false;
+    descValue = putBitField(descValue, 22, 43 - 22 + 1, (uint64_t)immOffElems);
+  }
+  return true;
+}
+
 
 ///////////////////////////////////////////////////////////////////////////////
 // G4_SendDescRaw implementations
@@ -873,11 +1442,12 @@ static bool isHdcFloatAtomicMessage(SFID funcID, uint16_t msgType,
 }
 
 bool G4_SendDescRaw::isAtomicMessage() const {
-  if (isLscOp() && (desc.value & 0x3F) >= LSC_ATOMIC_IINC &&
-      (desc.value & 0x3F) <= LSC_ATOMIC_XOR) {
+  auto dscOpVal = desc.value & 0x3F;
+  if (isLscOp() && dscOpVal >= LSC_ATOMIC_IINC && dscOpVal <= LSC_ATOMIC_XOR) {
     return true;
   }
 
+
   auto funcID = getSFID();
   if (!isHDC())
     return false; // guard getMessageType() on SFID without a message type
@@ -1756,6 +2326,7 @@ bool G4_SendDescRaw::isBTS() const {
     case LSC_ADDR_TYPE_BSS:
     case LSC_ADDR_TYPE_SS:
     case LSC_ADDR_TYPE_BTI:
+    case LSC_ADDR_TYPE_SURF:
       return true;
     default:
       break;
