@@ -219,6 +219,12 @@ bool LVN::canReplaceUses(INST_LIST_ITER inst_it, UseList &uses,
       }
     }
 
+    if (useInst->getNumDst() && useInst->getDst() && !useInst->hasNULLDst() &&
+        useInst->getDst()->getTopDcl() == lvnDstTopDcl) {
+      canReplace = false;
+      break;
+    }
+
     // Compute a single positive stride if exists.
     unsigned int use_hs = 0;
     {
@@ -675,7 +681,6 @@ void LVN::removePhysicalVarRedefs(G4_DstRegRegion *dst) {
 }
 
 void LVN::removeFlagVarRedefs(G4_Declare *topdcl) {
-  auto it = dclValueTable.find(topdcl);
 
   for (auto &all : lvnTable) {
     for (auto it = all.second.begin(); it != all.second.end();) {
@@ -1088,7 +1093,7 @@ bool isNonUniformSrcRegion(G4_SrcRegRegion *srcRgn) {
 bool LVN::addValue(G4_INST *inst) {
   auto isAllowedOpcode = [](G4_INST *inst) {
     if (inst->opcode() == G4_mov || inst->opcode() == G4_shl ||
-        inst->opcode() == G4_shr)
+        inst->opcode() == G4_and || inst->opcode() == G4_shr)
       return true;
     return false;
   };
@@ -1414,7 +1419,6 @@ void LVN::addValueToTable(G4_INST *inst, Value &oldValue) {
   };
 
   for (auto &item : perInstValueCache) {
-    auto it = dclValueTable.find(item.first);
     item.second->active = true;
     dclValueTable[item.first].push_back(item.second);
   }
