@@ -44,6 +44,14 @@ void changeToIndirectSend(G4_INST *inst, G4_Declare *s0Var,
                           IR_Builder &builder, bool isLargeGRF) {
   // Change the send instruction to sendi
   G4_InstSend *Send = inst->asSendInst();
+  if (Send->opcode() == G4_sendg || Send->opcode() == G4_sendgc) {
+    G4_SendgDesc *desc = (G4_SendgDesc *)inst->getMsgDesc();
+    desc->setSrc0Len(totalRegs);
+    desc->setSrc1Len(0);
+    if (isLargeGRF) {
+      Send->setSendgx();
+    }
+  } else {
     G4_SendDescRaw *desc = Send->getMsgDescRaw();
     desc->setExtMessageLength(0);
     G4_Operand *msgDesc = inst->getSrc(2);
@@ -53,6 +61,7 @@ void changeToIndirectSend(G4_INST *inst, G4_Declare *s0Var,
     descImm |= totalRegs << 25;
     G4_Imm *msgDescImm = builder.createImm(descImm, Type_UD);
     inst->setSrc(msgDescImm, 2);
+  }
 
   // Replace source 0 with scalar register
   G4_SrcRegRegion *headerOpnd =
@@ -65,6 +74,18 @@ void changeToIndirectSend(G4_INST *inst, G4_Declare *s0Var,
   inst->setSrc(payloadToUse, 1);
 }
 
+bool SRSubPassAfterRA::isLargeGRFOpnd(G4_Operand *opnd) {
+  if (!opnd || opnd->isNullReg() || !opnd->isGreg()) {
+    return false;
+  }
+  return opnd->getLinearizedEnd() > (256 * builder.numEltPerGRF<Type_UB>() - 1);
+}
+
+bool SRSubPassAfterRA::canRegisterBeEncoded(G4_Operand *opnd) {
+  // In some platforms, mov is not removable and dst register > r255, cannot be
+  // gather instruction
+  return !isLargeGRFOpnd(opnd);
+}
 
 bool SRSubPassAfterRA::isRemoveAble(G4_INST *i) {
   if (i->opcode() != G4_mov) {
@@ -158,6 +179,11 @@ bool SRSubPassAfterRA::isRemoveAble(G4_INST *i) {
     }
   }
 
+  // In some platforms, mov is not removeable if the src register > r255
+  if (!canRegisterBeEncoded(src)) {
+    return false;
+  }
+
   return true;
 }
 
@@ -240,6 +266,10 @@ G4_INST *SRSubPassAfterRA::getRemoveableImm(G4_INST *inst,
 }
 
 // if multiple defined, cannot be removed
+// The define instruction is from the DU chain of the send source.
+// The opndNum is the source # in send.
+// immMovs recorded the mov immediate instructions which are define of the send
+// source.
 bool SRSubPassAfterRA::isDefinedMultipleTimes(
     G4_INST *defInst, Gen4_Operand_Number opndNum, regCandidatesBRA &dstSrcRegs,
     std::vector<G4_INST *> &immMovs,
@@ -250,6 +280,7 @@ bool SRSubPassAfterRA::isDefinedMultipleTimes(
                       builder.getGRFSize() - 1) /
                      builder.getGRFSize();
 
+  // Check if the register is defined in definedGRF, if not set it as defined.
   bool definedAlready = false;
   for (unsigned i = 0; i < dstSize; i++) {
     unsigned regNum = i + dst->getLinearizedStart() / builder.getGRFSize();
@@ -260,19 +291,47 @@ bool SRSubPassAfterRA::isDefinedMultipleTimes(
     }
   }
 
+  if (!definedAlready)
+    return false;
+
   // The startOffset is the offset to the declare
   unsigned startOffset = dst->getLeftBound() / builder.getGRFSize();
-  if (definedAlready) { // Defined multiple times
-    // if multiple defined, cannot be removed
-    auto iter =
-        std::find_if(dstSrcRegs.dstSrcMap.begin(), dstSrcRegs.dstSrcMap.end(),
-                     [opndNum, dst](regMapBRA regmap) {
-                       return regmap.opndNum == opndNum &&
-                              !((regmap.inst->getDst()->getLinearizedStart() >
-                                 dst->getLinearizedEnd()) ||
-                                (dst->getLinearizedStart() >
-                                 regmap.inst->getDst()->getLinearizedEnd()));
-                     });
+  // if multiple defined, cannot be removed
+  auto iter =
+      std::find_if(dstSrcRegs.dstSrcMap.begin(), dstSrcRegs.dstSrcMap.end(),
+                   [opndNum, dst](regMapBRA regmap) {
+                     return regmap.opndNum == opndNum &&
+                            !((regmap.inst->getDst()->getLinearizedStart() >
+                               dst->getLinearizedEnd()) ||
+                              (dst->getLinearizedStart() >
+                               regmap.inst->getDst()->getLinearizedEnd()));
+                   });
+  if (iter != dstSrcRegs.dstSrcMap.end()) {
+    // Remove the candidate
+    for (unsigned offset = startOffset; offset < startOffset + dstSize;
+         offset++) {
+      notRemoveableMap.push_back(std::make_pair(opndNum, offset));
+    }
+  }
+
+  // check if mov imm is multiple defined
+  auto immIter =
+      std::find_if(immMovs.begin(), immMovs.end(), [dst](G4_INST *immMov) {
+        return !(
+            (immMov->getDst()->getLinearizedStart() >
+             dst->getLinearizedEnd()) ||
+            (dst->getLinearizedStart() > immMov->getDst()->getLinearizedEnd()));
+      });
+
+  // If reused mov is multiple defined, the removed instructions which used
+  // the dst of the reused mov cannot be removed.
+  if (immIter != immMovs.end()) {
+    G4_INST *immMov = (*immIter);
+    G4_Operand *immDst = immMov->getDst();
+
+    auto iter = std::find_if(
+        dstSrcRegs.dstSrcMap.begin(), dstSrcRegs.dstSrcMap.end(),
+        [immDst](regMapBRA regmap) { return regmap.opnd == immDst; });
     if (iter != dstSrcRegs.dstSrcMap.end()) {
       for (unsigned offset = startOffset; offset < startOffset + dstSize;
            offset++) {
@@ -280,37 +339,10 @@ bool SRSubPassAfterRA::isDefinedMultipleTimes(
       }
     }
 
-    // check if mov imm is multiple defined
-    auto immIter =
-        std::find_if(immMovs.begin(), immMovs.end(), [dst](G4_INST *immMov) {
-          return !((immMov->getDst()->getLinearizedStart() >
-                    dst->getLinearizedEnd()) ||
-                   (dst->getLinearizedStart() >
-                    immMov->getDst()->getLinearizedEnd()));
-        });
-
-    // If reused mov is multiple defined, the removed instructions which used
-    // the dst of the reused mov cannot be removed.
-    if (immIter != immMovs.end()) {
-      G4_INST *immMov = (*immIter);
-      G4_Operand *immDst = immMov->getDst();
-
-      auto iter = std::find_if(
-          dstSrcRegs.dstSrcMap.begin(), dstSrcRegs.dstSrcMap.end(),
-          [immDst](regMapBRA regmap) { return regmap.opnd == immDst; });
-      if (iter != dstSrcRegs.dstSrcMap.end()) {
-        for (unsigned offset = startOffset; offset < startOffset + dstSize;
-             offset++) {
-          notRemoveableMap.push_back(std::make_pair(opndNum, offset));
-        }
-      }
-
-      immMovs.erase(immIter);
-    }
-    return true;
+    immMovs.erase(immIter);
   }
 
-  return false;
+  return true;
 }
 
 // Check if current instruction is the candidate of sendi.
@@ -321,7 +353,7 @@ bool SRSubPassAfterRA::isSRCandidateAfterRA(G4_INST *inst,
     return false;
   }
 
-  if (!inst->asSendInst()->getMsgDescRaw()) {
+  if (!inst->asSendInst()->getMsgDescRaw() && !inst->isSendg()) {
     return false;
   }
 
@@ -363,6 +395,12 @@ bool SRSubPassAfterRA::isSRCandidateAfterRA(G4_INST *inst,
     return false;
   }
 
+  if (inst->getDst() && !inst->getDst()->isNullReg()) {
+    // In some platforms, gather send doesn't support register > r255
+    if (!canRegisterBeEncoded(inst->getDst())) {
+      return false;
+    }
+  }
 
   int movInstNum = 0;
   int32_t firstDefID = 0x7FFFFFFF; // the ID of the first instruction define the
@@ -386,6 +424,9 @@ bool SRSubPassAfterRA::isSRCandidateAfterRA(G4_INST *inst,
     // Multiple defines are not allowed
     if (isDefinedMultipleTimes(defInst, opndNum, dstSrcRegs, immMovs,
                                notRemoveableMap, definedGRF)) {
+      if (!canRegisterBeEncoded(dst)) {
+        return false;
+      }
       continue;
     }
 
@@ -402,6 +443,9 @@ bool SRSubPassAfterRA::isSRCandidateAfterRA(G4_INST *inst,
         firstDefID = std::min(firstDefID, def.first->getLocalId());
         movInstNum++;
     } else {
+      if (!canRegisterBeEncoded(dst)) {
+        return false;
+      }
       if (defInst->opcode() == G4_mov && defInst->getSrc(0) &&
           defInst->getSrc(0)->isImm()) {
         // Check if there is mov instruction with same imm value
@@ -419,7 +463,7 @@ bool SRSubPassAfterRA::isSRCandidateAfterRA(G4_INST *inst,
           continue;
         }
       }
-      for (unsigned offset = startOffset; offset < startOffset + dstSize;
+      for (unsigned offset = startOffset; offset < (startOffset + dstSize);
            offset++) {
         notRemoveableMap.push_back(std::make_pair(opndNum, offset));
       }
@@ -453,6 +497,58 @@ bool SRSubPassAfterRA::isSRCandidateAfterRA(G4_INST *inst,
   if (movInstNum < 2) {
     return false;
   }
+  // Even keep original GRF of src0 or src1, the original one cannot be more than 255
+  unsigned short GRFSize = builder.getGRFSize();
+  int src0Size = inst->getMsgDesc()->getSrc0LenRegs();
+  G4_Operand *src0 = inst->getSrc(0);
+  int j = 0;
+  for (int i = 0; i < src0Size; i++) {
+    bool replaced = false;
+    int srcOffset = src0->getLeftBound() / GRFSize + i;
+    if (j < (int)dstSrcRegs.dstSrcMap.size() &&
+        dstSrcRegs.dstSrcMap[j].opndNum == Opnd_src0) {
+      int opndSize =
+          (dstSrcRegs.dstSrcMap[j].opnd->getLinearizedEnd() -
+           dstSrcRegs.dstSrcMap[j].opnd->getLinearizedStart() + GRFSize - 1) /
+          GRFSize;
+      int opndOffset = dstSrcRegs.dstSrcMap[j].offset;
+
+      if ((srcOffset >= opndOffset) && (srcOffset < opndOffset + opndSize)) {
+        replaced = true;
+        i += opndSize - 1;
+        j++;
+      }
+    }
+    if (!replaced && (src0->getLinearizedStart() / GRFSize + i) > 255) {
+      return false;
+    }
+  }
+
+  G4_Operand *src1 = inst->getSrc(0);
+  if (src1 && !src1->isNullReg()) {
+    int src1Size = inst->getMsgDesc()->getSrc1LenRegs();
+    for (int i = 0; i < src1Size; i++) {
+      bool replaced = false;
+      int src1ffset = src1->getLeftBound() / GRFSize + i;
+      if (j < (int)dstSrcRegs.dstSrcMap.size() &&
+          dstSrcRegs.dstSrcMap[j].opndNum == Opnd_src1) {
+        int opndSize =
+            (dstSrcRegs.dstSrcMap[j].opnd->getLinearizedEnd() -
+             dstSrcRegs.dstSrcMap[j].opnd->getLinearizedStart() + GRFSize - 1) /
+            GRFSize;
+        int opndOffset = dstSrcRegs.dstSrcMap[j].offset;
+
+        if ((src1ffset >= opndOffset) && (src1ffset < opndOffset + opndSize)) {
+          replaced = true;
+          i += opndSize - 1;
+          j++;
+        }
+      }
+      if (!replaced && (src1->getLinearizedStart() / GRFSize + i) > 255) {
+        return false;
+      }
+    }
+  }
 
   dstSrcRegs.firstDefID = firstDefID;
   // Sort according to the register order in the original payload
@@ -462,50 +558,135 @@ bool SRSubPassAfterRA::isSRCandidateAfterRA(G4_INST *inst,
   return true;
 }
 
+bool SRSubPassAfterRA::checkCandidateForLargeGRF(G4_INST *inst,
+                                                 regCandidatesBRA &dstSrcRegs) {
+  // Check the source only, because large GRF is dst doesn't affect index
+  // register encoding.
+  // At the same time, much check each register of indexed regsiters. Because
+  // the original register may be replace, may not be replaced. So cannot use
+  // operand based check
 
-unsigned short SRSubPassAfterRA::allocateS0(unsigned short UQNum) {
-  unsigned short freeSRSub = S0Index;
+  unsigned short GRFSize = builder.getGRFSize();
+  int j = 0;
+  // Check the source of define mov if the mov can be removed, otherwise, check
+  // original source 0 directly
+  int src0Size = inst->getMsgDesc()->getSrc0LenRegs();
+  G4_Operand *src0 = inst->getSrc(0);
+  for (int i = 0; i < src0Size; i++) {
+    if (j < (int)dstSrcRegs.dstSrcMap.size() &&
+        dstSrcRegs.dstSrcMap[j].opndNum == Opnd_src0) {
+      int opndSize =
+          (dstSrcRegs.dstSrcMap[j].opnd->getLinearizedEnd() -
+           dstSrcRegs.dstSrcMap[j].opnd->getLinearizedStart() + GRFSize - 1) /
+          GRFSize;
+      int srcOffset = src0->getLeftBound() / GRFSize + i;
+      int opndOffset = dstSrcRegs.dstSrcMap[j].offset;
+
+      if ((srcOffset >= opndOffset) && (srcOffset < opndOffset + opndSize)) {
+        for (int k = 0; k < opndSize; k++) {
+          if ((dstSrcRegs.dstSrcMap[j].opnd->getLinearizedStart() +
+               k * GRFSize) > (256 * builder.numEltPerGRF<Type_UB>() - 1)) {
+            dstSrcRegs.isLargeGRF = true;
+          }
+        }
+        i += opndSize - 1;
+        j++;
+        continue;
+      }
+    }
+
+    // Not replaced
+    if ((src0->getLinearizedStart() + i * GRFSize) >
+        (256 * builder.numEltPerGRF<Type_UB>() - 1)) {
+      dstSrcRegs.isLargeGRF = true;
+    }
+  }
+
+  // Check the ource of define mov if the mov can be removed, otherwise, check
+  // original source 1 register
+  G4_Operand *src1 = inst->getSrc(1);
+  int src1Size = inst->getMsgDesc()->getSrc1LenRegs();
+  if (src1 && !src1->isNullReg()) {
+    for (int i = 0; i < src1Size; i++) {
+      if (j < (int)dstSrcRegs.dstSrcMap.size() &&
+          dstSrcRegs.dstSrcMap[j].opndNum == Opnd_src1) {
+        int opndSize =
+            (dstSrcRegs.dstSrcMap[j].opnd->getLinearizedEnd() -
+             dstSrcRegs.dstSrcMap[j].opnd->getLinearizedStart() + GRFSize - 1) /
+            GRFSize;
+        int srcOffset = src1->getLeftBound() / GRFSize + i;
+        int opndOffset = dstSrcRegs.dstSrcMap[j].offset;
+
+        if ((srcOffset >= opndOffset) && (srcOffset < opndOffset + opndSize)) {
+          for (int k = 0; k < opndSize; k++) {
+            if ((dstSrcRegs.dstSrcMap[j].opnd->getLinearizedStart() +
+                 k * GRFSize) > (256 * builder.numEltPerGRF<Type_UB>() - 1)) {
+              dstSrcRegs.isLargeGRF = true;
+            }
+          }
+          i += opndSize - 1;
+          j++;
+          continue;
+        }
+      }
+
+      // Add source 1 register directly
+      if ((src1->getLinearizedStart() + i * GRFSize) >
+          (256 * builder.numEltPerGRF<Type_UB>() - 1)) {
+        dstSrcRegs.isLargeGRF = true;
+      }
+    }
+  }
+
+  return !dstSrcRegs.isLargeGRF;
+}
+
+unsigned short SRSubPassAfterRA::allocateS0RoundRobin(unsigned short UQNum) {
+  unsigned short freeSRSub = FirstFreeS0SubRegIndex;
   bool find = false;
-  for (; freeSRSub < S0SubRegNum;
+
+  // Scan from previous index
+  for (; freeSRSub < MaxS0SubRegNum;
        freeSRSub++) {
-    if (UQNum > 1 && (freeSRSub + 1) < S0SubRegNum) {
+    if (UQNum > 1 && (freeSRSub + 1) < MaxS0SubRegNum) {
       if (!UsedS0SubReg.isSet(freeSRSub) &&
           !UsedS0SubReg.isSet(freeSRSub + 1)) {
         find = true;
-        S0Index = freeSRSub + 2;
+        FirstFreeS0SubRegIndex = freeSRSub + 2;
         break;
       }
     } else if ((UQNum == 1) && !UsedS0SubReg.isSet(freeSRSub)) {
       find = true;
-      S0Index = freeSRSub + 1;
+      FirstFreeS0SubRegIndex = freeSRSub + 1;
       break;
     }
   }
 
   if (find) {
-    if (S0Index >= S0SubRegNum) {
-      S0Index = 0;
+    if (FirstFreeS0SubRegIndex >= MaxS0SubRegNum) {
+      FirstFreeS0SubRegIndex = 0;
     }
     return freeSRSub;
   }
 
-  for (freeSRSub = 0; freeSRSub < S0Index; freeSRSub++) {
-    if (UQNum > 1 && (freeSRSub + 1) < S0SubRegNum) {
+  // Not find, scan from the 0 to previous index
+  for (freeSRSub = 0; freeSRSub < FirstFreeS0SubRegIndex; freeSRSub++) {
+    if (UQNum > 1 && (freeSRSub + 1) < MaxS0SubRegNum) {
       if (!UsedS0SubReg.isSet(freeSRSub) &&
           !UsedS0SubReg.isSet(freeSRSub + 1)) {
         find = true;
-        S0Index = freeSRSub + 2;
+        FirstFreeS0SubRegIndex = freeSRSub + 2;
         break;
       }
     } else if ((UQNum == 1) && !UsedS0SubReg.isSet(freeSRSub)) {
       find = true;
-      S0Index = freeSRSub + 1;
+      FirstFreeS0SubRegIndex = freeSRSub + 1;
       break;
     }
   }
 
-  if (S0Index >= S0SubRegNum) {
-    S0Index = 0;
+  if (FirstFreeS0SubRegIndex >= MaxS0SubRegNum) {
+    FirstFreeS0SubRegIndex = 0;
   }
   // At most 2 Qwords required, since we reserved two QWords, should always find
   assert(find);
@@ -623,7 +804,10 @@ bool SRSubPassAfterRA::replaceWithSendiAfterRA(G4_BB *bb,
   if (dstSrcRegs.isLargeGRF) {
     UQNum = totalRegs > (TypeSize(Type_UQ) / TypeSize(Type_UW)) ? 2 : 1;
   }
-  unsigned short S0Sub = allocateS0(UQNum);
+  unsigned short S0Sub = 0;
+  if (builder.doRoundRobinScalarAlloc()) {
+    S0Sub = allocateS0RoundRobin(UQNum);
+  }
   G4_Declare *s0Var = builder.createTempScalar(UQNum, "S0_");
   s0Var->getRegVar()->setPhyReg(builder.phyregpool.getScalarReg(), 0);
   G4_DstRegRegion *dst =
@@ -685,15 +869,34 @@ void SRSubPassAfterRA::SRSubAfterRA(G4_BB *bb) {
   std::map<G4_INST *, regCandidatesBRA> candidates;
   std::map<G4_INST *, regCandidatesBRA>::iterator candidatesIt;
 
-  S0SubRegNum = builder.getScalarRegisterSizeInBytes() / 8;
-  UsedS0SubReg.resize(S0SubRegNum);
-  S0Index = 0;
+  UsedS0SubReg.resize(MaxS0SubRegNum);
+  UsedS0SubReg.clear();
+  FirstFreeS0SubRegIndex = 0;
 
   INST_LIST_ITER ii = bb->begin(), iend(bb->end());
   unsigned candidateStart = builder.getuint32Option(vISA_IndirectInstStart);
   unsigned candidateEnd = builder.getuint32Option(vISA_IndirectInstEnd);
   while (ii != iend) {
     G4_INST *inst = *ii;
+    if (inst->isSendg()) {
+      auto sendgInst = inst->asSendInst();
+      auto ind0 = sendgInst->getIND0();
+      if (ind0 && ind0->isS0()) {
+        unsigned byteOff = ind0->getLeftBound();
+        // For the byteOff / 8 >= MaxS0SubRegNum, it may be upper bound reserved
+        // sub registers.
+        if (byteOff / 8 < MaxS0SubRegNum)
+          UsedS0SubReg.set(byteOff / 8, true);
+      }
+      auto ind1 = sendgInst->getIND1();
+      if (ind1 && ind1->isS0()) {
+        unsigned byteOff = ind1->getLeftBound();
+        // For the byteOff / 8 >= MaxS0SubRegNum, it may be upper bound reserved
+        // sub registers.
+        if (byteOff / 8 < MaxS0SubRegNum)
+          UsedS0SubReg.set(byteOff / 8, true);
+      }
+    }
     regCandidatesBRA dstSrcRegs;
     if (!isSRCandidateAfterRA(inst, dstSrcRegs)) {
       ii++;
@@ -736,6 +939,7 @@ void SRSubPassAfterRA::SRSubAfterRA(G4_BB *bb) {
       INST_LIST_RITER scan_ri = ri;
       scan_ri++;
       G4_INST *rInst = *scan_ri;
+      bool unsupportGRF = false;
 
       while (rInst->getLocalId() > candidates[inst].firstDefID) {
         if (rInst->isDead()) {
@@ -797,6 +1001,13 @@ void SRSubPassAfterRA::SRSubAfterRA(G4_BB *bb) {
               int srcRegLB = (*dstSrcRegsIter).opnd->getLinearizedStart();
               int srcRegRB = (*dstSrcRegsIter).opnd->getLinearizedEnd();
               if (!(srcRegRB < dstRegLB || srcRegLB > dstRegRB)) {
+                // mov is not removeable and dst register > r255, cannot be
+                // gather instruction.
+                G4_DstRegRegion *removedDst = (*dstSrcRegsIter).inst->getDst();
+                if (!canRegisterBeEncoded(removedDst)) {
+                  unsupportGRF = true;
+                  break;
+                }
 
                 // Register is redefined
                 dstSrcRegsIter =
@@ -807,6 +1018,9 @@ void SRSubPassAfterRA::SRSubAfterRA(G4_BB *bb) {
             }
           }
         }
+        if (unsupportGRF) {
+          break;
+        }
         scan_ri++;
         if (scan_ri == rend) {
           break;
@@ -814,6 +1028,10 @@ void SRSubPassAfterRA::SRSubAfterRA(G4_BB *bb) {
         rInst = *scan_ri;
       }
 
+      if (unsupportGRF || !checkCandidateForLargeGRF(inst, candidates[inst])) {
+        candidates.erase(candidatesIt);
+        continue;
+      }
       // Due to extra mov for s0, so don't use s0 if equal or less than 1 mov
       // inst can be removed.
       if (candidates[inst].dstSrcMap.size() <= 1 &&
